package EM_LLR_Utils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;

import edu.tau.compbio.ds.SimilarityMatrix;
import edu.tau.compbio.stat.NormalDistribution;
import edu.tau.compbio.util.ProgressManager;

public class LLRScoresCalculator {

	ArrayList<String> ids;
	float[][] similarities;
	
	int sampleSizeForEM = 1000;
	float[][] similarities_sample_for_EM;
	ArrayList<String> ids_for_EM;

	float[][] weightedMatrix;

	float matesMaxP = 0.15f;
	boolean init = false;
	private NormalDistribution matesDist;
	private NormalDistribution nonMatesDist;
	private double sigmaF;
	private double sigmaT;
	private double sigmaSqT;
	private double sigmaSqF;
	private double meanF;
	private double meanT;
	private double oneMinusP;
	private double pMates;

	public LLRScoresCalculator(ArrayList<String> names, float[][] sims) {
		ids = names;
		similarities = sims;
	}

	public LLRScoresCalculator(ArrayList<String> names, double[][] sims) {
		ids = names;
		similarities = new float[sims.length][sims.length];
		for (int i = 0; i < sims.length; i++) {
			for (int j = 0; j < sims.length; j++) {
				similarities[i][j] = (float) sims[i][j];
			}
		}
	}

	public LLRScoresCalculator(float[][] sims) {
		similarities = sims;
	}

	public float[][] getClickWeightedMatrix(boolean doEM) {

		if (doEM)
			runEM();

		for (int i = 0; i < weightedMatrix.length; i++) {
			for (int j = 0; j < i; j++) {
				double sim = similarities[i][j];
				double score = getSimLikeLihood(sim,false);
				weightedMatrix[i][j] = (new Double(score)).floatValue();
				weightedMatrix[j][i] = weightedMatrix[i][j];
			}
		}

		return weightedMatrix;
	}

	public double getSimLikeLihood(double sim,boolean print) {
		double score = Math.log((pMates * sigmaF) / (oneMinusP * sigmaT));
		if (print){
			System.out.println("factor constant: " + score);
			System.out.println("deviation from mates: " + (Math.pow((sim - meanT), 2) / sigmaSqT));
			System.out.println("deviation from non mates: " + (Math.pow((sim - meanF), 2) / sigmaSqF));
		}
		score += (Math.pow((sim - meanF), 2) / sigmaSqF);
		score -= (Math.pow((sim - meanT), 2) / sigmaSqT);
		return score;
	}

	public double getMeanMatesLikelihood() {
		return getSimLikeLihood(meanT,false);
	}

	public void runEM() {
		System.out.println("Executing EM...");
		
		similarities_sample_for_EM = similarities;
		ids_for_EM = ids;
		
		if (similarities.length > sampleSizeForEM){
			ArrayList<Integer> shuff_indices = new ArrayList<Integer>(similarities.length);
			for (int i = 0; i < similarities.length; i++) {
				shuff_indices.add(i);
			}
			Collections.shuffle(shuff_indices);
			int[] selecedInds = new int[sampleSizeForEM];
			for (int i = 0; i < selecedInds.length; i++) {
				selecedInds[i] = shuff_indices.get(i);
			}
			
			similarities_sample_for_EM = new float[sampleSizeForEM][sampleSizeForEM];
			ids_for_EM = new ArrayList<String>(sampleSizeForEM);
			for (int i = 0; i < selecedInds.length; i++) {
				for (int j = 0; j < selecedInds.length; j++) {
					similarities_sample_for_EM[i][j] =similarities[selecedInds[i]][selecedInds[j]];
				}
				ids_for_EM.add(ids.get(selecedInds[i]));
			}
		}
		
		
		double[] regs = new double[similarities_sample_for_EM.length];
		Arrays.fill(regs, 1f);

		boolean _updateExp = true;
		boolean _updateStd = true;
		boolean _updateP = true;

		float _emMinPMates = 0.01f;
		float _emMaxPMates = matesMaxP;

		// SimpleParamsEM em = new SimpleParamsEM(new
		// SimilarityMatrix<String>(ids, similarities),regs);
		RegParamsEM em = new RegParamsEM(new SimilarityMatrix<String>(ids_for_EM,
				similarities_sample_for_EM), regs);
		em.setUpdateMode(_updateExp, _updateStd, _updateP);
		em.setPMatesRange(_emMinPMates, _emMaxPMates);
		em.setSameStd(false);
		if (init) {
			em.setInit(true);
		}
		ProgressManager progMan = new ProgressManager();
		em.run(progMan);
		matesDist = em.getMatesDistribution();
		nonMatesDist = em.getNonMatesDistribution();
		pMates = em.getMatesP();
		weightedMatrix = new float[ids.size()][ids.size()];

		sigmaF = nonMatesDist.getStd();
		sigmaT = matesDist.getStd();

		sigmaSqT = sigmaT * sigmaT * 2;
		sigmaSqF = sigmaF * sigmaF * 2;
		meanF = nonMatesDist.getExp();
		meanT = matesDist.getExp();

		oneMinusP = 1 - pMates;
	}

	public float[][] getWeightedMatrix() {
		return weightedMatrix;
	}

	public void setMatesMaxP(float matesMaxP) {
		this.matesMaxP = matesMaxP;
		init = true;
	}

	public void setDistributions(NormalDistribution mates,
			NormalDistribution nonMates, double pMates) {
		matesDist = mates;
		nonMatesDist = nonMates;
		this.pMates = pMates;

		weightedMatrix = new float[ids.size()][ids.size()];

		sigmaF = nonMatesDist.getStd();
		sigmaT = matesDist.getStd();

		sigmaSqT = sigmaT * sigmaT * 2;
		sigmaSqF = sigmaF * sigmaF * 2;
		meanF = nonMatesDist.getExp();
		meanT = matesDist.getExp();

		oneMinusP = 1 - pMates;
	}
	
}
