package EM_LLR_Utils;

import edu.tau.compbio.ds.SimilarityMatrix;
import edu.tau.compbio.stat.NormalDistribution;
import edu.tau.compbio.stat.StatDistribution;
import edu.tau.compbio.util.ProgressManager;

public class RegParamsEM extends AbstractParamsEM {

	private double _minInitialPMates = 0.05;
	private double _maxInitialPMates = 0.1;
	private double _minPMates = 0.01;
	private double _maxPMates = 0.1;
	
	public RegParamsEM(SimilarityMatrix<String> simMat,double[] regPs)
	{
		super(simMat,regPs);
		_selfInit = true;
	}
	public void setPMatesRange(float min,float max)
	{
		_minPMates = min;
		_maxPMates = max;
	}
	public void setInitial(NormalDistribution matesDist,NormalDistribution nonDist,double matesP)
	{
		_matesDist = matesDist;
		_nonDist = nonDist;
		_matesP = matesP;
		_selfInit = true;
	}
	public RegParamsEM(SimilarityMatrix<String> simMat,double[] regPs,int randCount)
	{
		super(simMat,regPs,randCount);
		_selfInit = true;
	}
	private void init(ProgressManager progMan)
	{
//		System.out.println(" ***************** In init ***************");
		progMan.setTitle("Seeking start point...");
		double simMean = _simMat.getAverageSimilarity();
		double simStd = _simMat.getSimilarityStd();
		
		int countAbove = 0, countAbovePlus=0, countAboveMinus = 0;
		int count = (_inpSize*(_inpSize-1))/2;
		for (int i=0;i<_inpSize;i++)
		{
			for (int j=i+1;j<_inpSize;j++)
			{
				float sim = this._simMat.getSimilarity(_inds[i], _inds[j]);//__sims[_inds[i]][_inds[j]];
				if (Float.isNaN(sim))
					continue;
				if (sim > simMean)
					countAbove++;
				if (sim >= simMean+simStd)
					countAbovePlus++;
				if (sim >= simMean-simStd)
					countAboveMinus++;
			}
		}
		double perAbove = (double) countAbove / count;
		double perAbovePlus = (double) countAbovePlus / count;
		double perAboveMinus = (double) countAboveMinus / count;
	
		double bestDiff = 1000000, currP, currStd;
		double currExpF, currExpT;
		
		double bestP=0;
		double bestTexp=0;
		double bestFexp=0;
		double bestStd=1;

		bestDiff = Double.MAX_VALUE;
		for(double delta = 0.5; delta <= 2.5; delta += 0.1)
		{
		    for(currP = _minInitialPMates; currP <= _maxInitialPMates; currP += 0.01) 
			{
//		    	System.out.println(" in init loop ");
			    // assuming Sf=St=s we get that s^2+p*Et^2+(1-p)*Ef^2=E(x^2)
			    // so valsStd should be equal to p(1-p)(Et-Ef)^2+s^2
			    currStd = Math.sqrt((simStd*simStd) / (1.+currP*(1.0-currP)*delta*delta));
				if( simStd<(simStd/2.0) )
				  continue;
				// assuming meanVal = p*Et+(1-p)*Ef
				currExpF = simMean - (currP * delta * currStd);
				NormalDistribution distribF = new NormalDistribution(currExpF, currStd);
				currExpT = currExpF + (delta * currStd);
				NormalDistribution distribT = new NormalDistribution(currExpT, currStd);
				double approxPerAbove = ((1 - currP)*(1 - distribF.approxPhi(simMean))) + 
								    (currP*(1 - distribT.approxPhi(simMean)));
				double approxPerPlus = ((1 - currP)*(1 - distribF.approxPhi(simMean+simStd))) + 
								    (currP*(1 - distribT.approxPhi(simMean+simStd)));
				double approxPerMinus = ((1 - currP)*(1 - distribF.approxPhi(simMean-simStd))) + 
								    (currP*(1 - distribT.approxPhi(simMean-simStd)));
				double currDiff = 	Math.abs(approxPerAbove - perAbove) +
									Math.abs(approxPerPlus - perAbovePlus) +
									Math.abs(approxPerMinus - perAboveMinus);
				if(currDiff < bestDiff) {
					bestDiff = currDiff;
					bestP = currP;
					bestTexp = currExpT;
					bestFexp = currExpF;
					if (currStd == 0)
					{
						bestStd = 1;
					}
					bestStd = currStd;
				}
			}
		}
		_matesDist = new NormalDistribution(bestTexp,bestStd);
		_nonDist = new NormalDistribution(bestFexp,bestStd);
		_matesP = bestP;
	}
	
//////////////////////////////////////////////////////////////////////////////////////////////
//	Func Name  : calcPFMult_i
//	Parameters : pfMult_i - case i result vector
//	             prob_i - case i current probability
//	             simVals - the similarity values vector
//				   distrib_i - case i current distribution
//	Description: computes for each similarity value Sk, the multiplication : pi * fi(Sk)
////////////////////////////////////////////////////////////////////////////////////////////	//
	protected void calcPFMultTrue(double[] pfMult_i, double prob_i, StatDistribution distrib_i)
	{
//		System.out.println("IN CALCPFMULTTRUE");
		int count = 0;
		for (int i=0;i<_inpSize;i++)
		{
			for (int j=i+1;j<_inpSize;j++)
			{
				float sim = this._simMat.getSimilarity(_inds[i], _inds[j]);//__sims[_inds[i]][_inds[j]];
				if (Float.isNaN(sim))
					continue;
				pfMult_i[count++] = prob_i * _regPs[_inds[i]] * _regPs[_inds[j]] * distrib_i.calcF(sim);

			}

		}
	}
	protected void calcPFMultFalse(double[] pfMult_i, double prob_i, StatDistribution distrib_i)
	{
//		System.out.println("IN CALCPFMULTFALSE");
		int count = 0;
		for (int i=0;i<_inpSize;i++)
		{
			for (int j=i+1;j<_inpSize;j++)
			{
				float sim = this._simMat.getSimilarity(_inds[i], _inds[j]);//__sims[_inds[i]][_inds[j]];
				if (Float.isNaN(sim))
					continue;
				pfMult_i[count++] =  (1- (prob_i * _regPs[_inds[i]] * _regPs[_inds[j]])) * distrib_i.calcF(sim);

			}

		}
	}
	
//////////////////////////////////////////////////////////////////////////////////////////////
//	Func Name  : CalcGi
//	Parameters : g_i - case i result vector
//				   pfMult_i - vector containing pi * fi(e) (case i) for each edge
//				   pfMult_other - vector containing p1-i * f1-i(e) (the other case - not i) 
//								  for each edge
//	Description: computes for each edge e, Gei = fi(e)*pi / (fi(e)*pi + f1-i(e)*p1-i).
//	Returns    : the sum of all the computed values.
////////////////////////////////////////////////////////////////////////////////////////////	//
	protected double calcGi(double[] g_i, double[] pfMult_i , double[] pfMult_other)
	{
//		System.out.println("###### IN CALC GI ########");
		double sumGi = 0;
		for (int k = 0; k < g_i.length; k++)
		{	
			g_i[k] = pfMult_i[k] / (pfMult_i[k] + pfMult_other[k]);
			sumGi += g_i[k];
		}
		return sumGi;
	}

	
	protected double calcCombinedStd(double[] g1_i,double[] g2_i,double sumG1_i,double sumG2_i,double exp1,double exp2)
	{
		double sigma = 0;
		int count = 0;
		for (int i=0;i<_inpSize;i++)
		{
			for (int j=i+1;j<_inpSize;j++)
			{
				float sim = this._simMat.getSimilarity(_inds[i], _inds[j]);//__sims[_inds[i]][_inds[j]];
				if (Float.isNaN(sim))
					continue;
				sigma += (g1_i[count] * Math.pow(exp1 - sim, 2));
				sigma += (g2_i[count] * Math.pow(exp2 - sim, 2));
				count++;
			}
		}
		return Math.sqrt(sigma/(sumG1_i+sumG2_i));
	}
//////////////////////////////////////////////////////////////////////////////////////////////
//	Func Name  : CalcLikelihood
//	Parameters : pfMult_mates - vector containing pMates * fMates(e) for each edge
//				   pfMult_nonMates - vector containing pNonMates * fNonMates(e) 
//									 for each edge
//	Description: computes the likelihood of the edges given the current parameters estimation.
//	Returns    : the likelihood value
////////////////////////////////////////////////////////////////////////////////////////////	//
	protected double calcLikelihood(double[] pfMult_mates, double[] pfMult_nonMates)
	{
		double likelihood = 0;
		for (int k = 0; k < pfMult_mates.length; k++)
			likelihood += Math.log(pfMult_mates[k] + pfMult_nonMates[k]);
		return likelihood;
	}

//////////////////////////////////////////////////////////////////////////////////////////////
//	Func Name  : Run
//	Parameters : simVals - the similarity values vector
//				   matesDistrib - the resulting mates distribution (pointer to pointer)
//	             nonMatesDistrib - the resulting non-mates distribution (pointer to pointer)
//	             matesProb - the resulting mates probability (by ref)
//	Description: initialize the initial distribution options based on the given parameters.
//				   for each initial distribution : 
//				    at each iteration the parameters are maximized given the current parameters
//	              estimation. the iteration stops when the likelihood improvement is in significant.
//				   then choose the parameters the gives the best likelihood and update the result
//				   objects accordingly.
////////////////////////////////////////////////////////////////////////////////////////////	//
	public void run(ProgressManager progMan)
	{
		System.out.println(" in run " + _selfInit);
		int valsNum = _nonNaNCount;
		double[] pfMultTrue = new double[valsNum];
		double[] pfMultFalse = new double[valsNum];
		init(progMan);
		System.out.println("after init");
		System.out.println("Initial mates:"+_matesDist);
		System.out.println("Initial non-mates:"+_nonDist);
		System.out.println("Initial pmates:"+_matesP);
		progMan.setTitle("Starts iterating...");
		if(valsNum < MIN_VALS_NUM)
		{
			if((_matesDist != null) && (_nonDist != null)) {
			  if( valsNum>10 ) {
				//Reestimating P
				System.out.println("Reestimating P");
				double bestP = _matesP;
				System.out.println("current: "+ bestP);
				calcPFMultTrue(pfMultTrue, bestP, _matesDist); 
				calcPFMultFalse(pfMultFalse, bestP, _nonDist);
				double bestL = calcLikelihood(pfMultTrue, pfMultFalse);
				System.out.println("best L " + bestL);
				int v_counter = 0;
				for( int v_i=0 ; v_i<valsNum ; v_i++ ) 
				  if( (pfMultTrue[v_i]/(bestP))>(pfMultFalse[v_i]/(1.-bestP)) ) {
					v_counter++;
				  }
				for( double newP=_matesP+0.01 ; newP<=0.99 ; newP += 0.01 ) {
					calcPFMultTrue(pfMultTrue, newP, _matesDist); 
					calcPFMultFalse(pfMultFalse, newP, _nonDist);
					double currL = calcLikelihood(pfMultTrue, pfMultFalse);
					if( currL>bestL ) {
					  bestL = currL;
					  bestP = newP;
					}
				}
				//			cout << "probs=" << bestP << "," << (double)v_counter/(double)valsNum <<
				//  "(" << v_counter <<":" << valsNum <<")" << endl;
				bestP = (double)v_counter/(double)valsNum;
				_matesP = (_matesP>bestP) ? _matesP : bestP;
				if( _matesP > _maxPMates )
					_matesP = _maxPMates;
				
			  }
			} else
				throw new IllegalStateException("CEM::Run - not enough values");
		}

		if (_matesP <= 0 || _matesP > 1)
			throw new IllegalStateException ("CEM::Run - illegal value of initial mates prob");

		calcPFMultTrue(pfMultTrue, _matesP, _matesDist); 
		calcPFMultFalse(pfMultFalse, _matesP, _nonDist);
		double[] gTrue = new double[valsNum];
		double sumGTrue;
		double[] gFalse= new double[valsNum];
		double sumGFalse;

		double oldLikelihood = -1;
		double newLikelihood = calcLikelihood(pfMultTrue, pfMultFalse);
		
		int iters = 0;
		double exp1 = Double.NaN;
		double std1 = Double.NaN;
		double exp2 = Double.NaN;
		double std2 = Double.NaN;
		
		//iterate until the improvement is insignificant
//		System.out.println("IN EM: BEFORE WHILE");
	    while(((oldLikelihood == -1)|| (newLikelihood - oldLikelihood >= EPS_IMPROVE)) && !Double.isInfinite(newLikelihood))
		{
//	    	System.out.println("IN EM: IN WHILE");
			oldLikelihood = newLikelihood;
			
			sumGTrue = calcGi(gTrue, pfMultTrue, pfMultFalse);
//			System.out.println("gTRUE: " + sumGTrue);
			if(sumGTrue == 0)
				throw new IllegalStateException("sumGTrue == 0");
			sumGFalse = calcGi(gFalse, pfMultFalse, pfMultTrue);
//			System.out.println("gFALSE: " + sumGFalse);
			if(sumGFalse == 0)
				throw new IllegalStateException("sumGFalse == 0");

			if (_updateExp)
			{
				exp1 = calcExp(gTrue, sumGTrue);
				exp2 = calcExp(gFalse, sumGFalse);
			}
//			System.out.println("exp1: " + exp1);
//			System.out.println("exp2: " + exp2);
			if (_updateStd)
			{
				if (!_sameStd)
				{
					std1 = calcStd(gTrue, sumGTrue, exp1);
					std2 = calcStd(gFalse, sumGFalse, exp2);
				}
				else
				{
					std1 = std2 = calcCombinedStd(gTrue, gFalse,sumGTrue,sumGFalse,exp1,exp2);
				}
			}
//			System.out.println("std1: " + std1);
//			System.out.println("std2: " + std2);
			if (_updateExp && _updateStd)
				_matesDist.init(exp1,std1);
			else if (_updateExp)
				_matesDist.initExp(exp1);
			else if (_updateStd)
				_matesDist.initStd(std1);
			
//			System.out.println(_matesDist);
			
			if (_updateExp && _updateStd)
				_nonDist.init(exp2,std2);
			else if (_updateExp)
				_nonDist.initExp(exp2);
			else if (_updateStd)
				_nonDist.initStd(std2);
			
			System.out.println(_nonDist);
			
			//_matesP = sumGTrue / (sumGTrue + sumGFalse);
			if (_updateP)
			{
				_matesP = sumGTrue / _regsSum;
			}

			calcPFMultTrue(pfMultTrue, _matesP, _matesDist); 
			calcPFMultFalse(pfMultFalse, _matesP, _nonDist);
			newLikelihood = calcLikelihood(pfMultTrue, pfMultFalse);
			if (newLikelihood < oldLikelihood - 2)
			{
				//throw new IllegalStateException("The likelihood went down during EM");
				System.err.println("The likelihood went down during EM");
			}
			iters++;
			if( _matesP < _minPMates )
				_matesP = _minPMates;
			if (_matesP > _maxPMates)
				_matesP = _maxPMates;
			
			progMan.setTitle("Iteration "+iters);
			if (iters % _plotIterCount == 0)
			{
				System.out.println(iters+", likelihood="+newLikelihood+", mates:"+_matesDist+", non-mates:"+_nonDist+", pmates:"+_matesP);
			}
		}
	    
		_likelihood = newLikelihood;
		System.out.println("EM finished after "+iters+ " iterations");
		System.out.println("Mates Distribution:"+_matesDist);
		System.out.println("Non-Mates Distribution:"+_nonDist);
		System.out.println("Mates P:"+_matesP);
	}
	
	public void setInit(boolean i){
		super._selfInit=i;
	}
}
