package ModuleMapSolutionEval;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import math_utils.ClassesFileObject;


public class AlgorithmEvaluationResults {
	String algoName;
	double numFolds;
	LinkedList<int[]> moduleSizes;
	DC_analysis_evaluation[] evals;
	double meanFriendsDC;
	double meanNonFriendsDC;
	double meanCoverage;
	double meanFoldModuleSize;
	double medianFoldModuleSize;
	double meanNumEdges = 0;
	double meanNumNonEdges = 0;
	double meanNumModules = 0;
	double meanMaxSize = 0;
	
	double meanAboveThrCoverage = 0;
	double meanAboveThrNumModules = 0;
	double meanAboveThrNumFrEdges = 0;
	double friendsVsNonFriendsPvalue = 0;
	
	int numCalculatedFolds = -1;
	
	public AlgorithmEvaluationResults(String name, int folds) {
		numFolds=folds;
		algoName=name;
		moduleSizes = new LinkedList<int[]>();
		evals = new DC_analysis_evaluation[folds];
	}
	
	public void addFoldResults(List<HashSet<Integer>> modules, short[][] areFriends, ClassesFileObject obj, int classOI) {
		numCalculatedFolds++;
		evals[numCalculatedFolds] = new DC_analysis_evaluation(obj, classOI, modules,areFriends);
		if (modules.size()==0){
			return;
		}
		HashSet<Integer> coveredGenes  = new HashSet<Integer>();
		int[] sizes = new int[modules.size()];
		int[] sizesCopy = new int[sizes.length];
		int c=0;
		double currMean=0;
		double maxModuleSize=0;
		for (Iterator<HashSet<Integer>> iterator = modules.iterator(); iterator.hasNext();) {
			HashSet<Integer> m = (HashSet<Integer>) iterator.next();
			coveredGenes.addAll(m);
			sizes[c]=m.size();
			sizesCopy[c] = sizes[c];
			currMean+=m.size();
			if (m.size()>maxModuleSize){
				maxModuleSize=m.size();
			}
			c++;
		}
		
		currMean /= (double)modules.size();
		Arrays.sort(sizesCopy);
		double currMedian = sizesCopy[sizesCopy.length/2];
		meanFoldModuleSize += currMean/numFolds;
		medianFoldModuleSize += currMedian/numFolds;
		meanCoverage += coveredGenes.size()/numFolds;
		moduleSizes.add(sizes);
		meanFriendsDC+=evals[numCalculatedFolds].getMeanEdgesDC()/numFolds;
		meanNonFriendsDC += evals[numCalculatedFolds].getMeanNonEdgesDC()/numFolds;
		meanNumEdges+=evals[numCalculatedFolds].getNumEdges()/numFolds;
		meanNumNonEdges += evals[numCalculatedFolds].getNumNonEdges() /numFolds;
		meanNumModules+=(double)modules.size()/numFolds;
		meanMaxSize += maxModuleSize/numFolds;
		
		meanAboveThrCoverage += evals[numCalculatedFolds].aboveThrCoverage/numFolds;
		meanAboveThrNumFrEdges += evals[numCalculatedFolds].numFrAboveThr/numFolds;
		friendsVsNonFriendsPvalue += evals[numCalculatedFolds].wilcoxonPvalue/numFolds;
		meanAboveThrNumModules += evals[numCalculatedFolds].aboveThrNumModules/numFolds;
		
	}
	
	@Override
	public String toString() {
		String out = algoName+"\t"+meanFriendsDC+"\t"+meanNonFriendsDC+"\t"+meanCoverage+"\t"
		+meanFoldModuleSize+"\t"+medianFoldModuleSize+"\t"+meanNumEdges+"\t"+meanNumNonEdges
		+"\t"+meanNumModules+"\t"+meanMaxSize
		+"\t"+meanAboveThrCoverage+"\t"+meanAboveThrNumModules+"\t"+meanAboveThrNumFrEdges+"\t"+friendsVsNonFriendsPvalue;
		return out;
	}
	
	public static String getToStringHeader(){
		return "AlgorithmName\tF_DC\tNon_F_DC\tCoverage\tMeanModuleSize\t" +
				"MedianModuleSize\tNumF\tNumNonF\tNumModules\tMaxSize\tAbove0.4Coverage\tAbove0.4NumModules\tAbove0.4NumFrEdges\tPvalue";
	}
}
