package ModuleMapSolutionEval;

import edu.tau.compbio.math.TailProbs;
import global_analysis.StatisticalFriendsTester;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;

import sun.text.normalizer.CharTrie.FriendAgent;

import math_utils.ClassesFileObject;
import math_utils.ExpressionMatrix;
import math_utils.staticMathHelperMethods;

public class DC_analysis_evaluation {
	ClassesFileObject clObj;
	short[][] map;
	List<HashSet<Integer>> sets;
	List<float[]> set2meanPattern;
	int classInd;
	int classFrom;
	int classTo;
	
	// results containers
	float[][] dcMat;
	private float numEdges;
	private float numNonEdges;
	private float meanEdgesDC;
	private float meanNonEdgesDC;
	
	boolean ignoreBelowThr=true;
	double thr=0.4;
	short[][] areAboveThr;
	boolean[] hasAboveThrFriends;
	// above threshold stats
	float numFrAboveThr=0;
	float numNonFrAboveThr=0;
	float aboveThrMeanDC = 0;
	float aboveThrCoverage = 0;
	float aboveThrNumModules = 0;
	// keep the correlation differences
	LinkedList<Float> freindsDC;
	LinkedList<Float> nonFriendsDC;
	double wilcoxonPvalue;
	
	public DC_analysis_evaluation(ClassesFileObject obj,int cInd,List<HashSet<Integer>> s, short[][] map) {
		clObj=obj;
		classInd=cInd;
		sets=s;
		classFrom = clObj.getClassesBorders()[classInd*2];
		classTo = clObj.getClassesBorders()[classInd*2+1];
		set2meanPattern = new ArrayList<float[]>(sets.size());
		ExpressionMatrix means = clObj.getData().matrixByClusteringResults(sets);
		for (int i = 0; i < means.matrix.length; i++) {
			set2meanPattern.add(means.matrix[i]);
		}
		this.map=map;
		
		getEdgesDC();
		calcStatistics();
	}
	
	public float getAveragePatternDC(int i, int j){
		float c1 = staticMathHelperMethods.pearsonCorrelation(set2meanPattern.get(i), set2meanPattern.get(j), classFrom, classTo,false);
		float c2 = staticMathHelperMethods.pearsonCorrelation(set2meanPattern.get(i), set2meanPattern.get(j), classFrom, classTo,true);
		float diff = c1-c2;
		if (diff<0){
			diff*=-1;
		}
		return diff;
	}
	
	public void getEdgesDC(){
		dcMat = new float[map.length][map.length];
		areAboveThr = new short[map.length][map.length];
		hasAboveThrFriends = new boolean[dcMat.length];
		for (int i = 0; i < map.length; i++) {
			for (int j = 0; j < i; j++) {
				float currDC = getAveragePatternDC(i, j);
				dcMat[i][j]=currDC;
				dcMat[j][i]=currDC;
				if (currDC>=thr){
					areAboveThr[i][j]=1;
					areAboveThr[j][i]=1;
					aboveThrMeanDC+=currDC;
					if (map[i][j]==1){
						numFrAboveThr++;
						hasAboveThrFriends[i]=true;
						hasAboveThrFriends[j]=true;
					}
					else{
						numNonFrAboveThr++;
					}
				}
			}
		}
		if (numFrAboveThr>0)
		aboveThrMeanDC/=numFrAboveThr;
		
		freindsDC = new LinkedList<Float>();
		nonFriendsDC = new LinkedList<Float>();
		for (int i = 0; i < map.length; i++) {
			for (int j = 0; j < i; j++) {
				if (map[i][j]==1){
					freindsDC.add(dcMat[i][j]);
				}
				else{
					nonFriendsDC.add(dcMat[i][j]);
				}
			}
		}
		if (freindsDC.size()>0 && nonFriendsDC.size()>0){
			double z = StatisticalFriendsTester.getWilcoxonTestZscoreApprox(freindsDC, nonFriendsDC);
			System.out.println(z);
			wilcoxonPvalue = 1-TailProbs.probNormalC(z);
			System.out.println(wilcoxonPvalue);
		}
		else{
			wilcoxonPvalue=1;
		}
		
	}
	
	public void calcStatistics(){
		numEdges=0;
		numNonEdges=0;
		meanEdgesDC=0;
		meanNonEdgesDC = 0;
		for (int i = 0; i < map.length; i++) {
			for (int j = 0; j < i; j++) {
				if (map[i][j]==1){
					numEdges++;
					meanEdgesDC+=dcMat[i][j];
				}
				else{
					numNonEdges++;
					meanNonEdgesDC += dcMat[i][j];
				}
			}
		}
		if (numEdges>0){
			meanEdgesDC/=numEdges;
		}
		if (numNonEdges>0){
			meanNonEdgesDC/=numNonEdges;
		}
		
		for (int i = 0; i < hasAboveThrFriends.length; i++) {
			if (hasAboveThrFriends[i]){
				aboveThrCoverage+=sets.get(i).size();
				aboveThrNumModules++;
			}
		}
	}
	
	public int getClassFrom() {
		return classFrom;
	}
	public int getClassInd() {
		return classInd;
	}
	public int getClassTo() {
		return classTo;
	}
	public ClassesFileObject getClObj() {
		return clObj;
	}
	public float[][] getDcMat() {
		return dcMat;
	}
	public float getMeanEdgesDC() {
		return meanEdgesDC;
	}
	public float getMeanNonEdgesDC() {
		return meanNonEdgesDC;
	}
	public float getNumEdges() {
		return numEdges;
	}
	public float getNumNonEdges() {
		return numNonEdges;
	}
}
