# this script runs TANGO on multiple solutions
# it then compares the statistics of the solutions

def getRanksFromHashScores(d,rev=True):
    scores = d.values()
    sorted_scores = sorted(scores,reverse=rev)
    # print sorted_scores
    ranks = {}
    for a in d.keys():
        s = d[a]
        r = 0
        while r<len(sorted_scores)and sorted_scores[r]>s:
            r+=1
        ranks[a]=r+1
        # print a+"\t"+str(s)+"\t"+str(r)
    return ranks
    

import os
files_prefix = "yeast_sols/degAtLeast1Graph_"
seedAlgos = ["DicerStar","dicer","mbcDicer","hierarchical","globalNodeAddition"]
# seedAlgos = ["DicerStar"]
improveAlgos = ["global","local"]
organism = "s.cerevisiae"
out_file = "out.txt"
dummy_exp_mat = "yeast_sols/degAtLeast1Graph_dummy_exp_mat.txt"
algo2numTerms ={}
algo2pEnrichedClusters = {}
algo2avgRawPvalue = {}
algo2numEnrichments = {}
algo2moduleMap = {}
algo2pEnrichedEdges = {}
algo2numEnrichedEdges = {}
for seed in seedAlgos:
    for imp in improveAlgos:
        # run Tango
        cmd = "java -jar executable_forBatchMode.jar 5"
        alg = files_prefix+seed+"_"+imp+".txt"
        cmd = cmd + " "+dummy_exp_mat+" "+organism+" "+alg+" 1000 3000 0.05 \"\" " + out_file
        os.system(cmd)        
        # analyze the results
        f = open(out_file)
        lines = f.readlines()
        f.close()
        lines = lines[1:]
        algo = seed+"_"+imp
        algo2numEnrichments[algo]=len(lines)
        terms = {}
        modules = {}
        pvals = []
        for l in lines:
            arr = l.split("\t")
            modules[arr[0]]=1
            terms[arr[1]]=1
            pvals.append(float(arr[3]))
        numTs = len(terms.keys())
        algo2numTerms[algo]=numTs
        avgP = sum(pvals)/len(pvals)
        algo2avgRawPvalue[algo]=avgP
        # get clustering stats
        f = open(alg.replace('/','\\'))
        clustNames = {}
        for l in f:
            arr = l.split("\t")
            clustNames[arr[1]]=1
        f.close()
        pE = len(modules.keys())/float(len(clustNames.keys()))
        algo2pEnrichedClusters[algo]=pE
        # get module map
        mmap_path = files_prefix+seed+"_"+imp+"_module_map.sif"
        f = open(mmap_path.replace('/','\\'))
        mmap = {}
        for l in f:
            arr = l.split("\t")
            a = int(arr[0].split("_")[1])+1
            b = int(arr[2].split("_")[1])+1
            t1 = (a,b)
            t2 = (b,a)
            # print t1,t2
            mmap[t1]=1
            mmap[t2]=1
        f.close()
        algo2moduleMap[algo]=mmap
        n=0.0
        for m1 in modules:
            for m2 in modules:
                t = (int(m1),int(m2))
                # print "\t"+str(t)
                if mmap.has_key(t):
                    n+=1.0
        n = n/2
        # print "##### " + str(n)
        numEdges = len(mmap.keys())/2
        p=n/numEdges
        algo2pEnrichedEdges[algo]=p
        algo2numEnrichedEdges[algo]=n


f = open ("algo_TANGO_comparisons_with_edges.txt","w")
algos = sorted(algo2numTerms.keys())
f.write("algorithm\tNumEnrichedTerms\tPercentEnrichedModules\tTotalNumEnrichments\tMeanRawPvalue\tPercentEdges\tNumEdges\n")
for a in algos:
    f.write(a+"\t")
    f.write(str(algo2numTerms[a])+"\t")
    f.write(str(algo2pEnrichedClusters[a])+"\t")
    f.write(str(algo2numEnrichments[a])+"\t")
    f.write(str(algo2avgRawPvalue[a])+"\t")
    f.write(str(algo2pEnrichedEdges[a])+"\t")
    f.write(str(algo2numEnrichedEdges[a])+"\t")
    f.write("\n")    


# rank algorithms and print the ranks matrix
f.write("\n\nRanks matrix\n\n")
for a in algos:
    f.write(a+"\t")
    f.write(str(getRanksFromHashScores(algo2numTerms)[a])+"\t")
    f.write(str(getRanksFromHashScores(algo2pEnrichedClusters)[a])+"\t")
    f.write(str(getRanksFromHashScores(algo2numEnrichments)[a])+"\t")
    f.write(str(getRanksFromHashScores(algo2avgRawPvalue,False)[a])+"\t")
    f.write(str(getRanksFromHashScores(algo2pEnrichedEdges)[a])+"\t")
    f.write(str(getRanksFromHashScores(algo2numEnrichedEdges)[a])+"\t")
    f.write("\n")  


f.close()
        
