package global_analysis;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import math_utils.StaticGraphFunctions;

public abstract class GlobalImproveAlgorithm {
	
	/* Algorithm parameters */
	float alpha = 0.005f;
	float beta = 0.2f;
	int maxModuleSize=3000;
	// this is the between gain parameter
	float gamma = 1;
	int g2Power = 1;

	// the data
	float[][] g1;
	float[][] g2;
	List<HashSet<Integer>> modules;
	
	boolean isModule[];
	List<HashSet<Integer>> moduleFriendsList;
	List<HashSet<Integer>> moduleNonSignificantFriendsList;
	// between modules scores
	float[][] modules_between_score_g1;
	float[][] modules_between_score_g2;
	
	int numMerges=0;
	
	/* Auxiliary members */
	ArrayList<LinkedList<Integer>> neighborListsOfG2;
	StatisticalFriendsTester friendsTester;
	boolean discreteTest = true;
	double pValueFriendshipThreshod=0.01;

	public GlobalImproveAlgorithm(float[][] g1,float[][] g2,float edgethr, boolean disc) {
		this.g1=g1;
		this.g2=g2;
		discreteTest = disc;
		
		initFreindsCalcMembers(edgethr);
		
		ArrayList<HashSet<Integer>> ms = new ArrayList<HashSet<Integer>>(g1.length);
		for (int i = 0; i < g2.length; i++) {
			HashSet<Integer> singleton = new HashSet<Integer>();
			singleton.add(i);
			ms.add(singleton);
		}
		init(ms);
	}
	
	// empty improver, can be used to calculate score
	public GlobalImproveAlgorithm(float[][] g1,float[][] g2) {
		this.g1=g1;
		this.g2=g2;
		discreteTest = true;
		initFreindsCalcMembers(0);
	}
	
	public GlobalImproveAlgorithm(float[][] g1,float[][] g2,float edgethr, List<HashSet<Integer>> ms, boolean disc) {
		this.g1=g1;
		this.g2=g2;
		discreteTest = disc;
		
		initFreindsCalcMembers(edgethr);	
		ms = addSingletons(g2, ms);
		init(ms);
	}
	
	public GlobalImproveAlgorithm(float[][] g1,float[][] g2,float edgethr, List<HashSet<Integer>> ms, boolean disc, float a) {
		alpha = a;
		this.g1=g1;
		this.g2=g2;
		discreteTest = disc;
		initFreindsCalcMembers(edgethr);	
		init(ms);
	}

	public List<HashSet<Integer>> addSingletons(float[][] g2, List<HashSet<Integer>> ms) {
		
		List<HashSet<Integer>> output = new LinkedList<HashSet<Integer>>();
		for (Iterator<HashSet<Integer>> iterator = ms.iterator(); iterator.hasNext();) {
			HashSet<Integer> m = (HashSet<Integer>) iterator.next();
			output.add(new HashSet<Integer>(m));
		}
		
		boolean[] isCovered = new boolean[g1.length];
		for (Iterator<HashSet<Integer>> iterator = ms.iterator(); iterator.hasNext();) {
			HashSet<Integer> module = (HashSet<Integer>) iterator.next();
			for (Iterator<Integer> iterator2 = module.iterator(); iterator2.hasNext();) {
				Integer inModule = (Integer) iterator2.next();
				isCovered[inModule]=true;
			}
		}
		// add all non covered as singletons
		for (int i = 0; i < g1.length; i++) {
			if (isCovered[i]){
				continue;
			}
			HashSet<Integer> singleton = new HashSet<Integer>();
			singleton.add(i);
			output.add(singleton);
		}
		return output;
	}
	
	public void init(List<HashSet<Integer>> modules){
		setModules(modules);
		int n = modules.size();
		moduleFriendsList = new ArrayList<HashSet<Integer>>(n);
		moduleNonSignificantFriendsList = new ArrayList<HashSet<Integer>>(n);
		modules_between_score_g1 = new float[n][n];
		modules_between_score_g2 = new float[n][n];
		isModule = new boolean[n];
		for (int i = 0; i < isModule.length; i++) {
			isModule[i]=true;
			moduleFriendsList.add(new HashSet<Integer>());
			moduleNonSignificantFriendsList.add(new HashSet<Integer>());
			for (int j = 0; j < i; j++) {
				modules_between_score_g1[i][j] = getModuleVsModuleInGraph(modules.get(i), modules.get(j), g1);
				modules_between_score_g1[j][i]= modules_between_score_g1[i][j];
				
				modules_between_score_g2[i][j] = getModuleVsModuleInGraph(modules.get(i), modules.get(j), g2,g2Power);
				modules_between_score_g2[j][i]= modules_between_score_g2[i][j];
				
				short currRelation = areModuleFriends(modules.get(i), modules.get(j)).result;
				if (currRelation==1){
					moduleFriendsList.get(i).add(j);
					moduleFriendsList.get(j).add(i);
				}
				else if (currRelation==0){
					moduleNonSignificantFriendsList.get(i).add(j);
					moduleNonSignificantFriendsList.get(j).add(i);
				}
			}
		}
	}
	
	public void initFreindsCalcMembers(float thr){
		calcNeighborListsOfG2(thr);
		friendsTester = new StatisticalFriendsTester(g2, neighborListsOfG2);
		friendsTester.edgeThr=thr;
		// the Wilcoxon test is extremely slow
		// -> change the threshold to save tests
		if (! discreteTest){
			friendsTester.minSizeForAntifriendship=1;
		}
	}
	
	public void calcNeighborListsOfG2(float thr){
		neighborListsOfG2 = new ArrayList<LinkedList<Integer>>(g2.length);
		for (int i = 0; i < g2.length; i++) {
			LinkedList<Integer> currNeis = new LinkedList<Integer>();
			for (int j = 0; j < g2.length; j++) {
				if (i==j){continue;};
				if (g2[i][j]> thr){
					currNeis.add(j);
				}
			}
			neighborListsOfG2.add(currNeis);
		}
	}

	public static float getGeneVsModuleScore(int gene, float[][] graph, HashSet<Integer> module) {
		float score = 0;
		for (Iterator<Integer> iterator = module.iterator(); iterator.hasNext();) {
			Integer inModule = (Integer) iterator.next();
			score += graph[gene][inModule];
		}
		return score;
	}

	public static float getModuleVsModuleInGraph(HashSet<Integer> m1, HashSet<Integer> m2, float[][] graph) {
		float score = 0;
		for (Iterator<Integer> iterator = m2.iterator(); iterator.hasNext();) {
			Integer inM2 = (Integer) iterator.next();
			score += getGeneVsModuleScore(inM2, graph, m1);
		}
		return score;
	}
	
	public static float getGeneVsModuleScore(int gene, float[][] graph, HashSet<Integer> module,float pow) {
		float score = 0;
		for (Iterator<Integer> iterator = module.iterator(); iterator.hasNext();) {
			Integer inModule = (Integer) iterator.next();
			score += (new Double(Math.pow(graph[gene][inModule],pow))).floatValue();
		}
		return score;
	}

	public static float getModuleVsModuleInGraph(HashSet<Integer> m1, HashSet<Integer> m2, float[][] graph,float pow) {
		float score = 0;
		for (Iterator<Integer> iterator = m2.iterator(); iterator.hasNext();) {
			Integer inM2 = (Integer) iterator.next();
			score += getGeneVsModuleScore(inM2, graph, m1,pow);
		}
		return score;
	}

	/**
	 * This method calculates the gain of merging two modules
	 * @param m1Index
	 * @param m2Index
	 * @return
	 */
	public MergeOperationObject getModuleMergeGain(int m1Index, int m2Index) {

		MergeOperationObject obj = new MergeOperationObject(m1Index, m2Index,moduleFriendsList.get(m1Index),moduleFriendsList.get(m2Index)
				,moduleNonSignificantFriendsList.get(m1Index),moduleNonSignificantFriendsList.get(m2Index));
		
		if (modules.get(m1Index).size() > maxModuleSize || modules.get(m2Index).size() > maxModuleSize){
			obj.mergeGain = -Float.MAX_VALUE;
			obj.withinGain = -Float.MAX_VALUE;
			return obj;
		}
		
		// step 1: the g1 gain of merging the two clusters
		float withinGain = modules_between_score_g1[m1Index][m2Index];
		obj.setWithinGain(withinGain);
		if (withinGain<0){
			obj.setMergeGain(-1);
			return obj;
		}
		// step 2: if modules were friends we loose the added value in g2
		short areM1M2FriendsCoef = areModuleFriends(m1Index, m2Index);
		if (areM1M2FriendsCoef<1){areM1M2FriendsCoef=0;}
		float betweenLoss = -areM1M2FriendsCoef
				* modules_between_score_g2[m1Index][m2Index];
		obj.setBetweenLoss(betweenLoss);

		HashSet<Integer> union = new HashSet<Integer>();
		union.addAll(modules.get(m1Index));
		union.addAll(modules.get(m2Index));
		obj.setUnion(union);
		
		// step 3: calculate the added value of the new module as compared to other modules
		// For each module X:
		// 1. Check if the new module is a friends of X
		// 2. Get the new score between X and the union in g2
		// 3. Get the old score between X and module 1 and X and module 2 in g2
		// 4. Add the difference between the new score and the old score
		
		// we add the following heuristic to boost running time:
		// if X is not a friend of both modules then we assume the union will not
		// be significant as well
		float total_between_gain = 0;
//		short[] friends = new short[areModulesFriends.length];
//		Arrays.fill(friends, (short)-1);
		
		// get friends and non friends of m1 and m2
		HashSet<Integer> possibleFriends = new HashSet<Integer>();
		possibleFriends.addAll(moduleFriendsList.get(m1Index));
		possibleFriends.addAll(moduleNonSignificantFriendsList.get(m1Index));
		possibleFriends.addAll(moduleFriendsList.get(m2Index));
		possibleFriends.addAll(moduleNonSignificantFriendsList.get(m2Index));
		
		for (Iterator<Integer> iterator = possibleFriends.iterator(); iterator.hasNext();) {
			Integer i = (Integer) iterator.next();
			if (!isModule[i] || i == m1Index || i == m2Index) {
				continue;
			}
			short areFriendsNow = 0;
			if (areModulesAntiFriends(i, m1Index)|| areModulesAntiFriends(i, m2Index)){
				areFriendsNow=-1;
			}
			else{
				if (discreteTest){
					areFriendsNow = areModuleFriends(union, modules.get(i)).result;
				}
				else{
					areFriendsNow = -1;
					if (areModulesAntiFriends(i, m1Index)&& areModulesAntiFriends(i, m2Index)){
						areFriendsNow=1;
					}
				}
				
			}
			
			if (areFriendsNow==1){
				obj.getFij().add(i);
			}
			if (areFriendsNow==0){
				obj.getNonSigFriendsOfUnion().add(i);
			}
			
//			friends[i]=areFriendsNow;
			
			short currNewCoef = areFriendsNow;
			if (currNewCoef<1){currNewCoef=0;}
			
			float newBetweenScore = currNewCoef
					* (modules_between_score_g2[i][m1Index] + modules_between_score_g2[i][m2Index]);
			float oldBetweenScore = 0;
			if (areModuleFriends(i, m1Index)==1){
				oldBetweenScore+=modules_between_score_g2[i][m1Index];
			}
			if (areModuleFriends(i, m2Index)==1){
				oldBetweenScore+=modules_between_score_g2[i][m2Index];
			}
				
			float between_gain = newBetweenScore - oldBetweenScore;
			total_between_gain += between_gain;
		}
		
		float gain = gamma*withinGain+betweenLoss+total_between_gain; 
		obj.setMergeGain(gain);
		
//		if (gain > 10){
//			System.out.println("%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%");
//			System.out.println(gain);
//			System.out.println(withinGain);
//			System.out.println(betweenLoss);
//			System.out.println(total_between_gain);
//			System.out.println("%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%");
//		}
		
		return obj;

	}
	
	public float getModuleMergeGainIgnoreFriendsRelationships(int m1Index, int m2Index) {
		if (modules.get(m1Index).size() > maxModuleSize || modules.get(m2Index).size() > maxModuleSize){
			return -Float.MAX_VALUE;
		}
		return modules_between_score_g1[m1Index][m2Index];		
	}

	/**
	 * This method merges two modules
	 * The new module index is min(m1Index,m2Index)
	 * @param m1Index
	 * @param m2Index
	 */
	public void doMergeIgnoreFriendRelationships(int m1Index, int m2Index){
			
		numMerges++;
		
		// if m1>m2 swap
		if (m1Index > m2Index){
			int tmp = m1Index;
			m1Index = m2Index;
			m2Index=tmp;
		}
		// m2 is no longer a module
		isModule[m2Index]=false;
		// unite nodes
		HashSet<Integer> union = new HashSet<Integer>();
		union.addAll(modules.get(m1Index));
		union.addAll(modules.get(m2Index));
		
		// update all matrices
		for (int i = 0; i < isModule.length; i++) {
			if (!isModule[i] || i == m1Index || i == m2Index) {
				continue;
			}
			
			// update between scores of g1			
			float newBetweenInG1 = (modules_between_score_g1[i][m1Index] + modules_between_score_g1[i][m2Index]);
			modules_between_score_g1[i][m1Index] = newBetweenInG1;
			modules_between_score_g1[m1Index][i] = newBetweenInG1;
			
			// update between scores of g2
			float newBetweenInG2 = (modules_between_score_g2[i][m1Index] + modules_between_score_g2[i][m2Index]);
			modules_between_score_g2[i][m1Index] = newBetweenInG2;
			modules_between_score_g2[m1Index][i] = newBetweenInG2;
		}
		// set the modules object
		modules.set(m1Index, union);
		modules.set(m2Index, null);
		
	}
	
	/**
	 * This method performs a set of merge operations
	 * We assume that the friend relationships of the new modules
	 * do not overlap.
	 * That is, a module x that is not in one of this objects, can 
	 * be a friend of at most on new united model.
	 * Under this assumption, we can use the Fij list of the 
	 * objects to update the areModuleFriends matrix.
	 * @param m1Index
	 * @param m2Index
	 */
	public void doMerge(Collection<MergeOperationObject> objs){
		numMerges+=objs.size();
		
//		System.out.println(" before merge: " + getModuleStatistics());
		
		for (Iterator<MergeOperationObject> iterator = objs.iterator(); iterator.hasNext();) {
			MergeOperationObject obj = (MergeOperationObject) iterator
					.next();
			
//			System.out.println(obj.toFullString());
			
			int m1Index = obj.getI();
			int m2Index = obj.getJ();
			
			// m2 is no longer a module
			isModule[m2Index]=false;
			
			// update are module friends
			HashSet<Integer> unionFriends = obj.getFij();
			moduleFriendsList.set(m2Index, null);
			moduleFriendsList.set(m1Index, unionFriends);
			
			
			HashSet<Integer> nonSigUnionFriends = obj.getNonSigFriendsOfUnion();
			moduleNonSignificantFriendsList.set(m2Index, null);
			moduleNonSignificantFriendsList.set(m1Index, nonSigUnionFriends);
			
			for (int i = 0; i < isModule.length; i++) {
				if (!isModule[i]){continue;}
				moduleFriendsList.get(i).remove(new Integer(m1Index));
				moduleFriendsList.get(i).remove(new Integer(m2Index));
				moduleNonSignificantFriendsList.get(i).remove(new Integer(m1Index));
				moduleNonSignificantFriendsList.get(i).remove(new Integer(m2Index));
				
				// update between scores of g1			
				float newBetweenInG1 = (modules_between_score_g1[i][m1Index] + modules_between_score_g1[i][m2Index]);
				modules_between_score_g1[i][m1Index] = newBetweenInG1;
				modules_between_score_g1[m1Index][i] = newBetweenInG1;
				
				// update between scores of g2			
				float newBetweenInG2 = (modules_between_score_g2[i][m1Index] + modules_between_score_g2[i][m2Index]);
				modules_between_score_g2[i][m1Index] = newBetweenInG2;
				modules_between_score_g2[m1Index][i] = newBetweenInG2;
			}
					
			// update modules
			// set the modules object
			modules.set(m1Index, obj.getUnion());
			modules.set(m2Index, null);
			
			for (Iterator<Integer> iterator2 = unionFriends.iterator(); iterator2
					.hasNext();) {
				Integer friendOfUnion = (Integer) iterator2.next();
				if (! isModule[friendOfUnion]){continue;}
				moduleFriendsList.get(friendOfUnion).add(m1Index);
			}
			
			for (Iterator<Integer> iterator2 = nonSigUnionFriends.iterator(); iterator2
					.hasNext();) {
				Integer nonSigF = (Integer) iterator2.next();
				if (! isModule[nonSigF]){continue;}
				moduleNonSignificantFriendsList.get(nonSigF).add(m1Index);
			}
			
		}
		
//		System.out.println(" after merge: " + getModuleStatistics());
	}

	public short areModuleFriends(int a, int b) {
		if (moduleFriendsList.get(a).contains(b) || moduleFriendsList.get(b).contains(a)){return 1;}
		if (moduleNonSignificantFriendsList.get(a).contains(b)){return 0;}
		return -1;
	}
	
	public boolean areModulesAntiFriends(int a, int b) {
		return !moduleFriendsList.get(a).contains(b) && !moduleNonSignificantFriendsList.get(a).contains(b);
	}
	
	
	public List<HashSet<Integer>> updateModuleList(int minSize, int minNumFriends) {
		
		ArrayList<HashSet<Integer>> nModules= new ArrayList<HashSet<Integer>>();
		
		for (int i = 0; i < isModule.length; i++) {
			if (!isModule[i]|| modules.get(i).size()<minSize){continue;};
			int numFriends = moduleFriendsList.get(i).size();
			if (numFriends<minNumFriends){continue;};
			nModules.add(modules.get(i));
		}
		
		modules=nModules;
		init(modules);
		return modules;
	}
	
	public void cleanup(){
		
//		System.out.println("before cleanup: " + getModuleStatistics());
		
		if (numMerges < 200 ){
			return;
		}
		
		numMerges=0;
		
		ArrayList<HashSet<Integer>> nModules = new ArrayList<HashSet<Integer>>();
		for (int i = 0; i < modules.size(); i++) {
			if (!isModule[i]){
				continue;
			}
			nModules.add(modules.get(i));
		}
		
		int n = nModules.size();
		modules = nModules;
		
//		short[][] n_areModulesFriends = new short[n][n];
		ArrayList<HashSet<Integer>> n_moduleFriendsList = new ArrayList<HashSet<Integer>>(n);
		ArrayList<HashSet<Integer>> n_moduleNonSignificantFriendsList= new ArrayList<HashSet<Integer>>(n);
		float[][] n_modules_between_score_g1 = new float[n][n];
		float[][] n_modules_between_score_g2 = new float[n][n];
		boolean[] n_isModule = new boolean[n];
		
		int iC=0;
		for (int i = 0; i < isModule.length; i++) {
			if (!isModule[i]){
				continue;
			}
			n_moduleFriendsList.add(new HashSet<Integer>());
			n_moduleNonSignificantFriendsList.add(new HashSet<Integer>());
			n_isModule[iC]=true;
			int jC=0;
			for (int j = 0; j < i; j++) {
				if (!isModule[j]){
					continue;
				}
				n_modules_between_score_g1[iC][jC] = modules_between_score_g1[i][j];
				n_modules_between_score_g1[jC][iC]= modules_between_score_g1[i][j];
				
				n_modules_between_score_g2[iC][jC] = modules_between_score_g2[i][j];
				n_modules_between_score_g2[jC][iC] = modules_between_score_g2[i][j];
				
				short currRelation = areModuleFriends(i, j); 
				if (currRelation==1){
					n_moduleFriendsList.get(iC).add(jC);
					n_moduleFriendsList.get(jC).add(iC);
				}
				
				if (currRelation==0){
					n_moduleNonSignificantFriendsList.get(iC).add(jC);
					n_moduleNonSignificantFriendsList.get(jC).add(iC);
				}
				
				jC++;
			}
			
			iC++;
		}
		
		moduleFriendsList = n_moduleFriendsList;
		moduleNonSignificantFriendsList = n_moduleNonSignificantFriendsList;
		modules_between_score_g1=n_modules_between_score_g1;
		modules_between_score_g2 = n_modules_between_score_g2;
		isModule = n_isModule;
		Runtime.getRuntime().gc();
		
//		System.out.println("after cleanup: " + getModuleStatistics());

	}
	
	/**
	 * A bad module is a module that:
	 * 1. has no friends
	 * 2. has no possible positive gain in g1
	 * 3. has no possible positive gain in g2
	 */
	public void cleanSingletons(){
		int numRemoved = 0;
		for (int i = 0; i < isModule.length; i++) {
			if (!isModule[i]){
				continue;
			}
			int numFriends = moduleFriendsList.get(i).size();
			int numPossibleGainsInG1 = 0;
			int numPossibleGainsInG2 = 0;
			for (int j = 0; j < isModule.length; j++) {
				if (j==i || ! isModule[j]){
					continue;
				}
				if (modules_between_score_g1[i][j]>0){
					numPossibleGainsInG1++;
				}
				if (modules_between_score_g2[i][j]>0){
					numPossibleGainsInG2++;
				}
			}
			
			int total = numFriends+numPossibleGainsInG1+numPossibleGainsInG2;
			if (total==0|| modules.get(i).size()==1){
				numRemoved++;
				isModule[i]=false;
			}
		}
		
//		System.out.println("removed : " + numRemoved);
		cleanup();
	}
	
	public List<HashSet<Integer>> getModulesCopy() {
		List<HashSet<Integer>> copy = new ArrayList<HashSet<Integer>>(modules.size());
		for (Iterator<HashSet<Integer>> iterator = modules.iterator(); iterator.hasNext();) {
			HashSet<Integer> m = (HashSet<Integer>) iterator.next();
			copy.add(new HashSet<Integer>(m));
		}
		return copy;
	}
	
	public void setModules(List<HashSet<Integer>> modules) {
		this.modules = new ArrayList<HashSet<Integer>>();
		for (Iterator<HashSet<Integer>> iterator = modules.iterator(); iterator.hasNext();) {
			HashSet<Integer> m = (HashSet<Integer>) iterator.next();
			this.modules.add(new HashSet<Integer>(m));
		}
	}
	
	public short[][] getAreModulesFriends() {
		short[][] mat = new short[modules.size()][modules.size()];
		for (int i = 0; i < mat.length; i++) {
			Arrays.fill(mat[i], (short) -1);
		}
		for (int i = 0; i < moduleFriendsList.size(); i++) {
			if (! isModule[i]){continue;}
			HashSet<Integer> currFriends = moduleFriendsList.get(i);
			for (Iterator<Integer> iterator = currFriends.iterator(); iterator.hasNext();) {
				Integer f = (Integer) iterator.next();
				mat[i][f]=1;
				mat[f][i]=1;
			}
		}
		for (int i = 0; i < moduleNonSignificantFriendsList.size(); i++) {
			if (! isModule[i]){continue;}
			HashSet<Integer> currFriends = moduleNonSignificantFriendsList.get(i);
			for (Iterator<Integer> iterator = currFriends.iterator(); iterator.hasNext();) {
				Integer f = (Integer) iterator.next();
				if (mat[i][f]==1){System.out.println("ERROR!!!");}
				mat[i][f]=0;
				mat[f][i]=0;
			}
		}
		return mat;
	}
	
	public String getAreModuleFriendsStatistics(){
		short[][] areModulesFriends = getAreModulesFriends();
		float n = 0;
		float numZ = 0;
		float num1=0;
		float numMinus1=0;
		for (int i = 0; i < areModulesFriends.length; i++) {
			if (!isModule[i]){continue;}
			for (int j = 0; j < i; j++) {
				if (!isModule[j]){continue;}
				n++;
				if (areModulesFriends[i][j]==1){
					num1++;
				}
				else if (areModulesFriends[i][j]==-1){
					numMinus1++;
				}
				else {
					numZ++;
				}
			}
		}
		
		float pZ = numZ/n;
		float p1 = num1/n;
		float pMinus1 = numMinus1/n;
		int numCCs = StaticGraphFunctions.getConnectedComponents(areModulesFriends, 0).size();
		String out = numZ+","+pZ+"\t"+num1+","+p1+"\t"+numMinus1+","+pMinus1+"\t"+numCCs;
		return out;
	}
	
	public String getModuleStatistics(){
		String out = "";
		float meanSize = 0;
		float numModules = 0;
		float meanFriendDeg = 0;
		float numModulesWithFriends = 0;
		float numSmallModules = 0;
		for (int i = 0; i < modules.size(); i++) {
			if (!isModule[i]){
				for (int j = 0; j < modules.size(); j++) {
					if (! isModule[j]){continue;}
					if (moduleFriendsList.get(j).contains(i)){
						System.err.println("module friends list contains non module object");
						throw (new RuntimeException());
					}
					
				}
				continue;
			}
			// Do tests
			// exit if one of them fails
			if (moduleFriendsList.get(i).contains(i)){
				System.err.println("self loop in friends graph");
				throw (new RuntimeException());
			}
			if (moduleNonSignificantFriendsList.get(i).contains(i)){
				System.err.println("self loop in non significant friends graph");
				throw (new RuntimeException());
			}
			if (modules.get(i)==null){
				System.err.println("Null module");
				throw (new RuntimeException());
			}
			meanSize+=modules.get(i).size();
			numModules++;
			meanFriendDeg+=moduleFriendsList.get(i).size();
			if (moduleFriendsList.get(i).size()>0){
				numModulesWithFriends++;
			}
			if (modules.get(i).size() <=3){
				numSmallModules++;
			}
		}
		meanSize/=numModules;
		meanFriendDeg/=numModules;
		out = numModules+"\t"+meanSize+"\t"+meanFriendDeg+"\t"+numModulesWithFriends+"\t"+numSmallModules;
		
		checkFuziness();
		
		return out;
		
	}

	public void checkFuziness() {
		boolean[] coverage = new boolean[g1.length];
		Arrays.fill(coverage,false);
		for (Iterator<HashSet<Integer>> iterator = modules.iterator(); iterator.hasNext();) {
			HashSet<Integer> m = (HashSet<Integer>) iterator.next();
			for (Iterator<Integer> iterator2 = m.iterator(); iterator2.hasNext();) {
				Integer inM = (Integer) iterator2.next();
				if (coverage[inM]){
					System.err.println("Error, the solution is fuzzy");
					throw (new RuntimeException());
				}
				coverage[inM]=true;
			}
		}
	}

	// methods that implementing algorithms should override
	
	public AreModuleFriendsTestResult areModuleFriends(HashSet<Integer> a, HashSet<Integer> b) {
		return null;
	}
	
	public void runAlgo(){
		
	}

}
