package global_analysis;

import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

public class GreedyHillClimber extends GlobalImproveAlgorithm {

	int maxPositiveMovesCacheSize = 1000;
	int minSizeForFriendship = 1;
	int minModuleSize = 5;
	
	public GreedyHillClimber(float[][] g1, float[][] g2,float edgethr,boolean disc) {
		super(g1, g2,edgethr, disc);
	}

	public GreedyHillClimber(float[][] g1, float[][] g2) {
		super(g1, g2);
	}

	public GreedyHillClimber(float[][] g1, float[][] g2,float edgethr,
			List<HashSet<Integer>> ms,boolean disc) {
		super(g1, g2,edgethr, ms,disc);
	}
	
	public GreedyHillClimber(float[][] g1, float[][] g2,float edgethr,
			List<HashSet<Integer>> ms,boolean disc,int minS) {
		super(g1, g2,edgethr, ms,disc);
		setMinModuleSize(minS);
	}
	
	public GreedyHillClimber(float[][] g1, float[][] g2,float edgethr,
			List<HashSet<Integer>> ms,boolean disc,float a) {
		super(g1, g2,edgethr, ms,disc,a);
	}
	
	public void setMinModuleSize(int minModuleSize) {
		this.minModuleSize = minModuleSize;
	}

	@Override
	public AreModuleFriendsTestResult areModuleFriends(HashSet<Integer> a, HashSet<Integer> b) {
		
		if (a.size()<minSizeForFriendship || b.size() < minSizeForFriendship){
			AreModuleFriendsTestResult res = new AreModuleFriendsTestResult();
			res.p=-1;
			res.result=0;
			return res;
		}
		
		if (discreteTest) {
			// areModuleFriendsHyperGeometricMaxPMetaTest
			// areModuleFriendsHyperGeometricFisherMetaTest
			return friendsTester.areModuleFriendsHyperGeometricFisherMetaTest(
					a, b, alpha,beta);
		} else {
			return friendsTester.areModuleFriendsDICERSimpleTest(a, b);
		}
	}

	/**
	 * This method adds singletons to modules, based only on g1
	 */
	public void doNodeAdditionKeepCEmpty(int maxSize) {
		while (true) {

			double currBestGain = -Float.MAX_VALUE;
			int ind1 = -1;
			int ind2 = -1;
			for (int i = 0; i < modules.size(); i++) {
				if (!isModule[i] || modules.get(i).size()>maxSize) {
					continue;
				}
				for (int j = 0; j < i; j++) {
					if (!isModule[j] || modules.get(j).size()>maxSize
							|| (modules.get(i).size() > 1 && modules.get(j).size() > 1)) {
						continue;
					}

					double curr_gain = getModuleMergeGainIgnoreFriendsRelationships(i, j);
									
					if (curr_gain > currBestGain) {
						currBestGain = curr_gain;
						ind1 = i;
						ind2 = j;
					}
				}
			}

			if (currBestGain <= 0) {
				break;
			}

			doMergeIgnoreFriendRelationships(ind1, ind2);
		}
		
		numMerges = Integer.MAX_VALUE;
		updateModuleList(-1, -1);
	}
	
	public void doNodeAdditionKeepCEmpty() {
		doNodeAdditionKeepCEmpty(Integer.MAX_VALUE);
	}

	public void doNodeAddition() {
		doModuleMergesFast(1,-1);
	}
	
	public void doModuleMerges() {
		doModuleMergesFast(Integer.MAX_VALUE,-1);
	}
	
	public void doModuleMergesFast(int maxModuleSize, int minModuleSizeToConsider) {
		while (true) {
			TopKScoresContainer<MergeOperationObject> positiveGainMoves = new TopKScoresContainer<MergeOperationObject>(maxPositiveMovesCacheSize);
			for (int i = 0; i < modules.size(); i++) {
				if (!isModule[i] || modules.get(i).size()<minModuleSizeToConsider) {
					continue;
				}
				for (int j = 0; j < i; j++) {
					if (!isModule[j] || modules.get(j).size()<minModuleSizeToConsider) {
						continue;
					}
					
					if (modules.get(i).size() > maxModuleSize || modules.get(j).size() > maxModuleSize) {
						continue;
					}
					
					MergeOperationObject obj = getModuleMergeGain(i, j);
					if (obj.getMergeGain() > 0){
						positiveGainMoves.add(obj);
					}
				}
			}

			if (positiveGainMoves.getList().size() == 0) {
				break;
			}
			
			LinkedList<MergeOperationObject> selectedMoves = new LinkedList<MergeOperationObject>();
			int pointer = 0;
			while (pointer < positiveGainMoves.getList().size()){
				
				MergeOperationObject currBest = positiveGainMoves.getList().get(pointer);
				selectedMoves.add(currBest);
				
				LinkedList<MergeOperationObject> toRemove = new LinkedList<MergeOperationObject>();
				HashSet<Integer> fi_u_fj = new HashSet<Integer>(currBest.getFiUnionFj());
				fi_u_fj.add(currBest.getI());
				fi_u_fj.add(currBest.getJ());
				
				HashSet<Integer> Fij =  new HashSet<Integer>(currBest.getFij());
				Fij.add(currBest.getI());
				Fij.add(currBest.getJ());
				
				for (int i = pointer+1; i < positiveGainMoves.getList().size(); i++) {
									
					MergeOperationObject move = positiveGainMoves.getList().get(i);
					
					if (currBest.getI()==move.getI()||currBest.getI()==move.getJ()||
							currBest.getJ()==move.getI()||currBest.getJ()==move.getJ()){
						toRemove.add(move);
						continue;
					}
					
					HashSet<Integer> m_fi_u_fj = move.getFiUnionFj();
					boolean removed =false;
					for (Iterator<Integer> iterator2 = m_fi_u_fj.iterator(); iterator2
							.hasNext();) {
						Integer in_m_fi_u_fj = (Integer) iterator2.next();
						if (fi_u_fj.contains(in_m_fi_u_fj)){
							toRemove.add(move);
							removed = true;
							break;
						}
					}
					if (removed){continue;}
					HashSet<Integer> m_fij = move.getFij();
					for (Iterator<Integer> iterator2 = m_fij.iterator(); iterator2
							.hasNext();) {
						Integer in_m_fij = (Integer) iterator2.next();
						if (Fij.contains(in_m_fij)){
							toRemove.add(move);
							break;
						}
					}
				}
				
				positiveGainMoves.getList().removeAll(toRemove);
				pointer++;
			}
			
			System.out.println("num selected moves: " + selectedMoves.size());
			doMerge(selectedMoves);
			cleanup();
			Runtime.getRuntime().gc();
		}

		numMerges = Integer.MAX_VALUE;
		cleanup();
		
	}

//	@Override
	public void runAlgo() {
		runImprovementStage(false);
	}
	public void runAlgo(boolean cor) {
		runImprovementStage(cor);
	}
	
	public void runImprovementStage(boolean BonfCorrection) {
		
		// first, check that the initial solution is ok
		System.out.println(getModuleStatistics());
		
		if (discreteTest){
			for (float tmp_beta = alpha; tmp_beta <=0.2+alpha ; tmp_beta += 0.1f) {
				doImproveBasicModuleMerges(tmp_beta,true);
			}
		}
		else{
			doImproveBasicModuleMerges(alpha,true);
			updateModuleList(minModuleSize, 1);
		}
		if (BonfCorrection){
			alpha = 0.1f/friendsTester.countTests;
			System.out.println(alpha);
			init(modules);
			updateModuleList(minModuleSize, 1);
		}
		else{
			updateModuleList(minModuleSize, 1);
		}
		// test for errors
		System.out.println(getModuleStatistics());
		
	}

	public void doImproveBasicModuleMerges(float tmp_beta, boolean useSingletons) {
		beta=tmp_beta;
		modules = addSingletons(g2, modules);
		init(modules);
		updateModuleList(1,1);
		doNodeAdditionKeepCEmpty(50);
		if (useSingletons)
			doNodeAddition();
		updateModuleList(minModuleSize,0);
		doModuleMerges();
	}
	
}
