package global_analysis;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;

import math_utils.WekaStatistics;

import edu.tau.compbio.math.TailProbs;

public class StatisticalFriendsTester {
	
	/* members*/
	float[][] g2;
	ArrayList<LinkedList<Integer>> g2Neis;
	float edgeThr=0;
	int minSizeForAntifriendship = 3;
	int countTests = 0;
	
	public StatisticalFriendsTester(float[][] g,ArrayList<LinkedList<Integer>> g2N) {
		g2=g;
		g2Neis=g2N;
	}
	
	public AreModuleFriendsTestResult areModuleFriendsHyperGeometricFisherMetaTest(
			HashSet<Integer> a, HashSet<Integer> b, float alpha,float beta) {

		countTests++;
		double ChiSquareStatistic = 0;
		for (Iterator<Integer> iterator = b.iterator(); iterator.hasNext();) {
			Integer bNode = (Integer) iterator.next();
			double p = getNodeVsModuleHyperGeometricP(bNode, a);
			ChiSquareStatistic -= Math.log(p);
		}
		
		for (Iterator<Integer> iterator = a.iterator(); iterator.hasNext();) {
			Integer aNode = (Integer) iterator.next();
			double p = getNodeVsModuleHyperGeometricP(aNode, b);
			ChiSquareStatistic -= Math.log(p);
		}
		
		ChiSquareStatistic *=2;
		
		double p=1;
		if (Double.isInfinite(ChiSquareStatistic)){
			p=0;
		}
		else{
			int dfs = a.size()+b.size();
			p = WekaStatistics.chiSquaredProbability(ChiSquareStatistic, dfs);
		}
		
		AreModuleFriendsTestResult res = new AreModuleFriendsTestResult();
		res.p=p;
		if (p<=alpha){
			res.result=1;
		}
		else if (p>beta && (a.size()>=minSizeForAntifriendship|| b.size()>=minSizeForAntifriendship)){
			res.result=-1;
		}
		else{
			res.result=0;
		}
		return res;
	}
	
	
	public AreModuleFriendsTestResult areModuleFriendsWilcoxonStouffer(
			HashSet<Integer> a, HashSet<Integer> b, float alpha,float beta) {
	
		countTests++;
		double stouferZ = getSumOfZScoresBVsA(a, b);
		stouferZ += getSumOfZScoresBVsA(b, a);
		double  k = a.size()+b.size();
		stouferZ /= Math.sqrt(k);
		
		double p = 1-TailProbs.probNormalC(stouferZ);
		
		AreModuleFriendsTestResult res = new AreModuleFriendsTestResult();
		res.p=p;
		if (p<=alpha){
			res.result=1;
		}
		else if (p>beta && (a.size()>=minSizeForAntifriendship || b.size()>=minSizeForAntifriendship)){
			res.result=-1;
		}
		else{
			res.result=0;
		}
		return res;
	}
	
	// alpha here is the minimal node percentage with positive value vs. the other module
	public AreModuleFriendsTestResult areModuleFriendsDICERSimpleTest(
			HashSet<Integer> a, HashSet<Integer> b) {
	
		AreModuleFriendsTestResult res = new AreModuleFriendsTestResult();
		res.result=-1;
		res.p=1;
		
		double score = GlobalImproveAlgorithm.getModuleVsModuleInGraph(a, b, g2);
		if (score>0){
			res.result=1;
			res.p=score;
		}	
		
		return res;
	}
	
	public static AreModuleFriendsTestResult areModuleFriendsDICERSimpleTest(
			HashSet<Integer> a, HashSet<Integer> b, float[][]g2) {
	
		AreModuleFriendsTestResult res = new AreModuleFriendsTestResult();
		res.result=-1;
		res.p=1;
		
		double score = GlobalImproveAlgorithm.getModuleVsModuleInGraph(a, b, g2);
		if (score>0){
			res.result=1;
			res.p=score;
		}	
		
		return res;
	}

	public double getSumOfZScoresBVsA(HashSet<Integer> a, HashSet<Integer> b) {
		double stouferZ=0;
		for (Iterator<Integer> iterator = b.iterator(); iterator.hasNext();) {
			Integer inB = (Integer) iterator.next();
			
			LinkedList<Float> vsAScores = new LinkedList<Float>();
			LinkedList<Float> vsCompScores = new LinkedList<Float>();
			
			for (int i = 0; i < g2.length; i++) {
				if (i==inB){
					continue;
				}
				if (a.contains(i)){
					vsAScores.add(g2[inB][i]);
				}
				else{
					vsCompScores.add(g2[inB][i]);
				}
			}
			
			double Z = getWilcoxonTestZscoreApprox(vsAScores, vsCompScores);
			stouferZ += Z;
			
		}
		return stouferZ;
	}
	
	

	public double getNodeVsModuleHyperGeometricP(int node, HashSet<Integer> module) {
		
		int numSuccess = g2Neis.get(node).size();
		if (numSuccess<2){
			return 1;
		}
		int sampleSize = module.size();
		int numSuccessInSample=0;
		for (Iterator<Integer> iterator = module.iterator(); iterator.hasNext();) {
			Integer inModule = (Integer) iterator.next();
			if (g2[node][inModule]>edgeThr){
				numSuccessInSample++;
			}
		}
		
		
		int bgSize = g2.length;
		
		double hyperGeomP = 1 - TailProbs.probHyperGeometricC(bgSize,
				sampleSize, numSuccess, numSuccessInSample);

		return hyperGeomP;
	}
	
	public static double getWilcoxonTestZscoreApprox(Collection<Float> a, Collection<Float> b){
		
		ArrayList<Float> allScores = new ArrayList<Float>(a);
		allScores.addAll(b);
		
		Collections.sort(allScores);
		
		float[] ranks = new float[allScores.size()];
		
		float currRank = 1;
		int i=0;
		while(i < allScores.size()) {
			
			int j = i;
			while (j<allScores.size() && Math.abs(allScores.get(i)-allScores.get(j))<1E-50){
				j++;
			}		
			float currSumRanks = 0;
			for (int k = i; k < j; k++) {
				currSumRanks+=currRank+(k-i);
			}
			currSumRanks /= (j-i);
			for (int k = i; k < j; k++) {
				ranks[k]=currSumRanks;
			}
			currRank += (j-i);
			i=j;
		}
		
		Hashtable<Float, Float> score2rank = new Hashtable<Float, Float>();
		
		for (int j = 0; j < ranks.length; j++) {
			score2rank.put(allScores.get(j), ranks[j]);
//			System.out.println(allScores.get(j)+"\t"+ranks[j]);
		}
		
		float aSumOfRanks = 0;
		for (Iterator<Float> iterator = a.iterator(); iterator.hasNext();) {
			Float inA = (Float) iterator.next();
			aSumOfRanks+=score2rank.get(inA);
		}
//		System.out.println("A sum of ranks:\t"+aSumOfRanks);
		
		float bSumOfRanks = 0;
		for (Iterator<Float> iterator = b.iterator(); iterator.hasNext();) {
			Float inB = (Float) iterator.next();
			bSumOfRanks+=score2rank.get(inB);
		}
		
//		System.out.println("B sum of ranks:\t"+bSumOfRanks);
		
		float n1 = a.size();
		// for two-sided test, compute uA and take the minimum
		float uA = aSumOfRanks - (n1*(n1+1)/2); 	
		float n2 = b.size();
//		float uB = bSumOfRanks - (n2*(n2+1)/2);
//		System.out.println("uA+uB:\t" + (uA+uB));
		double U = uA;
		double mean = n1*n2/2;
		double sd = Math.sqrt((n1*n2*(n1+n2+1)/12f));
		double Z = (U-mean)/sd;
//		System.out.println(Z);
//		double p = 2*TailProbs.probNormalC(Z);
//		System.out.println(p);
		
		return Z;
	}
	
	public static void main(String[] args) {
		
		// Test1
		System.out.println("#### Test 1");
		LinkedList<Float> A = new LinkedList<Float>();
		LinkedList<Float> B = new LinkedList<Float>();
		
		A.add(0f);
		A.add(2f);
		A.add(5f);
		A.add(6f);
		A.add(8f);
		A.add(11f);
		A.add(11f);
		A.add(14f);
		
		B.add(1.0f);
		B.add(2.0f);
		B.add(6.0f);
		B.add(6.0f);
		B.add(7.0f);
		B.add(10.0f);
		B.add(11.0f);
		B.add(11.0f);
		
		StatisticalFriendsTester f = new StatisticalFriendsTester(null,null);
		System.out.println(f.getWilcoxonTestZscoreApprox(A, B));
		
		// Test2
		System.out.println("#### Test 2");
		A = new LinkedList<Float>();
		B = new LinkedList<Float>();
		
		A.add(0.44f);
		A.add(0.45f);
		A.add(0.52f);
		A.add(0.53f);
		A.add(0.56f);
		A.add(0.58f);
		A.add(0.2f);
		A.add(0.1f);
		A.add(0.3f);
		
		B.add(0.45f);
		B.add(0.5f);
		B.add(0.61f);
		B.add(0.63f);
		B.add(0.75f);
		B.add(0.85f);
		B.add(0.93f);
		
		double z = f.getWilcoxonTestZscoreApprox(A, B);
		System.out.println(z);
		double p = 1-TailProbs.probNormalC(z);
		System.out.println(p);
		
		// Test2
		System.out.println("#### Test 2");
		A = new LinkedList<Float>();
		B = new LinkedList<Float>();
		
		A.add(0.44f);
		A.add(0.45f);
		A.add(0.52f);
		A.add(0.53f);
		A.add(0.56f);
		A.add(0.58f);
		A.add(0.2f);
		A.add(0.1f);
		A.add(0.3f);
		
		B.add(0.05f);
		B.add(0.05f);
		B.add(0.061f);
		B.add(0.063f);
		B.add(0.075f);
		B.add(0.085f);
		B.add(0.093f);
		
		z = f.getWilcoxonTestZscoreApprox(A, B);
		System.out.println(z);
		p = 1-TailProbs.probNormalC(z);
		System.out.println(p);
	}
}
