package graph_algorithms;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;

import math_utils.SimilarityMatrix;

public class BiModuleFinderUsingBicliqueAlgo extends MetaModuleFinder {

	private int MAX_BICS=50000;
	int sleepTime = 1000;
	int maxNumIterations = 600*60;
	int max_size = 7;
	int min_size = 5;
	private String FPMBC_path = "Z:\\davidama\\DICER_analysis\\dicer_2013\\fp_mbc\\FP-MBC.exe";
	String data_path = "tmp_for_fp_mbc.txt";
	String out_path = "fp_mbc_results.txt";

	public BiModuleFinderUsingBicliqueAlgo(float[][] c, float[][] dc,
			ArrayList<String> ids) {
		super(c, dc, ids);		
	}
	
	public BiModuleFinderUsingBicliqueAlgo(float[][] c, float[][] dc,
			ArrayList<String> ids, int min_s) {
		super(c, dc, ids);		
		min_size = min_s;
	}
	
	public void addNumToTmpFiles(int num){
		data_path = "tmp_for_fp_mbc"+num+".txt";
		out_path = "fp_mbc_results"+num+".txt";
	}
	
	public LinkedList<HashSet<Integer>> getFPMBCResults(String in_path,String out_path,
			int minSize) {

		// FP-MBC parameters
		String data_type = "" + 1;
		String sup_thr = "" + 2*minSize;
		String size_thr = "" + minSize;

		LinkedList<HashSet<Integer>> mcl_clusters = new LinkedList<HashSet<Integer>>();

		try {
			String[] cmd = { FPMBC_path, in_path, data_type, sup_thr,
					size_thr, out_path };
			Process p = Runtime.getRuntime().exec(cmd);
			int countIters=0;
			while (true) {
				countIters++;
				if (countIters>maxNumIterations){
//					System.out.println(size_thr+"\ttoo many iterations");
					p.destroy();
					break;
				}
				Thread.currentThread().sleep(sleepTime);
				BufferedReader reader = null;
				try{
					reader = new BufferedReader(new FileReader(
							out_path));
				} catch(IOException e){
//					System.out.println(size_thr+"\tout file does not exists");
					continue;
				}
				String line = "";
				int count = 0;
				while ((line = reader.readLine()) != null && count < 2 * MAX_BICS ) {
					count++;
				}
				reader.close();
//				System.out.println("num bics: " + count);
				if (count >= 2 * MAX_BICS) {
//					System.out.println(size_thr+"\ttoo many bics, destroy!");
					p.destroy();
					break;
				}

				try {
					p.exitValue();
//					System.out.println(size_thr+"\tprocess ended naturally");
					break;
				} catch (Exception e) {
					continue;
				}
			}
			// p.waitFor();

			BufferedReader reader = new BufferedReader(new FileReader(out_path));

			String line = "";
			while ((line = reader.readLine()) != null) {
				String[] l = line.split(" ");
				HashSet<Integer> cluster = new HashSet<Integer>();
				for (int i = 1; i < l.length; i++) {
					try {
						cluster.add(Integer.parseInt(l[i]));
					} catch (Exception e) {
						// System.out.println("not a number: " + l[i]);
					}
				}
				mcl_clusters.add(cluster);
				
				if (mcl_clusters.size() > MAX_BICS){
					break;
				}

			}

			return mcl_clusters;

		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return null;
	}

	public LinkedList<HashSet<Integer>> getFPMBCResultsAfterCleanup(
			String data_path, String out_path) {
		
		// create the similarity matrix
		SimilarityMatrix sim = new SimilarityMatrix();
		sim.similarities = new float[consistentCorrelationsGraph.length][consistentCorrelationsGraph.length];
		for (int i = 0; i < sim.similarities.length; i++) {
			for (int j = 0; j < sim.similarities[0].length; j++) {
				sim.similarities[i][j] = DC_graph[i][j];
			}
		}
		sim.setHeader(sim.similarities.length);
		LinkedList<HashSet<Integer>> mbc_results = new LinkedList<HashSet<Integer>>();
		for (int size = max_size; size >= min_size; size -= 1) {
			while(true){
				double curr_threshold = likelihoodThresholdForDC;
				// System.out.println("curr threshold: " + curr_threshold);
				try {
					sim.print_FP_MBC_Input(data_path, curr_threshold);
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
					continue;
				}

				LinkedList<HashSet<Integer>> curr_results = getFPMBCResults(data_path,out_path, size);
				if (curr_results.size()==0){
					break;
				}
				ArrayList<LinkedList<HashSet<Integer>>> curr_bicliques = mbc_results_to_pairs(curr_results);
				eraseEdgesBySolution(sim, mbc_results,curr_bicliques);
			}
		}
//		System.out.println("total num of bics: " + mbc_results.size());
		return mbc_results;
	}
	
	@Override
	public void findSeedPairs(int minSeedSize) {
		
		modules = new LinkedList<HashSet<Integer>>();
		meta_modules = new LinkedList<MetaModule>();

		wasUsed = new boolean[DC_graph.length];
		Arrays.fill(wasUsed, false);

		// step 1, write the consistent correlations graph to a file
		
		createInputForBicliquesAlgo(data_path);
		LinkedList<HashSet<Integer>> mcl_clusters = getFPMBCResultsAfterCleanup(data_path,out_path);
		ArrayList<LinkedList<HashSet<Integer>>> bicliques = mbc_results_to_pairs(mcl_clusters);
		
//		System.out.println("num bics: " + bicliques.size());

		boolean[] wasUsed = new boolean[bicliques.size()];
		Arrays.fill(wasUsed, false);

		int iterations_counter = -1;
		while (iterations_counter < 1000000) {
//			System.out.println("iteration "+iterations_counter);
//			checkFuziness();
			iterations_counter++;

			int max_size = 0;
			int max_index = -1;
			for (int i = 0; i < wasUsed.length; i++) {
				if (wasUsed[i]) {
					continue;
				}
				int curr_size = 0;
				curr_size += bicliques.get(i).get(0).size();
				curr_size += bicliques.get(i).get(1).size();
				double set1Size = bicliques.get(i).get(0).size();
				double set2Size = bicliques.get(i).get(0).size();
				double ratio = Math.min(set1Size / set2Size, set2Size
						/ set1Size);

				if (curr_size > max_size && ratio >= sizesConsistencyThr) {
					max_size = curr_size;
					max_index = i;
				}
				else if (ratio < sizesConsistencyThr || curr_size < 2*minSeedSize) {
					wasUsed[i] = true;
				}
			}
			
			if (max_index == -1) {
				break;
			}

			HashSet<Integer> u = bicliques.get(max_index).get(0);
			HashSet<Integer> v = bicliques.get(max_index).get(1);
//			checkIntersectionExists(u,v);
			wasUsed[max_index] = true;

			greedyRemoveNodesFromTwoSides(u, v);

			if (u.size() < minSeedSize || v.size() < minSeedSize) {
				continue;
			}

			double s1 = u.size();
			double s2 = v.size();
			if (s1 >= minSeedSize && s2 >= minSeedSize
					&& ((Math.min(s1 / s2, s2 / s1)) >= sizesConsistencyThr)) {
				
				modules.add(new HashSet<Integer>(u));
//				checkFuziness();
				modules.add(new HashSet<Integer>(v));
//				checkFuziness();
				
				MetaModule currPair = new MetaModule(new HashSet<Integer>(u),
						new HashSet<Integer>(v), this);
				meta_modules.add(currPair);

				// go over all covered genes and mark as used
				markAsUsed(u);
				markAsUsed(v);

				// remove current MM genes from all other bicliques
				HashSet<Integer> mm_genes = new HashSet<Integer>();
				mm_genes.addAll(u);
				mm_genes.addAll(v);

				for (int i = 0; i < wasUsed.length; i++) {
					if (wasUsed[i]) {
						continue;
					}
					HashSet<Integer> set1 = bicliques.get(i).get(0);
					HashSet<Integer> set2 = bicliques.get(i).get(1);
					for (Iterator<Integer> iterator = mm_genes.iterator(); iterator
							.hasNext();) {
						Integer gene = (Integer) iterator.next();
						if (set1.contains(gene)) {
							set1.remove(gene);
						}
						if (set2.contains(gene)) {
							set2.remove(gene);
						}
					}
				}

			}
			
//			System.out.println("end of iteration");
//			checkFuziness();
		}
		
		System.out.println(modules.size());
	}

	public ArrayList<LinkedList<HashSet<Integer>>> mbc_results_to_pairs(
			LinkedList<HashSet<Integer>> mcl_clusters) {
		ArrayList<LinkedList<HashSet<Integer>>> bicliques = new ArrayList<LinkedList<HashSet<Integer>>>();

		for (Iterator<HashSet<Integer>> iterator = mcl_clusters.iterator(); iterator
				.hasNext();) {
			HashSet<Integer> set1 = (HashSet<Integer>) iterator.next();
			HashSet<Integer> set2 = null;
			if (iterator.hasNext()) {
				set2 = (HashSet<Integer>) iterator.next();
			} else {
				break;
			}

			LinkedList<HashSet<Integer>> pair = new LinkedList<HashSet<Integer>>();
			if (checkIntersectionExists(set1, set2)){
				System.err.println("solver output contains intersection of biclique sides");
				System.err.println("Check its input for details");
				continue;
			}
			pair.add(set1);
			pair.add(set2);
			bicliques.add(pair);
		}

		// System.out.println("num initial bicliques: " + bicliques.size());
		return bicliques;
	}

	public String createInputForBicliquesAlgo(String data_path) {

		SimilarityMatrix sim = new SimilarityMatrix();
		sim.similarities = new float[consistentCorrelationsGraph.length][consistentCorrelationsGraph.length];
		for (int i = 0; i < sim.similarities.length; i++) {
			for (int j = 0; j < sim.similarities[0].length; j++) {
				sim.similarities[i][j] = (new Double(DC_graph[i][j]))
						.floatValue();
			}
		}

		sim.setHeader(sim.similarities.length);
		
		try {
			sim.print_FP_MBC_Input(data_path, 0);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return data_path;
	}
	
	public int eraseEdgesBySolution(SimilarityMatrix sim,
			LinkedList<HashSet<Integer>> mbc_results,
			ArrayList<LinkedList<HashSet<Integer>>> curr_bicliques) {
		int numErased = 0;
		for (Iterator<LinkedList<HashSet<Integer>>> iterator = curr_bicliques
				.iterator(); iterator.hasNext();) {
			LinkedList<HashSet<Integer>> bic = (LinkedList<HashSet<Integer>>) iterator
					.next();
			HashSet<Integer> u = bic.get(0);
			HashSet<Integer> v = bic.get(1);
			mbc_results.add(u);
			mbc_results.add(v);

			for (Iterator<Integer> iterator2 = v.iterator(); iterator2
					.hasNext();) {
				Integer inV = (Integer) iterator2.next();
				for (Iterator<Integer> iterator3 = u.iterator(); iterator3
						.hasNext();) {
					Integer inU = (Integer) iterator3.next();
					if (sim.similarities[inU][inV] > likelihoodThresholdForDC) {
						numErased++;
					}

					sim.similarities[inU][inV] = -Float.MAX_VALUE;
					sim.similarities[inV][inU] = -Float.MAX_VALUE;
				}
			}
		}
		return numErased;
	}
	
	/**
	 * This methods receives as input two distinct sets. The set elements
	 * represent the index of a gene in the matrices. The algorithm greedily
	 * tries to remove a gene from T or S until the remaining sets satisfy the
	 * meta-module definition.
	 * 
	 * This method overrides the original method.
	 * This implementation ignores the between score.
	 * This is because we start with a biclique and thus
	 * the between scores are always positive.
	 * 
	 * @param T
	 * @param S
	 * @return
	 */
	@Override
	public boolean greedyRemoveNodesFromTwoSides(Collection<Integer> T,
			Collection<Integer> S) {

		
		Hashtable<Integer, Double> inTNode2WithinScore = getWithinSetScores(T);
		Hashtable<Integer, Double> inSNode2WithinScore = getWithinSetScores(S);

		while (true) {

			if (T.size() <= minSeedModuleSize || S.size() <= minSeedModuleSize) {
				return false;
			}

			double minScore = Double.MAX_VALUE;
			int minCombScoreIndex = -1;
		

			for (Iterator<Integer> iterator = T.iterator(); iterator.hasNext();) {
				Integer node = (Integer) iterator.next();
				if (inTNode2WithinScore.get(node)  < minScore) {
					minScore = inTNode2WithinScore.get(node);
					minCombScoreIndex = node;
				}
			}

			for (Iterator<Integer> iterator = S.iterator(); iterator.hasNext();) {
				Integer node = (Integer) iterator.next();
				if (inSNode2WithinScore.get(node) < minScore) {
					minScore = inSNode2WithinScore.get(node);
					minCombScoreIndex = node;
				}
			}

			if (minScore > likelihoodThresholdForCG
					|| minCombScoreIndex==-1 ) {
				return true;
			}

			int minNodeSelected = minCombScoreIndex;
			if (T.contains(minNodeSelected)) {

				inTNode2WithinScore.remove(minNodeSelected);
				T.remove(minNodeSelected);

				for (Iterator<Integer> iterator2 = T.iterator(); iterator2
						.hasNext();) {
					Integer inT = (Integer) iterator2.next();
					double score = inTNode2WithinScore.get(inT);
					score -= consistentCorrelationsGraph[inT][minNodeSelected];
					inTNode2WithinScore.put(inT, score);
				}

			} else if (S.contains(minNodeSelected)) {

				inSNode2WithinScore.remove(minNodeSelected);
				S.remove(minNodeSelected);

				for (Iterator<Integer> iterator2 = S.iterator(); iterator2
						.hasNext();) {
					Integer inS = (Integer) iterator2.next();
					double score = inSNode2WithinScore.get(inS);
					score -= consistentCorrelationsGraph[inS][minNodeSelected];
					inSNode2WithinScore.put(inS, score);
				}
			}
		}
	}
	
	public void checkFuziness() {
		boolean[] coverage = new boolean[DC_graph.length];
		Arrays.fill(coverage,false);
		for (Iterator<HashSet<Integer>> iterator = modules.iterator(); iterator.hasNext();) {
			HashSet<Integer> m = (HashSet<Integer>) iterator.next();
			for (Iterator<Integer> iterator2 = m.iterator(); iterator2.hasNext();) {
				Integer inM = (Integer) iterator2.next();
				if (coverage[inM]){
					System.err.println("Error, the solution is fuzzy");
					throw (new RuntimeException());
				}
				coverage[inM]=true;
			}
		}
	}
	
	public boolean checkIntersectionExists(Collection<Integer> a, Collection<Integer> b){
		for (Iterator<Integer> iterator = b.iterator(); iterator.hasNext();) {
			Integer inB = (Integer) iterator.next();
			if (a.contains(inB)){
				return true;
			}
		}
		return false;
	}


}
