package graph_algorithms;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import math_utils.SimilarityMatrix;

public class BiModulesFinderUsingClusteringSol extends MetaModuleFinder {
	
	List<HashSet<Integer>> clusters=null;

	public BiModulesFinderUsingClusteringSol(float[][] c, float[][] dc,
			ArrayList<String> ids) {
		super(c, dc, ids);
		// TODO Auto-generated constructor stub
	}

	public LinkedList<HashSet<Integer>> getMCLClustersByThreshold(
			String tmp_file_for_mcl, float threshold) {

		LinkedList<HashSet<Integer>> mcl_clusters = new LinkedList<HashSet<Integer>>();
		String mcl_pat = "/home/gaga/davidama/mcl-latest.tar/bin/mcl";
		String mcl_results_path = "test.txt";
		try {
			String[] mcl_cmd = { mcl_pat, tmp_file_for_mcl, "--abc", "-I",
					"" + threshold, "-o", mcl_results_path };
			Process p = Runtime.getRuntime().exec(mcl_cmd);
			p.waitFor();

			BufferedReader reader = new BufferedReader(new FileReader(
					mcl_results_path));

			String line = "";
			while ((line = reader.readLine()) != null) {
				String[] l = line.split("\t");
				HashSet<Integer> cluster = new HashSet<Integer>();
				for (int i = 0; i < l.length; i++) {
					try {
						cluster.add(Integer.parseInt(l[i]));
					} catch (Exception e) {
						// System.out.println("not a number: " + l[i]);
					}
				}
				if (cluster.size() > 2)
					mcl_clusters.add(cluster);
			}
			System.out.println("num clusters in mcl: " + mcl_clusters.size());
			return mcl_clusters;

		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	public LinkedList<HashSet<Integer>> getHierarchicalClustering() {
		return HierarchicalClusterer.doCompleteHierarchicalClustering(
				likelihoodThresholdForCG, consistentCorrelationsGraph,
				minSeedModuleSize);
	}

	@Override
	public void findSeedPairs(int minSeedSize) {

		modules = new LinkedList<HashSet<Integer>>();
		meta_modules = new LinkedList<MetaModule>();

		wasUsed = new boolean[DC_graph.length];
		Arrays.fill(wasUsed, false);

		if (clusters==null){
			clusters = getHierarchicalClustering();
		} 
		
		for (Iterator<HashSet<Integer>> iterator = clusters.iterator(); iterator.hasNext();) {
			HashSet<Integer> clust = (HashSet<Integer>) iterator.next();
			if (clust==null){
				System.out.println("null cluster");
				return;
			}
		}

		// go over cluster pairs
		// calculate the average dc between
		double[][] betweenClustersScores = new double[clusters.size()][clusters
				.size()];
		boolean[] isCovered = new boolean[clusters.size()];
		Arrays.fill(isCovered, false);
		int first_pointer = 0;
		for (Iterator<HashSet<Integer>> iterator = clusters.iterator(); iterator
				.hasNext();) {
			HashSet<Integer> firstCluster = (HashSet<Integer>) iterator.next();

			int second_pointer = 0;
			for (Iterator<HashSet<Integer>> iterator2 = clusters.iterator(); iterator2
					.hasNext();) {
				HashSet<Integer> second_cluster = (HashSet<Integer>) iterator2
						.next();

				if (second_pointer == first_pointer) {
					break;
				}

				MetaModule currMM = new MetaModule(firstCluster,
						second_cluster, this);
				betweenClustersScores[first_pointer][second_pointer] = currMM.betweenAvgLLRScore;
				betweenClustersScores[second_pointer][first_pointer] = currMM.betweenAvgLLRScore;
				second_pointer++;
			}

			first_pointer++;
		}

		int numClusters = clusters.size();
		while (numClusters > 0) {
			int max_first_i = -1;
			int max_second_i = -1;
			double max_score = likelihoodThresholdForDC;
			for (int i = 0; i < isCovered.length; i++) {
				if (isCovered[i]) {continue;}
				for (int j = 0; j < i; j++) {
					if (isCovered[j]) {continue;}
					if (betweenClustersScores[i][j] > max_score) {
						max_score = betweenClustersScores[i][j];
						max_first_i = i;
						max_second_i = j;
					}
				}
			}
			if (max_first_i == -1) {
				break;
			}

			HashSet<Integer> u = clusters.get(max_first_i);
			HashSet<Integer> v = clusters.get(max_second_i);

			greedyRemoveNodesFromTwoSides(u, v);

			if (u.size() < minSeedSize || v.size() < minSeedSize) {
				betweenClustersScores[max_first_i][max_second_i] = -Double.MAX_VALUE;
				betweenClustersScores[max_second_i][max_first_i] = -Double.MAX_VALUE;
				continue;
			}

			double s1 = u.size();
			double s2 = v.size();
			if (s1 >= minSeedSize && s2 >= minSeedSize
					&& ((Math.min(s1 / s2, s2 / s1)) >= sizesConsistencyThr)) {

				modules.add(new HashSet<Integer>(u));
				modules.add(new HashSet<Integer>(v));
				MetaModule currPair = new MetaModule(new HashSet<Integer>(u),
						new HashSet<Integer>(v), this);
				meta_modules.add(currPair);

				// go over all covered genes and mark as used
				isCovered[max_first_i] = true;
				isCovered[max_second_i] = true;
			} else {
				betweenClustersScores[max_first_i][max_second_i] = -Double.MAX_VALUE;
				betweenClustersScores[max_second_i][max_first_i] = -Double.MAX_VALUE;
			}

		}
	}

	public LinkedList<HashSet<Integer>> getMCLSol() {
		// step 1, write the consistent correlations graph to a file
		SimilarityMatrix sim = new SimilarityMatrix();
		sim.similarities = new float[consistentCorrelationsGraph.length][consistentCorrelationsGraph.length];
		for (int i = 0; i < sim.similarities.length; i++) {
			for (int j = 0; j < sim.similarities[0].length; j++) {
				sim.similarities[i][j] = (new Double(
						consistentCorrelationsGraph[i][j])).floatValue();
			}
		}

		sim.setHeader(sim.similarities.length);
		String tmp_file_for_mcl = "tmp.txt";
		try {
			sim.printMCLInput(tmp_file_for_mcl, 0);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		float min_val = 1.5f;
		float max_val = 3f;
		float epsilon = 0.05f;
		ArrayList<LinkedList<HashSet<Integer>>> solutions = new ArrayList<LinkedList<HashSet<Integer>>>();
		int max_num_clusters = 0;
		int max_num_index = 0;
		int pointer = 0;
		for (float thr = min_val; thr <= max_val; thr += epsilon) {
			LinkedList<HashSet<Integer>> curr_sol = getMCLClustersByThreshold(
					tmp_file_for_mcl, thr);
			solutions.add(curr_sol);
			if (curr_sol.size() > max_num_clusters) {
				max_num_clusters = curr_sol.size();
				max_num_index = pointer;
			}
			pointer++;
		}
		LinkedList<HashSet<Integer>> mcl_clusters = solutions
				.get(max_num_index);
		return mcl_clusters;
	}
	
//	public void setClusters(LinkedList<HashSet<Integer>> clusters) {
//		this.clusters = clusters;
//	}
	
	public void setClusters(List<HashSet<Integer>> clusters) {
		this.clusters = clusters;
	}

}
