package graph_algorithms;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;

public class HierarchicalClusterer {
	
	public static LinkedList<HashSet<Integer>> doAvgHierarchicalClustering(
			double threshold,float[][] weightedGraph, int minClusterSize) {

		int n = weightedGraph.length;
		boolean[] isSeed = new boolean[n];
		Arrays.fill(isSeed, true);

		Hashtable<Integer, HashSet<Integer>> listIndexInArray2components = new Hashtable<Integer, HashSet<Integer>>();
		double[][] simMat = new double[n][n];
		for (int i = 0; i < simMat.length; i++) {
			HashSet<Integer> l = new HashSet<Integer>();
			l.add(i);
			listIndexInArray2components.put(i, l);
			for (int j = 0; j < simMat.length; j++) {
				simMat[i][j] = weightedGraph[i][j];
			}
		}

		while (true) {

			// get the next best pair
			ArrayList<Integer> currSeedsIndices = new ArrayList<Integer>(
					listIndexInArray2components.keySet());

			double maxCon = -Double.MAX_VALUE;
			int first = -1;
			int second = -1;

			for (int i = 0; i < currSeedsIndices.size(); i++) {
				int curr1 = (Integer) currSeedsIndices.get(i);
				for (int j = 0; j < i; j++) {
					int curr2 = (Integer) currSeedsIndices.get(j);
					if (simMat[curr1][curr2] > maxCon) {
						maxCon = simMat[curr1][curr2];
						first = curr1;
						second = curr2;
					}
				}
			}

//			System.out.println("maximal connectivity: " + maxCon);

			if (maxCon < threshold || currSeedsIndices.size() == 1) {
				break;
			}

			// update the data
			// start with the matrix
			// always merge to the first
			double sizeFirst = listIndexInArray2components.get(first).size();
			double sizeScond = listIndexInArray2components.get(second).size();
			for (int i = 0; i < currSeedsIndices.size(); i++) {
				int curr1 = (Integer) currSeedsIndices.get(i);
				if (curr1 == first || curr1 == second) {
					continue;
				}

				double newScore = 0;
				double sizeCurr = listIndexInArray2components.get(curr1).size();
				newScore += (sizeFirst * sizeCurr * simMat[curr1][first] + sizeScond
						* sizeCurr * simMat[curr1][second]);
				newScore /= ((sizeFirst + sizeScond) * sizeCurr);
				simMat[curr1][first] = newScore;
				simMat[first][curr1] = simMat[curr1][first];
			}

			// update the lists
			HashSet<Integer> secondList = listIndexInArray2components
					.get(second);
			HashSet<Integer> firstList = listIndexInArray2components
					.get(first);
			firstList.addAll(secondList);

			// remove the second list from the pool
			listIndexInArray2components.remove(second);
		}

		LinkedList<HashSet<Integer>> finalClusters = new LinkedList<HashSet<Integer>>();
		for (Iterator<Integer> iterator = listIndexInArray2components.keySet()
				.iterator(); iterator.hasNext();) {
			Integer listIndex = (Integer) iterator.next();
			HashSet<Integer> clusterByIndices = listIndexInArray2components
					.get(listIndex);

			if (clusterByIndices.size() < minClusterSize) {
				continue;
			}
			finalClusters.add(clusterByIndices);
		}
		
//		System.out.println("hierarchical clustering detected: " + finalClusters.size());
		return finalClusters;

	}
	
	public static LinkedList<HashSet<Integer>> doCompleteHierarchicalClustering(
			double threshold,float[][] weightedGraph, int minClusterSize) {

		int n = weightedGraph.length;
		boolean[] isSeed = new boolean[n];
		Arrays.fill(isSeed, true);

		Hashtable<Integer, HashSet<Integer>> listIndexInArray2components = new Hashtable<Integer, HashSet<Integer>>();
		double[][] simMat = new double[n][n];
		for (int i = 0; i < simMat.length; i++) {
			HashSet<Integer> l = new HashSet<Integer>();
			l.add(i);
			listIndexInArray2components.put(i, l);
			for (int j = 0; j < simMat.length; j++) {
				simMat[i][j] = weightedGraph[i][j];
			}
		}

		while (true) {

			// get the next best pair
			ArrayList<Integer> currSeedsIndices = new ArrayList<Integer>(
					listIndexInArray2components.keySet());

			double maxCon = -Double.MAX_VALUE;
			int first = -1;
			int second = -1;

			for (int i = 0; i < currSeedsIndices.size(); i++) {
				int curr1 = (Integer) currSeedsIndices.get(i);
				for (int j = 0; j < i; j++) {
					int curr2 = (Integer) currSeedsIndices.get(j);
					if (simMat[curr1][curr2] > maxCon) {
						maxCon = simMat[curr1][curr2];
						first = curr1;
						second = curr2;
					}
				}
			}

//			System.out.println("maximal connectivity: " + maxCon);

			if (maxCon < threshold || currSeedsIndices.size() == 1) {
				break;
			}

			// update the data
			// start with the matrix
			// always merge to the first
			for (int i = 0; i < currSeedsIndices.size(); i++) {
				int curr1 = (Integer) currSeedsIndices.get(i);
				if (curr1 == first || curr1 == second) {
					continue;
				}

				double newScore = Math.min(simMat[curr1][first], simMat[curr1][second]);
				simMat[curr1][first] = newScore;
				simMat[first][curr1] = simMat[curr1][first];
			}

			// update the lists
			HashSet<Integer> secondList = listIndexInArray2components
					.get(second);
			HashSet<Integer> firstList = listIndexInArray2components
					.get(first);
			firstList.addAll(secondList);

			// remove the second list from the pool
			listIndexInArray2components.remove(second);
		}

		LinkedList<HashSet<Integer>> finalClusters = new LinkedList<HashSet<Integer>>();
		for (Iterator<Integer> iterator = listIndexInArray2components.keySet()
				.iterator(); iterator.hasNext();) {
			Integer listIndex = (Integer) iterator.next();
			HashSet<Integer> clusterByIndices = listIndexInArray2components
					.get(listIndex);

			if (clusterByIndices.size() < minClusterSize) {
				continue;
			}
			finalClusters.add(clusterByIndices);
		}
		
//		System.out.println("hierarchical clustering detected: " + finalClusters.size());
		return finalClusters;

	}

}
