package graph_algorithms;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;

/**
 * 
 * This class implements the meta-module merge algorithm. DICER uses a simple
 * local search to get an initial set of small meta-modules. Here, we merge
 * theses initial meta-modules using a greedy algorithm.
 * 
 */

public class MetaModulesMerger {

	/* Members */
	MetaModule[] pairs;
	MetaModulePairMerger[][] pairsMergingMatrix;

	public MetaModulesMerger(Collection<MetaModule> a) {
		pairs = new MetaModule[a.size()];
		int c = 0;
		for (Iterator<MetaModule> iterator = a.iterator(); iterator.hasNext();) {
			MetaModule clusterPair = (MetaModule) iterator.next();
			pairs[c] = clusterPair;
			c++;
		}

		pairsMergingMatrix = new MetaModulePairMerger[pairs.length][pairs.length];
		for (int i = 0; i < pairsMergingMatrix.length; i++) {
			for (int j = 0; j < i; j++) {
				pairsMergingMatrix[i][j] = MetaModulePairMerger.merge(pairs[i],
						pairs[j]);
				pairsMergingMatrix[j][i] = pairsMergingMatrix[i][j];
			}
		}
	}

	public void doMerges() {
		boolean[] isPair = new boolean[pairs.length];
		Arrays.fill(isPair, true);

		while (true) {
			// System.out.println("in merge loop");
			int bestI = -1;
			int bestJ = -1;
			double bestScore = -Double.MAX_VALUE;
			for (int i = 0; i < pairsMergingMatrix.length; i++) {
				if (!isPair[i]) {
					continue;
				}
				for (int j = 0; j < i; j++) {
					if (!isPair[j]) {
						continue;
					}
					if (pairsMergingMatrix[i][j] != null
							&& pairsMergingMatrix[i][j].newPair != null) {
						if (bestScore < pairsMergingMatrix[i][j].averageBetweenAddedDCScore) {
							bestScore = pairsMergingMatrix[i][j].averageBetweenAddedDCScore;
							bestI = i;
							bestJ = j;
						}
					}
				}
			}
			if (bestI == -1) {
				break;
			}

			// do merge
			pairs[bestI] = pairsMergingMatrix[bestI][bestJ].newPair;
			isPair[bestJ] = false;
			// update data
			for (int i = 0; i < isPair.length; i++) {
				if (!isPair[i] || i == bestI) {
					continue;
				}
				pairsMergingMatrix[i][bestI] = MetaModulePairMerger.merge(
						pairs[i], pairs[bestI]);
				pairsMergingMatrix[bestI][i] = pairsMergingMatrix[i][bestI];
			}
		}

		// update the pairs list
		int n = 0;
		for (int i = 0; i < isPair.length; i++) {
			if (isPair[i]
					&& pairs[i].U.size() >= MetaModuleFinder.minSeedModuleSize
					&& pairs[i].V.size() >= MetaModuleFinder.minSeedModuleSize) {
				n++;
			}
		}
		MetaModule[] newArr = new MetaModule[n];
		n = 0;
		for (int i = 0; i < isPair.length; i++) {
			if (isPair[i]
					&& pairs[i].U.size() >= MetaModuleFinder.minSeedModuleSize
					&& pairs[i].V.size() >= MetaModuleFinder.minSeedModuleSize) {
				newArr[n] = pairs[i];
				newArr[n].updatePairScore();
				n++;
			}
		}

		pairs = newArr;
	}

	public LinkedList<HashSet<Integer>> getModulesList() {
		LinkedList<HashSet<Integer>> r = new LinkedList<HashSet<Integer>>();
		for (int i = 0; i < pairs.length; i++) {
			r.add(pairs[i].U);
			r.add(pairs[i].V);
		}
		return r;
	}

	public LinkedList<LinkedList<Integer>> getMetaModules() {
		LinkedList<LinkedList<Integer>> r = new LinkedList<LinkedList<Integer>>();
		for (int i = 0; i < pairs.length; i++) {
			LinkedList<Integer> indices = new LinkedList<Integer>();
			indices.add(2 * i);
			indices.add(2 * i + 1);
			r.add(indices);
		}
		return r;
	}
}
