package graph_algorithms;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;

public class SingletonsAndMetaModulesMerger {
	MetaModule[] pairs;
	ArrayList<Integer> singletons;
	double[][] sing2pairsDCScores;
	double[][] sing2pairsGCScores;
	boolean[][] uOrV;
	MetaModuleFinder data;
	
	public SingletonsAndMetaModulesMerger(Collection<MetaModule> a,ArrayList<Integer> sigs,MetaModuleFinder d) {
		data=d;
		singletons=sigs;
		pairs = new MetaModule[a.size()];
		int c = 0;
		for (Iterator<MetaModule> iterator = a.iterator(); iterator.hasNext();) {
			MetaModule clusterPair = (MetaModule) iterator.next();
			pairs[c]=clusterPair;
			c++;
		}
		
		int n=sigs.size();
		int m = pairs.length;
		sing2pairsDCScores = new double[n][m];
		sing2pairsGCScores= new double [n][m];
		uOrV = new boolean[n][m];
	}

	public void doMerges(double thr1,double thr2) {
		boolean[] isSingleton = new boolean[singletons.size()];
		Arrays.fill(isSingleton, true);

		for (int i = 0; i < isSingleton.length; i++) {
			if (!isSingleton[i]) {
				continue;
			}
			HashSet<Integer> sigHash = new HashSet<Integer>();
			sigHash.add(singletons.get(i));

			for (int j = 0; j < pairs.length; j++) {

				// try add singleton to U
				double betweenScore1 = MetaModulePairMerger.getBetweenDCScores(
						sigHash, pairs[j].U, data);
				double withinScore1 = MetaModulePairMerger.getBetweenGCScores(
						sigHash, pairs[j].V, data);
				double combined1 = withinScore1 + betweenScore1;

				// try add singleton to V
				double betweenScore2 = MetaModulePairMerger.getBetweenDCScores(
						sigHash, pairs[j].V, data);
				double withinScore2 = MetaModulePairMerger.getBetweenGCScores(
						sigHash, pairs[j].U, data);
				double combined2 = withinScore2 + betweenScore2;

				if ((betweenScore1 <= thr1 || withinScore1 <= thr2)
						&& (betweenScore2 <= thr1 || withinScore2 <= thr2)) {
					continue;
				}

				if (betweenScore1 > thr1 && withinScore1 > thr2
						&& combined1 > combined2) {
					sing2pairsDCScores[i][j] = betweenScore1;
					sing2pairsGCScores[i][j] = withinScore1;
					uOrV[i][j] = false;
				} else if ((betweenScore2 > thr1 && withinScore2 > thr2
						&& combined2 > combined1)) {
					sing2pairsDCScores[i][j] = betweenScore2;
					sing2pairsGCScores[i][j] = withinScore2;
					uOrV[i][j] = true;
				}
				else{
					continue;
				}
			}
		}

		// do merges
		while (true) {
			int bestSing = -1;
			int bestPair = -1;
			double maxScore = -Double.MAX_VALUE;

			for (int i = 0; i < isSingleton.length; i++) {
				if (!isSingleton[i]) {
					continue;
				}
				for (int j = 0; j < pairs.length; j++) {
					if (sing2pairsDCScores[i][j] > maxScore){
						maxScore=sing2pairsDCScores[i][j];
						bestPair=j;
						bestSing=i;
					}
				}
			}
			
			if (bestSing==-1){
				break;
			}
			if (maxScore <= 0){
				break;
			}
			
			HashSet<Integer> sigHash = new HashSet<Integer>();
			sigHash.add(singletons.get(bestSing));
			isSingleton[bestSing]=false;
//			System.out.println("adding a singleton");
			
			if (uOrV[bestSing][bestPair]){
				pairs[bestPair].U.add(singletons.get(bestSing));
				// the singleton is added to U
				// go over all other singletons and update the scores
				for (int i = 0; i < isSingleton.length; i++) {
					if (!isSingleton[i]) {
						continue;
					}
					// both are in U, only the GC score changes
					if (uOrV[i][bestPair]){
						sing2pairsGCScores[i][bestPair] += data.consistentCorrelationsGraph[singletons.get(i)][singletons.get(bestSing)];
					}
					// the best is in U and the current other is in V, the DC score changes
					else{
						sing2pairsDCScores[i][bestPair] += data.DC_graph[singletons.get(i)][singletons.get(bestSing)];
					}
					
					if (sing2pairsGCScores[i][bestPair]<=thr2 || sing2pairsDCScores[i][bestPair]<thr1){
						sing2pairsGCScores[i][bestPair]=0;
						sing2pairsDCScores[i][bestPair]=0;
					}
				}
			}
			else{
				pairs[bestPair].V.add(singletons.get(bestSing));
				// the singleton is added to V
				// go over all other singletons and update the scores
				for (int i = 0; i < isSingleton.length; i++) {
					if (!isSingleton[i]) {
						continue;
					}
					// both are in V, only the GC score changes
					if (! uOrV[i][bestPair]){
						sing2pairsGCScores[i][bestPair] += data.consistentCorrelationsGraph[singletons.get(i)][singletons.get(bestSing)];
					}
					// the best is in V and the current other is in U, the DC score changes
					else{
						sing2pairsDCScores[i][bestPair] += data.DC_graph[singletons.get(i)][singletons.get(bestSing)];
					}
					
					if (sing2pairsGCScores[i][bestPair]<= thr2 || sing2pairsDCScores[i][bestPair]<thr1){
						sing2pairsGCScores[i][bestPair]=0;
						sing2pairsDCScores[i][bestPair]=0;
					}
				}
				
			}
		}
		
		for (int i = 0; i < pairs.length; i++) {
			pairs[i].updatePairScore();
		}
	}
	
	public  LinkedList<HashSet<Integer>> getModulesList(){
		LinkedList<HashSet<Integer>> r = new LinkedList<HashSet<Integer>>();
		
		for (int i = 0; i < pairs.length; i++) {
			r.add(pairs[i].U);
			r.add(pairs[i].V);
		}
		
		return r;
	}
	
	public LinkedList<LinkedList<Integer>> getMetaModules(){
		LinkedList<LinkedList<Integer>> r = new LinkedList<LinkedList<Integer>>();
		for (int i = 0; i < pairs.length; i++) {
			LinkedList<Integer> indices = new LinkedList<Integer>();
			indices.add(2*i);
			indices.add(2*i + 1);
			r.add(indices);
		}
		
		return r;
	}
}
