package main_scripts;

import global_analysis.GreedyHillClimber;
import graph_algorithms.BiModuleFinderUsingBicliqueAlgo;
import graph_algorithms.BiModulesFinderUsingClusteringSol;
import graph_algorithms.HierarchicalClusterer;
import graph_algorithms.MetaModule;
import graph_algorithms.MetaModuleFinder;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

public class AlgorithmComparisonRunner {
	
	float[][] g1;
	float[][] g2;
	
	Hashtable<String,List<HashSet<Integer>>> algo2seed;
	Hashtable<String,List<HashSet<Integer>>> algo2modulesAfterImprovement;
	Hashtable<String,List<MetaModule>> algo2MMs_afterImprove;
	Hashtable<String,boolean[][]> algo2friends;
	Hashtable<String,short[][]> algo2CMPs;
	
	float edgethr = 1;
	
	// seeds detection algorithm name
	String[] seedDetectionNames = {"hierarchical","dicer","DicerStar","mbcDicer","globalNodeAddition"};
//	String[] seedDetectionNames = {"globalNodeAddition","DicerStar"};
	String[] improvmentAlgoNames = {"local","global"};
//	String[] improvmentAlgoNames = {"global"};
	
	ArrayList<String> nodeIds;
	
	public AlgorithmComparisonRunner(float[][] g1, float[][] g2,ArrayList<String> ids, float edgethr) {
		this.g1=g1;
		this.g2=g2;
		nodeIds=ids;
		this.edgethr=edgethr;
		
		algo2seed = new Hashtable<String, List<HashSet<Integer>>>();
		algo2modulesAfterImprovement = new Hashtable<String, List<HashSet<Integer>>>();
		algo2MMs_afterImprove = new Hashtable<String,List<MetaModule>>();
		algo2friends = new Hashtable<String,boolean[][]>();
		algo2CMPs = new Hashtable<String,short[][]>();
	}
	

	public void doComparisons(boolean disc) {
	
		/*
		 * Run seed detection algorithms
		 */
		List<HashSet<Integer>> hierSeeds = HierarchicalClusterer
				.doCompleteHierarchicalClustering(0, g1, 1);
		updateHashForSeedAlgorithm(seedDetectionNames[0], new LinkedList<HashSet<Integer>>(hierSeeds));
		System.out.println(getModuleStatistics(hierSeeds));
		System.out.flush();
		
		MetaModuleFinder dicerSeedsFinder = new MetaModuleFinder(g1, g2, nodeIds);
		dicerSeedsFinder.findSeedPairs(2);
		updateHashForSeedAlgorithm(seedDetectionNames[1], dicerSeedsFinder);
		System.out.println(getModuleStatistics(dicerSeedsFinder.getModulesCopy()));
		dicerSeedsFinder=null;
		Runtime.getRuntime().gc();
		System.out.flush();
////		
		MetaModuleFinder DicerStar = new MetaModuleFinder(g1, g2, nodeIds);
		DicerStar.findSeedPairs(5);
		updateHashForSeedAlgorithm("DicerStar", DicerStar);
		System.out.println(getModuleStatistics(DicerStar.getModulesCopy()));
		DicerStar=null;
		Runtime.getRuntime().gc();
		System.out.flush();
//		
		BiModuleFinderUsingBicliqueAlgo mbcDicerFinder = new BiModuleFinderUsingBicliqueAlgo(g1, g2, nodeIds);
		mbcDicerFinder.addNumToTmpFiles(1);
		mbcDicerFinder.findSeedPairs(5);
		updateHashForSeedAlgorithm(seedDetectionNames[3], mbcDicerFinder);
		System.out.println(getModuleStatistics(mbcDicerFinder.getModulesCopy()));
		mbcDicerFinder = null;
		Runtime.getRuntime().gc();
		System.out.flush();
		
		GreedyHillClimber globalAnalysis = new GreedyHillClimber(g1, g2,edgethr,disc);
		globalAnalysis.doNodeAdditionKeepCEmpty();
		List<HashSet<Integer>> nodeAdditionSeeds = new LinkedList<HashSet<Integer>>(globalAnalysis.getModulesCopy());
		updateHashForSeedAlgorithm("globalNodeAddition", nodeAdditionSeeds);
		System.out.println(getModuleStatistics(nodeAdditionSeeds));
		globalAnalysis = null;
		Runtime.getRuntime().gc();
		System.out.flush();
		
		
		/*
		 * Run improvement stages
		 */
	
		for (int i = 0; i < seedDetectionNames.length; i++) {
			String currAlgo = seedDetectionNames[i];
			GreedyHillClimber climber = new GreedyHillClimber(g1, g2,edgethr, new LinkedList<HashSet<Integer>>(algo2seed.get(currAlgo)),disc);
			climber.runImprovementStage(false);
			climber.updateModuleList(5, 1);
			String newAlgoName = currAlgo+"_"+"global";
			algo2modulesAfterImprovement.put(newAlgoName, climber.getModulesCopy());
			algo2CMPs.put(newAlgoName, climber.getAreModulesFriends());
//			System.out.println(getModuleStatistics( climber.getModulesCopy()));
			climber=null;
			Runtime.getRuntime().gc();
			System.out.flush();
		}
		
		// improve hierarchical clustering and global seed using the DICER approach
		BiModulesFinderUsingClusteringSol improver = new BiModulesFinderUsingClusteringSol(g1, g2, nodeIds);
		improver.setClusters(new LinkedList<HashSet<Integer>>(hierSeeds));
		improver.findSeedPairs(2);
		improver.improveSolution(5);
		improver.calculateMetaGraph();
		String algoName = seedDetectionNames[0]+"_"+improvmentAlgoNames[0];
		algo2modulesAfterImprovement.put(algoName, improver.getModulesCopy());
		algo2CMPs.put(algoName, improver.getAreModulesFriends());
//		System.out.println(getModuleStatistics(improver.getModulesCopy()));
		improver=null;
		Runtime.getRuntime().gc();
		System.out.flush();
		
		improver = new BiModulesFinderUsingClusteringSol(g1, g2, nodeIds);
		improver.setClusters(new LinkedList<HashSet<Integer>>(nodeAdditionSeeds));
		improver.findSeedPairs(2);
		improver.improveSolution(5);
		improver.calculateMetaGraph();
		algoName = seedDetectionNames[4]+"_"+improvmentAlgoNames[0];
		algo2modulesAfterImprovement.put(algoName, improver.getModulesCopy());
		algo2CMPs.put(algoName, improver.getAreModulesFriends());
//		System.out.println(getModuleStatistics(improver.getModulesCopy()));
		improver=null;
		Runtime.getRuntime().gc();
		System.out.flush();
	}


	public void checkForNullModules(List<HashSet<Integer>> hierSeeds) {
		for (Iterator<HashSet<Integer>> iterator = hierSeeds.iterator(); iterator.hasNext();) {
			HashSet<Integer> seed = (HashSet<Integer>) iterator.next();
			if (seed==null){
				System.out.println("Null seed in solution!");
			}
		}
	}
	
	public void updateHashForSeedAlgorithm(String name,MetaModuleFinder dicerSeedsFinder){
		List<HashSet<Integer>> dicerSeeds = dicerSeedsFinder.getModulesCopy();
		algo2seed.put(name, new LinkedList<HashSet<Integer>>(dicerSeeds));
		dicerSeedsFinder.improveSolution(5);
		algo2modulesAfterImprovement.put(name+"_"+improvmentAlgoNames[0], dicerSeedsFinder.getModulesCopy());
		algo2CMPs.put(name+"_"+improvmentAlgoNames[0], dicerSeedsFinder.getAreModulesFriends());
		algo2MMs_afterImprove.put(name+"_"+improvmentAlgoNames[0], dicerSeedsFinder.getMeta_modules());
	}
	
	public void updateHashForSeedAlgorithm(String name,List<HashSet<Integer>> seeds){
		algo2seed.put(name, seeds);
	}
	
	public static String getModuleStatistics(List<HashSet<Integer>> modules){
		float averageSize = 0;
		float totalCoverage = 0;
		float numModules = modules.size();
		for (Iterator<HashSet<Integer>> iterator = modules.iterator(); iterator.hasNext();) {
			HashSet<Integer> m = (HashSet<Integer>) iterator.next();
			averageSize+=m.size();
			totalCoverage+=m.size();
		}
		averageSize/=numModules;
		String out = numModules+"\t"+averageSize+"\t"+totalCoverage;
		return out;
	}
	
	public static String getModuleMapStatistics(boolean[][] map){
		float numEdges = 0;
		float meanDegree = 0;
		for (int i = 0; i < map.length; i++) {
			for (int j = 0; j < i; j++) {
				if (map[i][j]){
					numEdges++;
				}
			}
		}
		meanDegree = 2*numEdges/(float)map.length;
		String out = map.length+"\t"+numEdges+"\t"+meanDegree;
		return out;
	}
	
	public static String getModuleMapStatistics(short[][] map){
		float numEdges = 0;
		float meanDegree = 0;
		for (int i = 0; i < map.length; i++) {
			for (int j = 0; j < i; j++) {
				if (map[i][j]==1){
					numEdges++;
				}
			}
		}
		meanDegree = 2*numEdges/(float)map.length;
		String out = map.length+"\t"+numEdges+"\t"+meanDegree;
		return out;
	}
	
	public Hashtable<String, List<HashSet<Integer>>> getAlgo2modulesAfterImprovement() {
		return algo2modulesAfterImprovement;
	}
	
	public Hashtable<String, List<HashSet<Integer>>> getAlgo2seed() {
		return algo2seed;
	}
	
	public void printModuleStatistics(){
		for (Iterator<String> iterator = algo2modulesAfterImprovement.keySet().iterator(); iterator.hasNext();) {
			String name = (String) iterator.next();
			System.out.println(name+"\t"+getModuleStatistics(algo2modulesAfterImprovement.get(name)));
		}
	}
	
	public void printModuleSolutions(String outPrefix){
		for (Iterator<String> iterator = algo2modulesAfterImprovement.keySet().iterator(); iterator.hasNext();) {
			String name = (String) iterator.next();
			String outFileName = outPrefix+name+".txt";
			try {
				printModules("module", outFileName, algo2modulesAfterImprovement.get(name), nodeIds);
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}
	
	public void printSeedSolutions(String outPrefix){
		for (Iterator<String> iterator = algo2seed.keySet().iterator(); iterator.hasNext();) {
			String name = (String) iterator.next();
			String outFileName = outPrefix+name+"_seeds.txt";
			try {
				printModules("module", outFileName, algo2seed.get(name), nodeIds);
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}
	
	public void printMetaGraphs(String outPrefix){
		for (Iterator<String> iterator = algo2CMPs.keySet().iterator(); iterator.hasNext();) {
			String name = (String) iterator.next();
			String outFileName = outPrefix+name+"_module_map.sif";
			try {
				printMetaGraph("module", outFileName, algo2CMPs.get(name));
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}
	
	public void printSeedsStatistics(){
		for (Iterator<String> iterator = algo2seed.keySet().iterator(); iterator.hasNext();) {
			String name = (String) iterator.next();
			System.out.println(name+"\t"+getModuleStatistics(algo2seed.get(name)));
		}
	}
	
	public static void printMetaGraph(String prefix, String path, short[][] areModulesFriends) throws IOException{
		BufferedWriter writer = new BufferedWriter(new FileWriter(path));
		
		for (int i = 0; i < areModulesFriends.length; i++) {
			for (int j = 0; j < i; j++) {
				if (areModulesFriends[i][j]==1){
					String seed1Name = prefix + "_" + i;
					String seed2Name = prefix + "_" + j;
					writer.write(seed1Name + "\tEdge\t" + seed2Name + "\n");
				}
			}
		}
		writer.close();
	}
	
	public static void printModules(String prefix, String path, List<HashSet<Integer>> modules,ArrayList<String> geneNames) throws IOException {

		BufferedWriter writer = new BufferedWriter(new FileWriter(path));
		int counter = 0;
		for (Iterator<HashSet<Integer>> iterator = modules.iterator(); iterator
				.hasNext();) {
			HashSet<Integer> module = (HashSet<Integer>) iterator.next();
			
			for (Iterator<Integer> iterator2 = module.iterator(); iterator2
					.hasNext();) {
				Integer geneIndex = (Integer) iterator2.next();
				String geneName = geneNames.get(geneIndex);
				String seedName = prefix + "_" + counter;
				writer.write(geneName + "\t" + seedName + "\n");
			}
			counter++;
		}
		writer.close();
	}
	
	// getters
	public float[][] getG1() {
		return g1;
	}
	public float[][] getG2() {
		return g2;
	}
	


}
