package main_scripts;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Random;

import math_utils.ExpressionMatrix;
import multithreaded_algo_runs.MultithreadedComparisonRunner;

public class AlgorithmComparison_main_for_binary_graphs {

	public static void main(String[] args) {
		if (args.length != 2){
			System.out.println("usage: <g1: within, e.g. PPI><g2: between, e.g., GI>");
			System.exit(1);
		}
		
		// step 1 read each file
		// get the genes in both files
		HashSet<String> g1_genes = getGenesInGraph(args[0]);
		HashSet<String> g2_genes = getGenesInGraph(args[1]);
		
		Hashtable<String, HashSet<String>> g1AsLists = getGraphAsHash(args[0]);
		Hashtable<String, HashSet<String>> g2AsLists = getGraphAsHash(args[1]);
		g1_genes = getGenesWithHighDegree(g1AsLists, 5);
		g2_genes = getGenesWithHighDegree(g2AsLists, 1);
		
		HashSet<String> intersect = new HashSet<String>();
		for (Iterator<String> iterator = g1_genes.iterator(); iterator.hasNext();) {
			String inG1 = (String) iterator.next();
			if (g2_genes.contains(inG1)){
				intersect.add(inG1);
			}
		}
		
		HashSet<String> union = new HashSet<String>(g1_genes);
		union.addAll(g2_genes);
		
		HashSet<String> geneSet = intersect;
		System.out.println("######## num of genes: " + geneSet.size()+ "###############");
				
		Hashtable<String, Integer> gene2index = new Hashtable<String, Integer>();
		ArrayList<String> geneIds = new ArrayList<String>(geneSet.size());
		updateGenes(geneSet, gene2index, geneIds);
		
		float[][] g1 = getGraphAsMatrix(args[0], geneSet, gene2index,1,-1);
		System.out.println("in g1: " + getNumEdges(g1));
		float[][] g2 = getGraphAsMatrix(args[1], geneSet, gene2index);
		System.out.println("in g2: " + getNumEdges(g2));	
		
		String prefix = "ppi_vs_specific_pos_gi_5";
		String outPath = prefix + "_";
		String dummy_exp_mat_path = prefix+"_dummy_exp_mat.txt";
		Random rand = new Random();
		float[][] mat = new float[geneIds.size()][2];
		for (int i = 0; i < mat.length; i++) {
			mat[i][0]=400+rand.nextFloat();
			mat[i][1]=400+rand.nextFloat();
		}
		String[] h = {"c1","c2"};
		ExpressionMatrix expMat = new ExpressionMatrix(mat,h, geneIds);
		BufferedWriter w;
		try {
			w = new BufferedWriter(new FileWriter(dummy_exp_mat_path));
			expMat.printMatrix(w);
			w.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
//		System.exit(1);
		
		MultithreadedComparisonRunner comp = new MultithreadedComparisonRunner(g1, g2, geneIds,0,0.005f);
		comp.setMinModuleSize(3);
		comp.setMinMSeedSize(2);
		comp.doComparisons(true);
		comp.printSeedsStatistics();
		comp.printModuleStatistics();
		
		comp.printModuleSolutions(outPath);
		comp.printSeedSolutions(outPath);
		comp.printMetaGraphs(outPath);
	}

	public static void updateGenes(HashSet<String> geneSet,
			Hashtable<String, Integer> gene2index, ArrayList<String> geneIds) {
		int c=0;
		for (Iterator<String> iterator = geneSet.iterator(); iterator.hasNext();) {
			String inBoth = (String) iterator.next();
			gene2index.put(inBoth, c);
			geneIds.add(inBoth);
			c++;
		}
	}
	
	public static float[][] getGraphAsMatrix(String args,
			HashSet<String> geneSet, Hashtable<String, Integer> gene2index) {
		return getGraphAsMatrix(args, geneSet, gene2index,1,-1);
	}

	public static float[][] getGraphAsMatrix(String args,
			HashSet<String> geneSet, Hashtable<String, Integer> gene2index,int edgeScore,int nonEdgeScore) {

		float[][] g1 = new float[geneSet.size()][geneSet.size()];
		for (int i = 0; i < g1.length; i++) {
			Arrays.fill(g1[i], nonEdgeScore);
			g1[i][i]=-1;
		}
		
		try {
			BufferedReader reader = new BufferedReader(new FileReader(args));
			String line = "";
			while ((line=reader.readLine())!=null){
				String[] arr = line.split("\t");
				if (arr.length<2){
					continue;
				}
				String a = arr[0];
				String b = arr[1];
				if (arr.length>2){
					b = arr[2];
				}
				if (geneSet.contains(a) && geneSet.contains(b)){
					int a_i = gene2index.get(a);
					int b_i = gene2index.get(b);
					if (a_i==b_i){
						continue;
					}
					g1[a_i][b_i]=edgeScore;
					g1[b_i][a_i]=edgeScore;
				}
			}
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return g1;
	}
	
	public static Hashtable<String, HashSet<String>> getGraphAsHash(String arg) {
		
		Hashtable<String, HashSet<String>> gene2neighs = new Hashtable<String, HashSet<String>>();
		
		try {
			BufferedReader reader = new BufferedReader(new FileReader(arg));
			String line = "";
			while ((line=reader.readLine())!=null){
				String[] arr = line.split("\t");
				if (arr.length<2){
					continue;
				}
				String a = arr[0];
				String b = arr[1];
				
				if (arr.length>2){
					b = arr[2];
				}
				
				if (! gene2neighs.containsKey(a)){
					gene2neighs.put(a, new HashSet<String>());
				}
				if (!gene2neighs.containsKey(b)){
					gene2neighs.put(b, new HashSet<String>());
				}
				
				gene2neighs.get(a).add(b);
				gene2neighs.get(b).add(a);
			}
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return gene2neighs;
	}

	public static HashSet<String> getGenesInGraph(String args) {
		HashSet<String> g1_genes = new HashSet<String>();
		try {
			BufferedReader reader = new BufferedReader(new FileReader(args));
			String line = "";
			while ((line=reader.readLine())!=null){
				String[] arr = line.split("\t");
				if (arr.length!=2){
					continue;
				}
				g1_genes.add(arr[0]);
				g1_genes.add(arr[1]);
			}
			reader.close();
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return g1_genes;
	}
	
	public static int getNumEdges(float[][] g){
		
		int num=0;
		for (int i = 0; i < g.length; i++) {
			for (int j = 0; j < i; j++) {
				if (g[i][j]>0){
					num++;
				}
			}
		}
		return num;
	}
	
	public static HashSet<String> getGenesWithHighDegree(Hashtable<String, HashSet<String>> g, int threshold){
		HashSet<String> selectedGenes = new HashSet<String>();
		
		for (Iterator<String> iterator = g.keySet().iterator(); iterator.hasNext();) {
			String gene = (String) iterator.next();
			if (g.get(gene).size()>=threshold){
				selectedGenes.add(gene);
			}
		}
		
		return selectedGenes;
	}
}
