package main_scripts;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.Random;

import math_utils.ExpressionMatrix;
import math_utils.staticMathHelperMethods;
import edu.tau.compbio.math.TailProbs;
import edu.tau.compbio.stat.NormalDistribution;
import EM_LLR_Utils.LLRScoresCalculator;

public class DCGraphsCalculator {
	
	protected double K=2;
	
	// Memebers for the statistical class-specific analysis
	protected TscoresCalculator tScoresCalc;
	
	protected float[][] realTscoresGraph;
	protected float[] randomScores;
	protected LLRScoresCalculator llrAnalysisForTscores;
	protected LLRScoresCalculator globalCorrelationsLLRAnalyzer;

	// The results of the statistical analysis: up DC, down DC and the CG graphs
	protected float[][] dGGraph;
	protected float[][] uGGraph;
	protected float[][] consistentCorrelationsGraph;
	protected int classIndex;
	
	public DCGraphsCalculator(TscoresCalculator t, int classI) {
		tScoresCalc = t;
		classIndex =classI;
	}
	
	
	public void doStatisticalAnalysis(float maxPmates) {
		getDCGraphs();
		getConsistentCorrelationsGraph(maxPmates);
	}
	
	public void doStatisticalAnalysis(float maxPmates, boolean estimateK, boolean determineKForUp, float minPositiveDC) {
		getDCGraphs(estimateK,determineKForUp,minPositiveDC);
		getConsistentCorrelationsGraph(maxPmates);
	}
	
	
	public void getDCGraphs() {
		getDCGraphs(false,false,0);
	}

	public void getDCGraphs(boolean estimateK, boolean determineKForUp, float minPositiveDC) {
		
		tScoresCalc.setGlobalGraphForOneClass(classIndex);
		realTscoresGraph = tScoresCalc.scoredTscoresGraph;
		float[] realScores = new float[realTscoresGraph.length
				* (realTscoresGraph.length - 1) / 2];
		int count = 0;
		for (int i = 0; i < realTscoresGraph.length; i++) {
			for (int j = 0; j < i; j++) {
				realScores[count] = realTscoresGraph[i][j];
				count++;
			}
		}

		float realAvg = staticMathHelperMethods.Average(realScores);
		float realSD = staticMathHelperMethods.sStatistic(realScores, realAvg);

		Runtime.getRuntime().gc();

		// get clustering
		// first get random DC scores
		int randSize = 500;
		int numGenes = tScoresCalc.data.matrix.length;
		
		if (randSize > numGenes/2){
			randSize = numGenes/2;
		}
		
		LinkedList<Float> randScores = new LinkedList<Float>();
		Random rand = new Random();
		
		float numRepeats=200;
		for (int i = 0; i < numRepeats; i++) {

			// get subMatrix
			LinkedList<Integer> rowIndices = new LinkedList<Integer>();
			for (int j = 0; j < randSize; j++) {
				int nextInt = rand.nextInt(numGenes);
				while (rowIndices.contains(nextInt)) {
					nextInt = rand.nextInt(numGenes);
				}
				rowIndices.add(nextInt);
			}

			ExpressionMatrix subMatrix = tScoresCalc.data
					.getSubMatrixByRowsIndices(rowIndices);
			ExpressionMatrix permutedMatrix = subMatrix
					.getShuffeledMatrixByColumns(rand);

			TscoresCalculator shuffComp = new TscoresCalculator(permutedMatrix,
					tScoresCalc.classesBorders, tScoresCalc.ClassesNames,
					"pearson");
			shuffComp.setGlobalGraphForOneClass(classIndex);
			float[][] matrix = shuffComp.scoredTscoresGraph;
			for (int j = 0; j < matrix.length; j++) {
				for (int j2 = 0; j2 < j; j2++) {
					randScores.add(matrix[j][j2]);
				}
			}
			
			shuffComp = null;
			subMatrix=null;
			matrix=null;
			permutedMatrix = null;
			Runtime.getRuntime().gc();
		}

		randomScores = floatListToArray(randScores);
		
		double randAvg = staticMathHelperMethods.Average(randomScores);
		double randSD = staticMathHelperMethods.sStatistic(randomScores,(float) randAvg);
		
		randAvg = (double)Math.round(randAvg * 100) / 100.0;
		randSD = (double)Math.round(randSD * 100) / 100.0;
		
		NormalDistribution realMates = new NormalDistribution(realAvg, realSD);
		NormalDistribution RandDist = new NormalDistribution(randAvg, randSD);
		
		// calculate pMates
		double negThreshold = randAvg + K * randSD;
		double z = (negThreshold - realAvg) / realSD;
		double pMates = 2 * TailProbs.probNormal(z);
		
		llrAnalysisForTscores = new LLRScoresCalculator(
				tScoresCalc.data.getGenesID(), realTscoresGraph);
		llrAnalysisForTscores.setDistributions(realMates, RandDist, pMates);

		// This is for estimating K
		if (estimateK){
			
			float[] corrs = new float[tScoresCalc.ClassesNames.length];
			corrs[classIndex]=minPositiveDC;
			if (!determineKForUp){
				corrs[classIndex]=-minPositiveDC;
			}
			
			float realTscore = tScoresCalc.getTscoreForCorrelations(corrs, classIndex);
			System.out.println("real t score for diff: " + realTscore);
			System.out.println("K, LLR");
			float[] possibleKs = new float[100];
			for (int i = 1; i < possibleKs.length; i++) {
				float currVal = (float)i*0.1f;
				K=currVal;
				possibleKs[i]=currVal;
				
				negThreshold = randAvg + currVal * randSD;
				z = (negThreshold - realAvg) / realSD;
				pMates = 2 * TailProbs.probNormal(z);
				
				llrAnalysisForTscores = new LLRScoresCalculator(
						tScoresCalc.data.getGenesID(), realTscoresGraph);
				llrAnalysisForTscores.setDistributions(realMates, RandDist, pMates);

				double currLLRScore = llrAnalysisForTscores.getSimLikeLihood(realTscore, false);
				System.out.println(currVal + ", " + currLLRScore);
				if (currLLRScore<0 && currVal >= K){
					System.out.println("selected k: " + currVal);
					break;
				}
			}
			//System.exit(1);
		}


		dGGraph = (llrAnalysisForTscores.getClickWeightedMatrix(false));
		llrAnalysisForTscores = null;
		Runtime.getRuntime().gc();
		
		uGGraph = new float[dGGraph.length][dGGraph.length];

		// set the graph as "down"
		// if the diff score if > 0 , set the likelihood as well
		int countPositives = 0;
		int countNegatives = 0;
		for (int i = 0; i < dGGraph.length; i++) {
			for (int j = 0; j < i; j++) {
				if (dGGraph[i][j] > 0 && realTscoresGraph[i][j] > 0) {
					countPositives++;
				}
				if (dGGraph[i][j] > 0 && realTscoresGraph[i][j] < 0) {
					countNegatives++;
				}

				if (realTscoresGraph[i][j] == 0 || dGGraph[i][j] < 0) {
					uGGraph[i][j] = dGGraph[i][j];
				} else {
					if (realTscoresGraph[i][j] > 0) {
						uGGraph[i][j] = dGGraph[i][j];
						dGGraph[i][j] *= -1;
					} else {
						uGGraph[i][j] = -dGGraph[i][j];
					}
				}

				uGGraph[j][i] = uGGraph[i][j];
				dGGraph[j][i] = dGGraph[i][j];
			}
		}
		
		realScores=null;
		realTscoresGraph=null;
		randomScores=null;
		Runtime.getRuntime().gc();
	}


	public void getConsistentCorrelationsGraph(float maxPmates) {
		System.out.println("calculating global graph");
		for (int i = 0; i < tScoresCalc.ClassesNames.length; i++) {
			globalCorrelationsLLRAnalyzer = new LLRScoresCalculator(
					tScoresCalc.data.getGenesID(),
					tScoresCalc.class2Correlations.get(i));
			globalCorrelationsLLRAnalyzer.setMatesMaxP(maxPmates);
			if (i == 0) {
				consistentCorrelationsGraph = (globalCorrelationsLLRAnalyzer
						.getClickWeightedMatrix(true));
			} else {
				float[][] tmpGraph = (globalCorrelationsLLRAnalyzer
						.getClickWeightedMatrix(true));
				for (int j = 0; j < tmpGraph.length; j++) {
					for (int j2 = 0; j2 < tmpGraph.length; j2++) {
						if (consistentCorrelationsGraph[j][j2]>tmpGraph[j][j2]){
							consistentCorrelationsGraph[j][j2]=tmpGraph[j][j2];
						}
					}
				}
			}
			
			globalCorrelationsLLRAnalyzer = null;
			Runtime.getRuntime().gc();
		}
	}
	
	public static float[] floatListToArray(LinkedList<Float> positiveScores) {
		float[] posScores = new float[positiveScores.size()];
		int counter = 0;
		for (Iterator<Float> iterator = positiveScores.iterator(); iterator
				.hasNext();) {
			Float score = (Float) iterator.next();
			posScores[counter] = score;
			counter++;
		}
		return posScores;
	}
	
	public void setK(double k) {
		K = k;
	}
	
	public static float[][] doubleToFloatMatrix(double[][] a){
		float[][] mat = new float[a.length][a[0].length];
		for (int i = 0; i < mat.length; i++) {
			for (int j = 0; j < mat[0].length; j++) {
				mat[i][j] = (new Double(a[i][j])).floatValue();
			}
		}
		
		return mat;
	}
}
