package main_scripts;

import graph_algorithms.BiModuleFinderUsingBicliqueAlgo;
import graph_algorithms.MetaModuleFinder;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;

public class DICER_main_for_binary_graphs {

	public static void main(String[] args) {
		if (args.length != 2){
			System.out.println("usage: <g1: within, e.g. PPI><g2: between, e.g., GI>");
			System.exit(1);
		}
		
		// step 1 read each file
		// get the genes in both files
		HashSet<String> g1_genes = getGenesInGraph(args[0]);
		HashSet<String> g2_genes = getGenesInGraph(args[1]);
		
		HashSet<String> intersect = new HashSet<String>();
		for (Iterator<String> iterator = g1_genes.iterator(); iterator.hasNext();) {
			String inG1 = (String) iterator.next();
			if (g2_genes.contains(inG1)){
				intersect.add(inG1);
			}
		}
		
		System.out.println("num of genes: " + intersect.size());
		
		Hashtable<String, Integer> gene2index = new Hashtable<String, Integer>();
		int c=0;
		ArrayList<String> geneIds = new ArrayList<String>(intersect.size());
		
		for (Iterator<String> iterator = intersect.iterator(); iterator.hasNext();) {
			String inBoth = (String) iterator.next();
			gene2index.put(inBoth, c);
			geneIds.add(inBoth);
			c++;
		}
		
		float[][] g1 = getGraphAsMatrix(args[0], intersect, gene2index);
		System.out.println("in ppi: " + getNumEdges(g1));
		float[][] g2 = getGraphAsMatrix(args[1], intersect, gene2index);
		System.out.println("in gi: " + getNumEdges(g2));
		
		BiModuleFinderUsingBicliqueAlgo mmFinder = new BiModuleFinderUsingBicliqueAlgo(g1, g2, geneIds);
		mmFinder.addNumToTmpFiles(100);
		

//		MetaModuleFinder mmFinder = new MetaModuleFinder(g1, g2, geneIds);
		
		
		mmFinder.setMinSeedModuleSize(2);
		mmFinder.setSizesConsistencyThr(0.1);
		mmFinder.getFinalSolution();
		System.out.println("num modules: " +mmFinder.modules.size());
		int genesInModules = 0;
		for (Iterator<HashSet<Integer>> iterator = mmFinder.modules.iterator(); iterator.hasNext();) {
			HashSet<Integer> module = (HashSet<Integer>) iterator.next();
//			System.out.println(module.size());
			genesInModules+=module.size();
		}
		System.out.println("Genes in modules: "+genesInModules);
		
		String out_modules = "modules.txt";
		String out_meta_graph = "module_graph.sif";
		if (mmFinder instanceof BiModuleFinderUsingBicliqueAlgo){
			out_modules = "modules_mbc_dicer.txt";
			out_meta_graph = "module_graph_mbc_dicer.sif";
		}
		else{
			out_modules = "modules_dicer.txt";
			out_meta_graph = "module_graph_dicer.sif";
		}
		
		try {
			mmFinder.printModules("module", out_modules, 2);
			mmFinder.printMetaGraph("module", out_meta_graph);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public static float[][] getGraphAsMatrix(String args,
			HashSet<String> intersect, Hashtable<String, Integer> gene2index) {
		// read each graph to a matrix
		// put 1 in edge
		// -1 otherwise
		// 1 in the diagonal
		float[][] g1 = new float[intersect.size()][intersect.size()];
		for (int i = 0; i < g1.length; i++) {
			Arrays.fill(g1[i], -1);
			g1[i][i]=1;
		}
		
		try {
			BufferedReader reader = new BufferedReader(new FileReader(args));
			String line = "";
			while ((line=reader.readLine())!=null){
				String[] arr = line.split("\t");
				if (arr.length!=2){
					continue;
				}
				String a = arr[0];
				String b = arr[1];
				if (intersect.contains(a) && intersect.contains(b)){
					int a_i = gene2index.get(a);
					int b_i = gene2index.get(b);
					g1[a_i][b_i]=1;
					g1[b_i][a_i]=1;
				}
			}
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return g1;
	}

	public static HashSet<String> getGenesInGraph(String args) {
		HashSet<String> g1_genes = new HashSet<String>();
		try {
			BufferedReader reader = new BufferedReader(new FileReader(args));
			String line = "";
			while ((line=reader.readLine())!=null){
				String[] arr = line.split("\t");
				if (arr.length!=2){
					continue;
				}
				g1_genes.add(arr[0]);
				g1_genes.add(arr[1]);
			}
			reader.close();
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return g1_genes;
	}
	
	public static int getNumEdges(float[][] g){
		
		int num=0;
		for (int i = 0; i < g.length; i++) {
			for (int j = 0; j < i; j++) {
				if (g[i][j]>0){
					num++;
				}
			}
		}
		
		return num;
		
	}
}
