package main_scripts;

import global_analysis.GlobalImproveAlgorithm;
import global_analysis.GreedyHillClimber;
import global_analysis.StatisticalFriendsTester;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;

public class GetSolutionEdgesPvalue {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		if (args.length != 4){
			System.out.println("usage: <g1: within, e.g. PPI><g2: between, e.g., GI><modules: txt file><map: sif file>");
			System.exit(1);
		}
		
		HashSet<String> g1_genes = AlgorithmComparison_main_for_binary_graphs.getGenesInGraph(args[0]);
		HashSet<String> g2_genes = AlgorithmComparison_main_for_binary_graphs.getGenesInGraph(args[1]);
		
		Hashtable<String, HashSet<String>> g1AsLists = AlgorithmComparison_main_for_binary_graphs.getGraphAsHash(args[0]);
		Hashtable<String, HashSet<String>> g2AsLists = AlgorithmComparison_main_for_binary_graphs.getGraphAsHash(args[1]);
		
		HashSet<String> intersect = new HashSet<String>();
		for (Iterator<String> iterator = g1_genes.iterator(); iterator.hasNext();) {
			String inG1 = (String) iterator.next();
			if (g2_genes.contains(inG1)){
				intersect.add(inG1);
			}
		}
		
		HashSet<String> union = new HashSet<String>(g1_genes);
		union.addAll(g2_genes);
		for (Iterator<String> iterator = union.iterator(); iterator.hasNext();) {
			String gene = (String) iterator.next();
			int deg1=0;
			int deg2=0;
			if (g1AsLists.containsKey(gene)){
				deg1=g1AsLists.get(gene).size();
			}
			if (g2AsLists.containsKey(gene)){
				deg2=g2AsLists.get(gene).size();
			}
		}
		
		HashSet<String> geneSet = union;
		Hashtable<String, Integer> gene2index = new Hashtable<String, Integer>();
		ArrayList<String> geneIds = new ArrayList<String>(geneSet.size());

		AlgorithmComparison_main_for_binary_graphs.updateGenes(geneSet, gene2index, geneIds);
		
		float[][] g1 = AlgorithmComparison_main_for_binary_graphs.getGraphAsMatrix(args[0], geneSet, gene2index);
		System.out.println("in g1: " + AlgorithmComparison_main_for_binary_graphs.getNumEdges(g1));
		float[][] g2 = AlgorithmComparison_main_for_binary_graphs.getGraphAsMatrix(args[1], geneSet, gene2index);
		System.out.println("in g2: " + AlgorithmComparison_main_for_binary_graphs.getNumEdges(g2));
		
		// parse the map
		HashSet<String> clusters = new HashSet<String>(AlgorithmComparison_main_for_binary_graphs.getGraphAsHash(args[3]).keySet());
		System.out.println("read: " + clusters.size() + " modules");
		Hashtable<String, Integer> cluster2index = new Hashtable<String, Integer>();
		Hashtable<Integer, String> ind2cluster = new Hashtable<Integer, String>();
		String[] clusterArr = new String[clusters.size()];
		int c=0;
		for (Iterator<String> iterator = clusters.iterator(); iterator.hasNext();) {
			String cl = (String) iterator.next();
			cluster2index.put(cl, c);
			ind2cluster.put(c, cl);
			clusterArr[c]=cl;
			c++;
		}
		
		// parse the clustering solution
		Hashtable<String, String> gene2cluster = new Hashtable<String, String>();
		Hashtable<String, HashSet<Integer>> cluster2geneinds = new Hashtable<String, HashSet<Integer>>();
		try {
			BufferedReader reader = new BufferedReader(new FileReader(args[2]));
			String l = "";
			while ((l=reader.readLine())!=null){
				String[] arr = l.split("\t");
				String gene = arr[0];
				String cl = arr[1];
				gene2cluster.put(gene, cl);
				if (! cluster2geneinds.containsKey(cl)){
					cluster2geneinds.put(cl, new HashSet<Integer>());
				}
				int geneInd = gene2index.get(gene);
				cluster2geneinds.get(cl).add(geneInd);
			}
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		System.out.println("calculating map scores");
		
		float[][] mmap = AlgorithmComparison_main_for_binary_graphs.getGraphAsMatrix(args[3], clusters, cluster2index);
		GlobalImproveAlgorithm dummy_imp = new GreedyHillClimber(g1, g2);
		System.out.println("before loop; " + mmap.length);
		for (int i = 0; i < mmap.length; i++) {
			HashSet<Integer> c1 = cluster2geneinds.get(ind2cluster.get(i));
			String c1_name = ind2cluster.get(i);
			for (int j = 0; j < i; j++) {
				if (mmap[i][j]<=0){continue;}
				HashSet<Integer> c2 = cluster2geneinds.get(ind2cluster.get(j));
				String c2_name = ind2cluster.get(j);
				double p = dummy_imp.areModuleFriends(c1, c2).p;
				System.out.println(c1_name+"\t"+c2_name+"\t"+p);
			}
		}
		
	}

}
