package main_scripts;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Map;

import math_utils.ExpressionMatrix;
import math_utils.staticMathHelperMethods;

/**
 * 
 * This class calculated the T-scores.
 * T-score is the measure of the normalized correlation
 * difference of a gene-pair (where the normalization is perform
 * vis-a-vis all other differences). 
 *
 */

public class TscoresCalculator {

		/* MEMBERS */
		public ExpressionMatrix data;
		int[] classesBorders;
		String[] ClassesNames;
		private int numClasses;
		String similarityType;

		public Map<Integer, float[][]> class2Correlations;
		public float[][] scoredTscoresGraph;
		Hashtable<Integer, Float> classesAvgs = new Hashtable<Integer, Float>();
		Hashtable<Integer, Float> classesVars = new Hashtable<Integer, Float>();

		public TscoresCalculator(String GEDataFilePath,
				String ClassesFilePath, String type) {
			data = new ExpressionMatrix(GEDataFilePath);
			ArrayList<Integer> fromTo = new ArrayList<Integer>();
			ArrayList<String> names = new ArrayList<String>();
			try {
				BufferedReader reader = new BufferedReader(new FileReader(
						ClassesFilePath));
				String currLine = "";
				while ((currLine = reader.readLine()) != null) {
					String[] line = currLine.split("\t");
					if (line.length != 3) {
						System.out.println("divide to classes file is not valid");
						System.exit(1);
					}
					numClasses++;
					fromTo.add(Integer.parseInt(line[0]));
					fromTo.add(Integer.parseInt(line[1]));
					names.add(line[2]);
				}
			} catch (FileNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			classesBorders = new int[fromTo.size()];
			for (int i = 0; i < classesBorders.length; i++) {
				classesBorders[i] = fromTo.get(i);
			}
			ClassesNames = new String[names.size()];
			for (int i = 0; i < ClassesNames.length; i++) {
				ClassesNames[i] = names.get(i);
			}
			classesBorders[classesBorders.length - 1] = data.getMatrix()[0].length - 1;

			similarityType = type;

		}

		public TscoresCalculator(ExpressionMatrix data, int[] borders,
				String[] names, String type) {
			this.data = data;
			classesBorders = borders;
			ClassesNames = names;
			similarityType = type;
		}
		
		public TscoresCalculator(ExpressionMatrix data,String cP, String type) {
			this.data = data;
			
			ArrayList<Integer> fromTo = new ArrayList<Integer>();
			ArrayList<String> names = new ArrayList<String>();
			try {
				BufferedReader reader = new BufferedReader(new FileReader(
						cP));
				String currLine = "";
				while ((currLine = reader.readLine()) != null) {
					String[] line = currLine.split("\t");
					if (line.length != 3) {
						System.out.println("divide to classes file is not valid");
						System.exit(1);
					}
					numClasses++;
					fromTo.add(Integer.parseInt(line[0]));
					fromTo.add(Integer.parseInt(line[1]));
					names.add(line[2]);
				}
			} catch (FileNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			classesBorders = new int[fromTo.size()];
			for (int i = 0; i < classesBorders.length; i++) {
				classesBorders[i] = fromTo.get(i);
			}
			ClassesNames = new String[names.size()];
			for (int i = 0; i < ClassesNames.length; i++) {
				ClassesNames[i] = names.get(i);
			}
			classesBorders[classesBorders.length - 1] = data.getMatrix()[0].length - 1;

			similarityType = type;
		}

		

		private void analyzeOneEdgeWithRespectToAClass(
				int i, int j, int j2, Map<Integer, Float> classesAvgs,
				Map<Integer, Float> classesVars) {
			
			boolean isEdgeUpInClass = true;
			boolean isEdgeDownInClass = true;
			
			float minDiff = Float.MAX_VALUE;
			float[][] firstClassMatrix = class2Correlations.get(j2);
			for (int k = 0; k < ClassesNames.length; k++) {
				if (j2 == k) {
					continue;
				}
				float[][] secondClassMatrix = class2Correlations.get(k);
				
				float diff = firstClassMatrix[i][j] - secondClassMatrix[i][j];
				float diffH0Avg = classesAvgs.get(j2) - classesAvgs.get(k);
				float diffH0var = classesVars.get(j2) + classesVars.get(k);
				float diffSD = (new Double(Math.sqrt(diffH0var))).floatValue();
				// standardize diff
				diff -= diffH0Avg;
				diff /= diffSD;
				
				if (diff > 0) {
					isEdgeDownInClass = false;
				}
				else if (diff < 0 ){
					isEdgeUpInClass = false;
				}
				
				if ((!isEdgeDownInClass) && (!isEdgeUpInClass)) {
					break;
				}

				if (Math.abs(diff) < minDiff) {
					minDiff = diff;
				}
			}
			
			if (isEdgeUpInClass || isEdgeDownInClass) {
				// System.out.println("found edge");
				scoredTscoresGraph[i][j] = minDiff;
				scoredTscoresGraph[j][i] = minDiff;
			}
		}
		
		public float getTscoreForCorrelations(float[] corrs, int classOfInterest) {
			
			boolean isEdgeUpInClass = true;
			boolean isEdgeDownInClass = true;
			
			float minDiff = Float.MAX_VALUE;
			for (int k = 0; k < ClassesNames.length; k++) {
				if (classOfInterest == k) {
					continue;
				}
				
				float diff = corrs[classOfInterest] - corrs[k];
				float diffH0Avg = classesAvgs.get(classOfInterest) - classesAvgs.get(k);
				float diffH0var = classesVars.get(classOfInterest) + classesVars.get(k);
				float diffSD = (new Double(Math.sqrt(diffH0var))).floatValue();
				// standardize diff
				diff -= diffH0Avg;
				diff /= diffSD;
				
				if (diff > 0) {
					isEdgeDownInClass = false;
				}
				else if (diff < 0 ){
					isEdgeUpInClass = false;
				}
				
				if ((!isEdgeDownInClass) && (!isEdgeUpInClass)) {
					break;
				}

				if (Math.abs(diff) < minDiff) {
					minDiff = diff;
				}
			}
			
			return minDiff;
		}

		public void claculateSimilarities() {

			class2Correlations = new Hashtable<Integer, float[][]>();

			int numOfGenes = data.matrix.length;
			for (int i = 0; i < ClassesNames.length; i++) {
				float[][] currentMatrix = new float[numOfGenes][numOfGenes];
				int classTo = classesBorders[2 * i + 1];
				int classFrom = classesBorders[2 * i];
				currentMatrix = staticMathHelperMethods.computePairWiseScoreMatrix(
						data.matrix, similarityType, classFrom, classTo);
				class2Correlations.put(i, currentMatrix);
			}
		}


		public void setGlobalGraphForOneClass(int classIndex) {

			claculateSimilarities();	
			for (int i = 0; i < ClassesNames.length; i++) {
				float currAvg = 0;
				for (int j = 0; j < class2Correlations.get(i).length; j++) {
					for (int j2 = 0; j2 < j; j2++) {
						currAvg += class2Correlations.get(i)[j][j2];
					}
				}
				float n = class2Correlations.get(i).length;
				currAvg /=(n*(n-1)/2);
				
				float currSD = 0;
				for (int j = 0; j < class2Correlations.get(i).length; j++) {
					for (int j2 = 0; j2 < j; j2++) {
						float tmp = class2Correlations.get(i)[j][j2]-currAvg;
						currSD += tmp*tmp;
					}
				}
				currSD /=((n*(n-1)/2)-1);
				
				classesAvgs.put(i, currAvg);
				classesVars.put(i, currSD);
			}

			scoredTscoresGraph = new float[data.matrix.length][data.matrix.length];

			// Check every edge separately
			for (int i = 0; i < data.matrix.length; i++) {
				for (int j = 0; j < i; j++) {
					analyzeOneEdgeWithRespectToAClass(i, j, classIndex,classesAvgs,classesVars);
				}
			}
		}
		
		public int[] getClassesBorders() {
			return classesBorders;
		}
	
	}

