package math_utils;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;

public class SimilarityMatrix {
	
	/* MEMBERS */
	public Hashtable<String, Integer> id2Index;
	public float[][] similarities;
	String[] header;
	
	public SimilarityMatrix() {
		// TODO Auto-generated constructor stub
	}
	
	public SimilarityMatrix(String path) {
		try {
			System.out.println("reading similarity matrix at: " + path);
			BufferedReader reader = new BufferedReader(new FileReader(path));
			
			// first read the header
			String line = reader.readLine();
			line.trim();
			String[] header = line.split("\t");
			id2Index = new Hashtable<String, Integer>();
			System.out.println("going over the header:");
			for (int i = 0; i < header.length; i++) {
				String currid = header[i];
				currid=currid.substring(1, currid.length()-1);
//				System.out.println(currid);
				id2Index.put(currid, i);
			}
			
			similarities = new float[header.length][header.length];
			int countRows=0;
			
			while ((line=reader.readLine())!=null){
				line.trim();
				String[] currLine = line.split("\t");
				for (int i = 1; i < currLine.length; i++) {
					Double currNum = Double.parseDouble(currLine[i]);
					float val = currNum.floatValue();
					similarities[countRows][i-1] = val;
				}
				countRows++;
			}
			
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		// pring the id2index data hash
//		for (Iterator<String> iterator = id2Index.keySet().iterator(); iterator.hasNext();) {
//			String id = (String) iterator.next();
////			System.out.println(id+ " - " + id2Index.get(id));
//		}
	}
	
	public float[][] getSimilarities() {
		return similarities;
	}
	public Hashtable<String, Integer> getId2Index() {
		return id2Index;
	}
	
	public float getSimilarity(String a, String b){
//		System.out.println("in get similarity: " + a+ "," + b);
		if (id2Index.containsKey(a) && id2Index.containsKey(b)){
//			System.out.println("both in hash");
			return similarities[id2Index.get(a)][id2Index.get(b)];
		}
		return 0;
	}
	
	public float getSimilarity(int a, int b){
		return similarities[a][b];
	}
	
	public int size(){
		return similarities.length;
	}
	
	public void createHeader(){
		header = new String[id2Index.keySet().size()];
//		System.out.println(id2Index.keySet().size());
		for (Iterator<String> iterator = id2Index.keySet().iterator(); iterator.hasNext();) {
			String id = (String) iterator.next();
//			System.out.println(id);
			int index = id2Index.get(id);
//			System.out.println(index);
			header[index]=id;
			
		}
	}
	
	
	public void printMatrix(){
		createHeader();
		for (int i = 0; i < similarities.length; i++) {
			System.out.print(header[i]);
			for (int j = 0; j < similarities.length; j++) {
				System.out.print("\t" + similarities[i][j]);
			}
			System.out.println();
		}
	}
	
	public void printMCLInput(String path,double thr) throws IOException{
		BufferedWriter writer = new BufferedWriter(new FileWriter(path));
		for (int i = 0; i < similarities.length; i++) {
			for (int j = 0; j < i; j++) {
				if (similarities[i][j] >= thr){
					writer.write(header[i] + " " + header[j] + "\n");
				}
			}
		}
		writer.close();
	}
	
	public void print_FP_MBC_Input(String path,double min_thr) throws IOException{
		BufferedWriter writer = new BufferedWriter(new FileWriter(path));
		for (int i = 0; i < similarities.length; i++) {
			String currLine = "";
			int numEdges = 0;
			for (int j = 0; j < similarities.length; j++) {
				if (similarities[i][j] >= min_thr && i!=j){
					numEdges++;
					if (numEdges==1){
						currLine += ""+j;
					}
					else{
						currLine += " "+j;
					}
				}
			}
			writer.write(currLine+"\n");
		}
		writer.close();
	}
	
	public void print_FP_MBC_Input(String path,double min_thr,double max_thr) throws IOException{
		BufferedWriter writer = new BufferedWriter(new FileWriter(path));
		for (int i = 0; i < similarities.length; i++) {
			String currLine = "";
			int numEdges = 0;
			for (int j = 0; j < similarities.length; j++) {
				if (similarities[i][j] >= min_thr
						&& similarities[i][j] < max_thr){
					numEdges++;
					if (numEdges==1){
						currLine += ""+j;
					}
					else{
						currLine += " "+j;
					}
				}
			}
			writer.write(currLine+"\n");
		}
		writer.close();
	}
	
	public void setHeader(String[] header) {
		this.header = header;
	}
	
	public void setHeader(ArrayList<String> l) {
		header = new String[l.size()];
		for (int i = 0; i < l.size(); i++) {
			header[i]=l.get(i);
		}
	}
	
	public void setHeader(int n) {
		header = new String[n];
		for (int i = 0; i < n; i++) {
			header[i]=""+i;
		}
	}
	
	public void standardize(){
		float avg = 0;
		float counter = 0;
		
		
		for (int i = 0; i < similarities.length; i++) {
			for (int j = 0; j <= i; j++) {
				avg += similarities[i][j];
				counter++;
			}
		}
		
		System.out.println("counter is " + counter);
		System.out.println("sum is " + avg);
		
		avg /= counter;
		System.out.println("in standardize, average is: " + avg);
		
		float sd = 0;
		for (int i = 0; i < similarities.length; i++) {
			for (int j = 0; j <= i; j++) {
				float tmp = similarities[i][j]-avg;
				sd += tmp*tmp; 
			}
		}
		
		sd /= (counter-1);
		sd  = (new Double(Math.sqrt(sd))).floatValue();
		System.out.println("in standardize, sd is :" + sd) ;
		
		for (int i = 0; i < similarities.length; i++) {
			for (int j = 0; j <= i; j++) {
				similarities[i][j] = similarities[i][j]-avg;
				similarities[i][j] /= sd;
				similarities[j][i] = similarities[i][j];
			}
		}
		
		
	}
}
