package multithreaded_algo_runs;

import graph_algorithms.MetaModule;
import graph_algorithms.MetaModuleFinder;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

public class MultithreadedComparisonRunner {
	
	float[][] g1;
	float[][] g2;
	
	Hashtable<String,List<HashSet<Integer>>> algo2seed;
	public Hashtable<String,List<HashSet<Integer>>> algo2modulesAfterImprovement;
	Hashtable<String,List<MetaModule>> algo2MMs_afterImprove;
	Hashtable<String,boolean[][]> algo2friends;
	public Hashtable<String,short[][]> algo2CMPs;
	
	float edgethr = 1;
	int numThreads = 2;
	float alpha = 0.0001f;
	
	int minMSeedSize = 5;
	int minModuleSize = 5;
	
	// seeds detection algorithm name
	ArrayList<String> seedDetectionNames = new ArrayList<String>();
	String[] improvmentAlgoNames = {"local","global"};
	
	ArrayList<String> nodeIds;
	
	public MultithreadedComparisonRunner(float[][] g1, float[][] g2,ArrayList<String> ids, float edgethr,float a) {
		alpha=a;
		this.g1=g1;
		this.g2=g2;
		nodeIds=ids;
		this.edgethr=edgethr;
		
		algo2seed = new Hashtable<String, List<HashSet<Integer>>>();
		algo2modulesAfterImprovement = new Hashtable<String, List<HashSet<Integer>>>();
		algo2MMs_afterImprove = new Hashtable<String,List<MetaModule>>();
		algo2friends = new Hashtable<String,boolean[][]>();
		algo2CMPs = new Hashtable<String,short[][]>();
	}
	

	public void doComparisons(boolean disc) {
	
		/*
		 * Run seed detection algorithms
		 */
		ArrayList<Thread> threads = new ArrayList<Thread>();
		Thread	mbc_dicer_t = new MBCDicerSeedsThread(this, "MBC-DICER", minMSeedSize);
		threads.add(mbc_dicer_t);
		seedDetectionNames.add("MBC-DICER");
		numThreads--;
		clearThreadsPool(threads);

//		Thread hier_t = new HierarchicalClusteringThread(this, "hierarchical");
//		threads.add(hier_t);
//		seedDetectionNames.add("hierarchical");
//		numThreads--;
//		clearThreadsPool(threads);
		
//		Thread dicer_t = new DicerSeedSearch(this, "DICER", 2);
//		threads.add(dicer_t);
//		seedDetectionNames.add("DICER");
//		numThreads--;
//		clearThreadsPool(threads);
		
		Thread dicer_star_t = new DicerSeedSearch(this, "DICER5", minMSeedSize);
		threads.add(dicer_star_t);
		seedDetectionNames.add("DICER5");
		numThreads--;
		clearThreadsPool(threads);
		
//		Thread node_add_t = new NodeAdditionThread(this, "NodeAddition", disc);
//		threads.add(node_add_t);
//		seedDetectionNames.add("NodeAddition");
//		numThreads--;
//		clearThreadsPool(threads);
		
		if (numThreads>0){
			numThreads=0;
			clearThreadsPool(threads);
		}
		
		/*
		 * Run improvement stages
		 */
		GlobalImproveThread[] global_ts = new GlobalImproveThread[seedDetectionNames.size()];
		for (int i = 0; i < seedDetectionNames.size(); i++) {
			if (algo2seed.get(seedDetectionNames.get(i)).size()>0){
				System.out.println(seedDetectionNames.get(i));
				global_ts[i] = new GlobalImproveThread(this, seedDetectionNames.get(i), disc, alpha,true);
				try {
					global_ts[i].join();
				} catch (InterruptedException e) {
					// TODO: handle exception
				}
			}
		}
	
		// improve hierarchical clustering and global seed using the DICER approach
		if (seedDetectionNames.contains("hierarchical")){
			LocalImproveThread hier_local_t = new LocalImproveThread(this, "hierarchical");
			LocalImproveThread node_add_local_t = new LocalImproveThread(this, "NodeAddition");
			// 	wait for the threads to finish
			try {
				hier_local_t.join();
				node_add_local_t.join();
			} catch (InterruptedException e) {
			// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}
	
	public void improveSolutionUsingG1Only(){
		
	}


	public void clearThreadsPool(ArrayList<Thread> threads) {
		if (numThreads<=0){
			try {
				int s = threads.size();
				for (int i = 0; i < s; i++) {threads.get(0).join();threads.remove(0);}
				Runtime.getRuntime().gc();
			} catch (InterruptedException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
		}
	}


	public void checkForNullModules(List<HashSet<Integer>> hierSeeds) {
		for (Iterator<HashSet<Integer>> iterator = hierSeeds.iterator(); iterator.hasNext();) {
			HashSet<Integer> seed = (HashSet<Integer>) iterator.next();
			if (seed==null){
				System.out.println("Null seed in solution!");
			}
		}
	}
	
	public void updateHashForSeedAlgorithm(String name,MetaModuleFinder dicerSeedsFinder){
		algo2seed.put(name, new LinkedList<HashSet<Integer>>(dicerSeedsFinder.getModulesCopy()));
		dicerSeedsFinder.improveSolution(minModuleSize);
		algo2modulesAfterImprovement.put(name+"_"+improvmentAlgoNames[0], dicerSeedsFinder.getModulesCopy());
		algo2CMPs.put(name+"_"+improvmentAlgoNames[0], dicerSeedsFinder.getAreModulesFriends());
		algo2MMs_afterImprove.put(name+"_"+improvmentAlgoNames[0], dicerSeedsFinder.getMeta_modules());
	}
	
	public void updateHashForSeedAlgorithm(String name,List<HashSet<Integer>> seeds){
		algo2seed.put(name, seeds);
	}
	
	public static String getModuleStatistics(List<HashSet<Integer>> modules){
		float averageSize = 0;
		float totalCoverage = 0;
		float numModules = modules.size();
		for (Iterator<HashSet<Integer>> iterator = modules.iterator(); iterator.hasNext();) {
			HashSet<Integer> m = (HashSet<Integer>) iterator.next();
			averageSize+=m.size();
			totalCoverage+=m.size();
		}
		averageSize/=numModules;
		String out = numModules+"\t"+averageSize+"\t"+totalCoverage;
		return out;
	}
	
	public static String getModuleMapStatistics(boolean[][] map){
		float numEdges = 0;
		float meanDegree = 0;
		for (int i = 0; i < map.length; i++) {
			for (int j = 0; j < i; j++) {
				if (map[i][j]){
					numEdges++;
				}
			}
		}
		meanDegree = 2*numEdges/(float)map.length;
		String out = map.length+"\t"+numEdges+"\t"+meanDegree;
		return out;
	}
	
	public static String getModuleMapStatistics(short[][] map){
		float numEdges = 0;
		float meanDegree = 0;
		for (int i = 0; i < map.length; i++) {
			for (int j = 0; j < i; j++) {
				if (map[i][j]==1){
					numEdges++;
				}
			}
		}
		meanDegree = 2*numEdges/(float)map.length;
		String out = map.length+"\t"+numEdges+"\t"+meanDegree;
		return out;
	}
	
	public Hashtable<String, List<HashSet<Integer>>> getAlgo2modulesAfterImprovement() {
		return algo2modulesAfterImprovement;
	}
	
	public Hashtable<String, List<HashSet<Integer>>> getAlgo2seed() {
		return algo2seed;
	}
	
	public void printModuleStatistics(){
		for (Iterator<String> iterator = algo2modulesAfterImprovement.keySet().iterator(); iterator.hasNext();) {
			String name = (String) iterator.next();
			System.out.println(name+"\t"+getModuleStatistics(algo2modulesAfterImprovement.get(name)));
		}
	}
	
	public void printModuleSolutions(String outPrefix){
		for (Iterator<String> iterator = algo2modulesAfterImprovement.keySet().iterator(); iterator.hasNext();) {
			String name = (String) iterator.next();
			String outFileName = outPrefix+name+".txt";
			try {
				printModules("module", outFileName, algo2modulesAfterImprovement.get(name), nodeIds);
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}
	
	public void printSeedSolutions(String outPrefix){
		for (Iterator<String> iterator = algo2seed.keySet().iterator(); iterator.hasNext();) {
			String name = (String) iterator.next();
			String outFileName = outPrefix+name+"_seeds.txt";
			try {
				printModules("module", outFileName, algo2seed.get(name), nodeIds);
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}
	
	public void printMetaGraphs(String outPrefix){
		for (Iterator<String> iterator = algo2CMPs.keySet().iterator(); iterator.hasNext();) {
			String name = (String) iterator.next();
			String outFileName = outPrefix+name+"_module_map.sif";
			try {
				printMetaGraph("module", outFileName, algo2CMPs.get(name));
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}
	
	public void printSeedsStatistics(){
		for (Iterator<String> iterator = algo2seed.keySet().iterator(); iterator.hasNext();) {
			String name = (String) iterator.next();
			System.out.println(name+"\t"+getModuleStatistics(algo2seed.get(name)));
		}
	}
	
	public static void printMetaGraph(String prefix, String path, short[][] areModulesFriends) throws IOException{
		BufferedWriter writer = new BufferedWriter(new FileWriter(path));
		
		for (int i = 0; i < areModulesFriends.length; i++) {
			for (int j = 0; j < i; j++) {
				if (areModulesFriends[i][j]==1){
					String seed1Name = prefix + "_" + i;
					String seed2Name = prefix + "_" + j;
					writer.write(seed1Name + "\tEdge\t" + seed2Name + "\n");
				}
			}
		}
		writer.close();
	}
	
	public static void printModules(String prefix, String path, List<HashSet<Integer>> modules,ArrayList<String> geneNames) throws IOException {

		BufferedWriter writer = new BufferedWriter(new FileWriter(path));
		int counter = 0;
		for (Iterator<HashSet<Integer>> iterator = modules.iterator(); iterator
				.hasNext();) {
			HashSet<Integer> module = (HashSet<Integer>) iterator.next();
			
			for (Iterator<Integer> iterator2 = module.iterator(); iterator2
					.hasNext();) {
				Integer geneIndex = (Integer) iterator2.next();
				String geneName = geneNames.get(geneIndex);
				String seedName = prefix + "_" + counter;
				writer.write(geneName + "\t" + seedName + "\n");
			}
			counter++;
		}
		writer.close();
	}
	
	// getters
	public float[][] getG1() {
		return g1;
	}
	public float[][] getG2() {
		return g2;
	}
	
	//setters
	public void setMinModuleSize(int minModuleSize) {
		this.minModuleSize = minModuleSize;
	}
	
	public void setMinMSeedSize(int minMSeedSize) {
		this.minMSeedSize = minMSeedSize;
	}
	
	


}

