package randomized_graphs;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Random;
import multithreaded_algo_runs.MultithreadedComparisonRunner;

public class ContinCorruptedMMap extends DiscreteCorruptedModuleMap {

	public ContinCorruptedMMap(int n, int n_mm, double p) {
		super(n, n_mm, p);
	}
	
	public ContinCorruptedMMap(int n, int n_mm, double p, Random r) {
		super(n, n_mm, p,r);
	}
	
	@Override
	public void corrupt_graph(float[][] g1, double p) {
		for (int i = 0; i < g1.length; i++) {
			for (int j = 0; j < i; j++) {
				double x = rand.nextGaussian();
				x*=p;
				g1[i][j] += x;
				g1[j][i] = g1[i][j];
			}
		}
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		String outPath = "Z:\\davidama\\DICER_analysis\\dicer_2013\\weighted_n500_6Modules_HGTest_minsize10_maxsize20_10repeats.txt";
		try {
			BufferedWriter writer = new BufferedWriter(new FileWriter(outPath));
			String header = "seed algo\timprove algo\tnumMMs\tnumCorruptedEdges\tp\trand index\tJaccard";

			System.out.println(header);
			writer.write(header + "\n");
			
			int repeats=10;
			Random r = new Random();

			for (int numMMs = 5; numMMs <= 5; numMMs += 2) {
				for (double p = 0; p <= 2; p += 0.4) {
						
					MultithreadedComparisonRunner comps = null;
					ContinCorruptedMMap corrupted_graph = null;
					
					Hashtable<String, Double> algo2jaccard = new Hashtable<String, Double>();
					for (int k = 0; k < repeats; k++) {
						corrupted_graph = new ContinCorruptedMMap(
								500, numMMs, p,r);

						comps = new MultithreadedComparisonRunner(
								corrupted_graph.g1, corrupted_graph.g2,
								corrupted_graph.node_names, 0,0.005f);
						
						comps.doComparisons(true);
						comps.printModuleStatistics();
						
						for (Iterator<String> iterator = comps.getAlgo2modulesAfterImprovement().keySet()
								.iterator(); iterator.hasNext();) {
							String algoName = (String) iterator.next();
							List<HashSet<Integer>> modules = comps.getAlgo2modulesAfterImprovement()
									.get(algoName);
							double runningMaxJ = corrupted_graph
									.getRunningMaxModulesSimilarityMeasure(modules);
							if (algo2jaccard.containsKey(algoName)){
								double currVal = algo2jaccard.get(algoName);
								algo2jaccard.put(algoName, currVal + runningMaxJ/repeats);
							}
							else{
								algo2jaccard.put(algoName, runningMaxJ/repeats);
							}
						}
						
						for (Iterator<String> iterator = comps
								.getAlgo2seed().keySet()
								.iterator(); iterator.hasNext();) {
							String algoName = (String) iterator.next();
							List<HashSet<Integer>> modules = comps.getAlgo2seed().get(algoName);
							if (modules == null) {continue;	}
							double runningMaxJ = corrupted_graph
									.getRunningMaxModulesSimilarityMeasure(modules);
							if (algo2jaccard.containsKey(algoName)){
								double currVal = algo2jaccard.get(algoName);
								algo2jaccard.put(algoName, currVal + runningMaxJ/repeats);
							}
							else{
								algo2jaccard.put(algoName, runningMaxJ/repeats);
							}
						}
					}
					
					for (Iterator<String> iterator = comps
							.getAlgo2modulesAfterImprovement().keySet()
							.iterator(); iterator.hasNext();) {
						String algoName = (String) iterator.next();
						List<HashSet<Integer>> modules = comps
								.getAlgo2modulesAfterImprovement()
								.get(algoName);
						double randIndex = corrupted_graph
								.getGlobalSimilarityMeasure(modules);
						double runningMaxJ =algo2jaccard.get(algoName);
						
						String[] nameArr = algoName.split("_");

						String outLine2 = nameArr[0] + "\t" + nameArr[1] + "\t"
								+ numMMs + "\t"
								+ corrupted_graph.count_corrputed_edges + "\t"
								+ p + "\t" + randIndex + "\t" + runningMaxJ;

						System.out.println(outLine2);
						writer.write(outLine2 + "\n");
					}

					for (Iterator<String> iterator = comps.getAlgo2seed()
							.keySet().iterator(); iterator.hasNext();) {
						String algoName = (String) iterator.next();
						List<HashSet<Integer>> modules = comps.getAlgo2seed().get(algoName);
						if (modules == null) {continue;	}
						
						double randIndex = corrupted_graph
								.getGlobalSimilarityMeasure(modules);
						double runningMaxJ = algo2jaccard.get(algoName);

						algoName += "\tseeds";
						String outLine2 = algoName + "\t" + numMMs + "\t"
								+ corrupted_graph.count_corrputed_edges + "\t"
								+ p + "\t" + randIndex + "\t" + runningMaxJ;

						System.out.println(outLine2);
						writer.write(outLine2 + "\n");
					}
				}		
			}
			
			writer.close();

		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}


	}

}
