package randomized_graphs;

import graph_algorithms.MetaModule;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Random;

import multithreaded_algo_runs.MultithreadedComparisonRunner;

public class DiscreteCorruptedModuleMap extends
		MetaModuleDiscreteCorruptedGraph {

	ArrayList<HashSet<Integer>> modules = new ArrayList<HashSet<Integer>>();
	int[] gene2module;

	public DiscreteCorruptedModuleMap(int n, int n_mm, double p) {

		modules = new ArrayList<HashSet<Integer>>();
		num_nodes = n;
		num_meta_modules = n_mm;
		corruption_p = p;

		init_graphs();

		for (int i = 0; i < num_meta_modules; i++) {
			add_perfect_meta_module();
		}
		corrupt_graphs();

	}

	public DiscreteCorruptedModuleMap(int n, int n_mm, double p, Random r) {
		rand = r;
		modules = new ArrayList<HashSet<Integer>>();
		num_nodes = n;
		num_meta_modules = n_mm;
		corruption_p = p;

		init_graphs();

		for (int i = 0; i < num_meta_modules; i++) {
			add_perfect_meta_module();
		}
		corrupt_graphs();

	}

	@Override
	public void add_perfect_meta_module() {
		addMMRandomTreeStrucure();
	}

	public void addMMStarStrucure() {
		int n1 = get_random_module_size();
		int n2 = get_random_module_size();
		// select random disjoint sets
		HashSet<Integer> m1 = null;

		if (mms.size() > 0) {
			// choose a random meta-module and extend it
			int mmIndex = rand.nextInt(mms.size());
			int uOrV = rand.nextInt(2);
			m1 = mms.get(mmIndex).getU();
			if (uOrV == 1) {
				m1 = mms.get(mmIndex).getV();
			}
		} else {
			m1 = getRandomModule(n1);
			modules.add(m1);
			add_clique_to_graph(m1, g1);
		}

		HashSet<Integer> m2 = getRandomModule(n2);
		modules.add(m2);

		// add cliques in g1
		add_clique_to_graph(m2, g1);

		// add biclique to g2
		add_biclique_to_graph(m1, m2, g2);

		// add mm
		MetaModule mm = new MetaModule(m1, m2, data_container);
		mms.add(mm);
	}

	public void addMMRandomTreeStrucure() {

		int n1 = get_random_module_size();
		int n2 = get_random_module_size();
		// select random disjoint sets
		HashSet<Integer> m1 = null;

		if (modules.size() > 0) {
			// choose a random meta-module and extend it
			int mmIndex = rand.nextInt(modules.size());
			m1 = modules.get(mmIndex);
		} else {
			m1 = getRandomModule(n1);
			modules.add(m1);
			add_clique_to_graph(m1, g1);
		}

		HashSet<Integer> m2 = getRandomModule(n2);
		modules.add(m2);

		// add cliques in g1
		add_clique_to_graph(m2, g1);

		// add biclique to g2
		add_biclique_to_graph(m1, m2, g2);

		// add mm
		MetaModule mm = new MetaModule(m1, m2, data_container);
		mms.add(mm);
	}

	public void printMapStructure() {
		for (Iterator<MetaModule> iterator = mms.iterator(); iterator.hasNext();) {
			MetaModule mm = (MetaModule) iterator.next();
			HashSet<Integer> a = mm.getU();
			HashSet<Integer> b = mm.getV();
			int a_i = modules.indexOf(a);
			int b_i = modules.indexOf(b);
			System.out.println(a_i + "\t" + b_i);
		}
	}

	public int[] calcGene2Module(List<HashSet<Integer>> modules) {
		int[] gene2module = new int[g1.length];
		Arrays.fill(gene2module, -1);
		int c = 0;
		for (Iterator<HashSet<Integer>> iterator = modules.iterator(); iterator
				.hasNext();) {
			HashSet<Integer> module = (HashSet<Integer>) iterator.next();
			for (Iterator<Integer> iterator2 = module.iterator(); iterator2
					.hasNext();) {
				Integer inM = (Integer) iterator2.next();
				gene2module[inM] = c;
			}
			c++;
		}

		return gene2module;
	}

	@Override
	public void corrupt_graphs() {

		for (int i = 0; i < 1; i++) {
			HashSet<Integer> randCliqueToG1 = getRandomModule(15);
			add_clique_to_graph(randCliqueToG1, g1);

			HashSet<Integer> randCliqueToG2 = getRandomModule(15);
			add_clique_to_graph(randCliqueToG2, g2);

			HashSet<Integer> first_randBiCliqueToG1 = getRandomModule(15);
			HashSet<Integer> second_randBiCliqueToG1 = getRandomModule(15);
			add_biclique_to_graph(first_randBiCliqueToG1,
					second_randBiCliqueToG1, g1);
			add_biclique_to_graph(getRandomModule(15), getRandomModule(15), g2);
		}

		corrupt_graph(g1, corruption_p);
		corrupt_graph(g2, corruption_p);
	}

	public double getGlobalSimilarityMeasure(List<HashSet<Integer>> otherModules) {

		int[] realSol = calcGene2Module(modules);
		int[] otherSol = calcGene2Module(otherModules);

		// calculate the Rand index
		float numCorrect = 0;
		float n = g1.length * (g1.length - 1) / 2;
		for (int i = 0; i < otherSol.length; i++) {
			for (int j = 0; j < i; j++) {
				if (realSol[i] == realSol[j] && otherSol[i] == otherSol[j]) {
					numCorrect++;
				}
				if (realSol[i] != realSol[j] && otherSol[i] != otherSol[j]) {
					numCorrect++;
				}
			}
		}

		return numCorrect / n;
	}

	public double getRunningMaxModulesSimilarityMeasure(
			List<HashSet<Integer>> otherSol) {
		double rmax1avg = get_a_vs_b_module_running_max_average_similarity(
				otherSol, modules);
		double rmax2avg = get_a_vs_b_module_running_max_average_similarity(
				modules, otherSol);
		double total_similarity = (rmax1avg * otherSol.size() + rmax2avg
				* modules.size())
				/ (float) (modules.size() + otherSol.size());
		return total_similarity;
	}

	public static void main(String[] args) {

		String outPath = "Z:\\davidama\\DICER_analysis\\dicer_2013\\test.txt";
		try {
			BufferedWriter writer = new BufferedWriter(new FileWriter(outPath));
			String header = "seed algo\timprove algo\tnumMMs\tnumCorruptedEdges\tp\trand index\tMeanJaccard";

			System.out.println(header);
			writer.write(header + "\n");

			int repeats = 1;
			Random r = new Random();

			for (int numMMs = 5; numMMs <= 5; numMMs += 1) {
				for (double p = 0; p <= 0.25; p += 0.03) {
					MultithreadedComparisonRunner comps = null;
					DiscreteCorruptedModuleMap corrupted_graph = null;
					Hashtable<String, Double> algo2jaccard = new Hashtable<String, Double>();
					for (int k = 0; k < repeats; k++) {
						corrupted_graph = new DiscreteCorruptedModuleMap(
								500, numMMs, p,r);

						comps = new MultithreadedComparisonRunner(
								corrupted_graph.g1, corrupted_graph.g2,
								corrupted_graph.node_names, 0,0.005f);
						
						comps.doComparisons(true);
						comps.printModuleStatistics();
						
						for (Iterator<String> iterator = comps.getAlgo2modulesAfterImprovement().keySet()
								.iterator(); iterator.hasNext();) {
							String algoName = (String) iterator.next();
							List<HashSet<Integer>> modules = comps.getAlgo2modulesAfterImprovement()
									.get(algoName);
							double runningMaxJ = corrupted_graph
									.getRunningMaxModulesSimilarityMeasure(modules);
							if (algo2jaccard.containsKey(algoName)){
								double currVal = algo2jaccard.get(algoName);
								algo2jaccard.put(algoName, currVal + runningMaxJ/repeats);
							}
							else{
								algo2jaccard.put(algoName, runningMaxJ/repeats);
							}
						}
						
						for (Iterator<String> iterator = comps
								.getAlgo2seed().keySet()
								.iterator(); iterator.hasNext();) {
							String algoName = (String) iterator.next();
							List<HashSet<Integer>> modules = comps.getAlgo2seed().get(algoName);
							if (modules == null) {continue;	}
							double runningMaxJ = corrupted_graph
									.getRunningMaxModulesSimilarityMeasure(modules);
							if (algo2jaccard.containsKey(algoName)){
								double currVal = algo2jaccard.get(algoName);
								algo2jaccard.put(algoName, currVal + runningMaxJ/repeats);
							}
							else{
								algo2jaccard.put(algoName, runningMaxJ/repeats);
							}
						}
					}
					
					for (Iterator<String> iterator = comps
							.getAlgo2modulesAfterImprovement().keySet()
							.iterator(); iterator.hasNext();) {
						String algoName = (String) iterator.next();
						List<HashSet<Integer>> modules = comps
								.getAlgo2modulesAfterImprovement()
								.get(algoName);
						double randIndex = corrupted_graph
								.getGlobalSimilarityMeasure(modules);
						double runningMaxJ =algo2jaccard.get(algoName);
						
						String[] nameArr = algoName.split("_");

						String outLine2 = nameArr[0] + "\t" + nameArr[1] + "\t"
								+ numMMs + "\t"
								+ corrupted_graph.count_corrputed_edges + "\t"
								+ p + "\t" + randIndex + "\t" + runningMaxJ;

						System.out.println(outLine2);
						writer.write(outLine2 + "\n");
					}

					for (Iterator<String> iterator = comps.getAlgo2seed()
							.keySet().iterator(); iterator.hasNext();) {
						String algoName = (String) iterator.next();
						List<HashSet<Integer>> modules = comps.getAlgo2seed().get(algoName);
						if (modules == null) {continue;	}
						
						double randIndex = corrupted_graph
								.getGlobalSimilarityMeasure(modules);
						double runningMaxJ = algo2jaccard.get(algoName);

						algoName += "\tseeds";
						String outLine2 = algoName + "\t" + numMMs + "\t"
								+ corrupted_graph.count_corrputed_edges + "\t"
								+ p + "\t" + randIndex + "\t" + runningMaxJ;

						System.out.println(outLine2);
						writer.write(outLine2 + "\n");
					}
				}
			}

			writer.close();

		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

}
