package randomized_graphs;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;

import graph_algorithms.BiModuleFinderUsingBicliqueAlgo;
import graph_algorithms.MetaModuleFinder;

public class MMContinuousCorruptedGraph extends MetaModuleDiscreteCorruptedGraph {
	
	/* Members */
	double sd = 1;
	

	public MMContinuousCorruptedGraph(int n, int n_mm,double s) {
		super(n, n_mm);
		sd=s;
		corrupt_graph(g1, sd);
		corrupt_graph(g2, sd);
	}
	
	@Override
	public void corrupt_graph(float[][] g1, double p) {
		for (int i = 0; i < g1.length; i++) {
			for (int j = 0; j < i; j++) {
				double x = rand.nextGaussian();
				x*=sd;
				g1[i][j] += x;
				g1[j][i] = g1[i][j];
			}
		}
	}
	
	@Override
	public void corrupt_graphs() {
//		add_random_cliques_and_bicliques();
		corrupt_graph(g1, corruption_p);
		corrupt_graph(g2, corruption_p);
	}
	
	public static void main(String[] args) {
		String outPath = "Z:\\davidama\\DICER_analysis\\dicer_2013\\mbc_dicer_contin_graph_withCliquesAndBicliques_3.txt";
		try {
			BufferedWriter writer = new BufferedWriter(new FileWriter(outPath));
			String header = "numMMs\tSD\tscore\trunning time";
			
			System.out.println(header);
			writer.write(header+"\n");

			for (int numMMs = 1; numMMs <= 10; numMMs += 9) {
				for (double sd = 0; sd <= 2; sd += 0.1) {
					MMContinuousCorruptedGraph corrupted_graph = new MMContinuousCorruptedGraph(
							1000, numMMs, sd);

					long startTime = System.currentTimeMillis();
					BiModuleFinderUsingBicliqueAlgo mmFinder = new BiModuleFinderUsingBicliqueAlgo(
							corrupted_graph.g1, corrupted_graph.g2,
							corrupted_graph.node_names);
					mmFinder.addNumToTmpFiles(5);
					mmFinder.getFinalSolution(5);
					long endTime = System.currentTimeMillis();

					long diff = endTime - startTime;
					diff /= 1000;

					double s = corrupted_graph
							.getRunningMaxSimilarityMeasure(mmFinder.getMeta_modules());

					String outLine = numMMs+"\t"+ sd + "\t" + s + "\t" + diff;

					System.out.println(outLine);
					writer.write(outLine + "\n");
				}
			}
			
			writer.close();

		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}
	
}
