package randomized_graphs;

import graph_algorithms.MetaModule;
import graph_algorithms.MetaModuleFinder;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Random;
import java.util.Set;

public class MetaModuleCorruptedGraph {

	/*MEMBERS*/
	int num_nodes = 1000;
	int num_meta_modules = 2;
	int module_size_min = 10;
	int module_size_max = 20;
	double corruption_p = 0.01;
	Random rand = new Random(1);
	ArrayList<MetaModule> mms;
	boolean[] is_in_mms;
	float edge_threshold = 1;
	float non_edge_threshold = -1;
	
	int count_corrputed_edges = 0;
	int count_corrputed_edges_within_mms = 0;
	
	ArrayList<String> node_names;
	float[][] g1;
	float[][] g2;
	private MetaModuleFinder data_container;
	
	public MetaModuleCorruptedGraph(int n,int n_mm,double p) {
		
		num_nodes=n;
		num_meta_modules = n_mm;
		corruption_p = p;
		
		init_graphs();
		for (int i = 0; i < num_meta_modules; i++) {
			add_perfect_meta_module();
		}
		corrupt_graphs();
	}
	
	public void init_graphs(){
		g1 = new float[num_nodes][num_nodes];
		g2 = new float[num_nodes][num_nodes];
		for (int i = 0; i < g1.length; i++) {
			Arrays.fill(g1[i], non_edge_threshold);
			Arrays.fill(g2[i], non_edge_threshold);
		}
		node_names = new ArrayList<String>(num_nodes);
		for (int i = 0; i < g1.length; i++) {
			node_names.add(""+i);
		}
		
		mms = new ArrayList<MetaModule>();
		is_in_mms = new boolean[num_nodes];
		Arrays.fill(is_in_mms, false);
		
		data_container = new MetaModuleFinder(g1, g2, node_names);
	}
	
	public void add_perfect_meta_module(){
		int n1 = get_random_module_size();
		int n2 = get_random_module_size();
		
		// select random disjoint sets
		HashSet<Integer> m1 = new HashSet<Integer>();
		HashSet<Integer> m2 = new HashSet<Integer>();
		while (m1.size()<n1) {
			int next_candidate = rand.nextInt(num_nodes);
			if (!is_in_mms[next_candidate]){
				m1.add(next_candidate);
				is_in_mms[next_candidate]=true;
			}
				
		}
		
		while (m2.size()<n2) {
			int next_candidate = rand.nextInt(num_nodes);
			if (! is_in_mms[next_candidate] && !m1.contains(next_candidate)){
				m2.add(next_candidate);
				is_in_mms[next_candidate]=true;
			}
		}
		
		// add cliques in g1
		add_clique_to_g1(m2);
		add_clique_to_g1(m1);
		
		// add biclique to g2
		add_biclique_to_g2(m1, m2);
		
		// add mm
		MetaModule mm = new MetaModule(m1, m2, data_container);
		mms.add(mm);
	}
	
	/**
	 * Go over the graphs, flip an edge with probability
	 * corruption_p
	 */
	public void corrupt_graphs(){
		corrupt_graph(g1,corruption_p);
		corrupt_graph(g2,corruption_p);
	}

	public void corrupt_graph(float[][] g1, double p) {
		for (int i = 0; i < g1.length; i++) {
			for (int j = 0; j < i; j++) {
				double x = rand.nextDouble();
				if (x<p){
					count_corrputed_edges++;
					
					// THIS IS WORNG!
					if (is_in_mms[i] && is_in_mms[j]){
						count_corrputed_edges_within_mms++;
					}
					
					if (g1[i][j]==edge_threshold){
						g1[i][j]=non_edge_threshold;
						g1[j][i]=non_edge_threshold;
						continue;
					}
					
					if (g1[i][j]==non_edge_threshold){
						g1[i][j]=edge_threshold;
						g1[j][i]=edge_threshold;
						continue;
					}
				}
			}
		}
	}

	public void add_clique_to_g1(HashSet<Integer> m2) {
		for (Iterator<Integer> iterator = m2.iterator(); iterator.hasNext();) {
			Integer in_module_1 = (Integer) iterator.next();
			for (Iterator<Integer> iterator2 = m2.iterator(); iterator2.hasNext();) {
				Integer in_module_2 = (Integer) iterator2.next();
				g1[in_module_1][in_module_2]=edge_threshold;
				g1[in_module_2][in_module_1]=edge_threshold;
			}
		}
	}
	
	public void add_biclique_to_g2(HashSet<Integer> m1,HashSet<Integer> m2) {
		for (Iterator<Integer> iterator = m1.iterator(); iterator.hasNext();) {
			Integer in_module_1 = (Integer) iterator.next();
			for (Iterator<Integer> iterator2 = m2.iterator(); iterator2.hasNext();) {
				Integer in_module_2 = (Integer) iterator2.next();
				g2[in_module_1][in_module_2]=edge_threshold;
				g2[in_module_2][in_module_1]=edge_threshold;
			}
		}
	}

	public int get_random_module_size() {
		int module1_size = rand.nextInt(module_size_min);
		module1_size+=(module_size_max-module_size_min);
		return module1_size;
	}
	
	public double getRunningMaxSimilarityMeasure(List<MetaModule> otherSol){
		double rmax1avg = get_a_vs_b_running_max_average_similarity(otherSol,mms);
		double rmax2avg = get_a_vs_b_running_max_average_similarity(mms,otherSol);
		double total_similarity = (rmax1avg+rmax2avg)/2.0;
		return total_similarity;
	}

	public double get_a_vs_b_running_max_average_similarity(
			List<MetaModule> otherSol,List<MetaModule> mms) {
		double rmax1avg=0;
		for (Iterator<MetaModule> iterator = otherSol.iterator(); iterator.hasNext();) {
			MetaModule inOther = (MetaModule) iterator.next();
			double currMax = 0;
			for (Iterator<MetaModule> iterator2 = mms.iterator(); iterator2.hasNext();) {
				MetaModule in_mm = (MetaModule) iterator2.next();
				double currSim = getMetaModulesSimilarity(inOther, in_mm);
				if (currSim>currMax){
					currMax=currSim;
				}
			}
			rmax1avg += currMax;
		}
		rmax1avg /= (double)otherSol.size();
		return rmax1avg;
	}
	
	public static double getMetaModulesSimilarity(MetaModule a, MetaModule b){
		HashSet<Integer> a_u = a.getU();
		HashSet<Integer> a_v = a.getV();
		
		HashSet<Integer> b_u = b.getU();
		HashSet<Integer> b_v = b.getV();
		
		double u_vs_u_score =0.5*(jaccardCoef(a_u, b_u)+jaccardCoef(a_v, b_v));
		double u_vs_v_score = 0.5*(jaccardCoef(a_v, b_u)+jaccardCoef(a_u, b_v));
		
		return Math.max(u_vs_u_score, u_vs_v_score);
	}
	
	public static float jaccardCoef(Set<Integer> a, Set<Integer> b) {
		float intersect = 0;
		for (Iterator<Integer> iterator = b.iterator(); iterator.hasNext();) {
			Integer inB = (Integer) iterator.next();
			if (a.contains(inB)) {
				intersect++;
			}
		}

		float union = a.size() + b.size() - intersect;

		return intersect / union;
	}
	
	public static void main(String[] args) {
		
		for (double p = 0; p < 1; p+=0.01) {
			MetaModuleCorruptedGraph corrupted_graph = new MetaModuleCorruptedGraph(1000,10,p);
			
			MetaModuleFinder mmFinder = new MetaModuleFinder(corrupted_graph.g1, corrupted_graph.g2, corrupted_graph.node_names);
			mmFinder.getFinalSolution(10, 5);
			
			double s = corrupted_graph.getRunningMaxSimilarityMeasure(mmFinder.meta_modules);
			System.out.println(corrupted_graph.count_corrputed_edges+"\t"+p+"\t"+s);
		}
		
	}
	
}
