package randomized_graphs;

import graph_algorithms.MetaModule;
import graph_algorithms.MetaModuleFinder;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Random;
import java.util.Set;

import main_scripts.AlgorithmComparisonRunner;

public class MetaModuleDiscreteCorruptedGraph {

	/* MEMBERS */
	int num_nodes = 1000;
	int num_meta_modules = 2;
	int module_size_min = 10;
	int module_size_max = 20;
	double corruption_p = 0.01;
	Random rand = new Random(1);
	ArrayList<MetaModule> mms;
	ArrayList<HashSet<Integer>> modules;
	boolean[] is_in_mms;
	float edge_threshold = 1;
	float non_edge_threshold = -1;

	int count_corrputed_edges = 0;
	int count_corrputed_edges_within_mms = 0;

	ArrayList<String> node_names;
	float[][] g1;
	float[][] g2;
	protected MetaModuleFinder data_container;

	public MetaModuleDiscreteCorruptedGraph(int n, int n_mm, double p) {

		num_nodes = n;
		num_meta_modules = n_mm;
		corruption_p = p;

		init_graphs();
		for (int i = 0; i < num_meta_modules; i++) {
			add_perfect_meta_module();
		}
		corrupt_graphs();
	}

	public MetaModuleDiscreteCorruptedGraph() {
		// TODO Auto-generated constructor stub
	}

	public MetaModuleDiscreteCorruptedGraph(int n, int n_mm) {

		num_nodes = n;
		num_meta_modules = n_mm;

		init_graphs();
		for (int i = 0; i < num_meta_modules; i++) {
			add_perfect_meta_module();
		}
	}

	public void init_graphs() {
		g1 = new float[num_nodes][num_nodes];
		g2 = new float[num_nodes][num_nodes];
		for (int i = 0; i < g1.length; i++) {
			Arrays.fill(g1[i], non_edge_threshold);
			Arrays.fill(g2[i], non_edge_threshold);
		}
		node_names = new ArrayList<String>(num_nodes);
		for (int i = 0; i < g1.length; i++) {
			node_names.add("" + i);
		}

		mms = new ArrayList<MetaModule>();
		is_in_mms = new boolean[num_nodes];
		Arrays.fill(is_in_mms, false);

		data_container = new MetaModuleFinder(g1, g2, node_names);
	}

	public void add_random_cliques_and_bicliques() {
		add_clique_to_graph(getRandomModule(15), g1);
		add_clique_to_graph(getRandomModule(15), g2);
		add_biclique_to_graph(getRandomModule(15), getRandomModule(15), g1);
		add_biclique_to_graph(getRandomModule(15), getRandomModule(15), g2);
	}

	public void add_perfect_meta_module() {
		int n1 = get_random_module_size();
		int n2 = get_random_module_size();

		// select random disjoint sets
		HashSet<Integer> m1 = getRandomModule(n1);
		HashSet<Integer> m2 = getRandomModule(n2);

		// add cliques in g1
		add_clique_to_graph(m2, g1);
		add_clique_to_graph(m1, g1);

		// add biclique to g2
		add_biclique_to_graph(m1, m2, g2);

		// add mm
		MetaModule mm = new MetaModule(m1, m2, data_container);
		mms.add(mm);
	}

	public HashSet<Integer> getRandomModule(int n1) {
//		System.out.println(n1);
		ArrayList<Integer> not_in_mms = new ArrayList<Integer>(g1.length);
		for (int i = 0; i < is_in_mms.length; i++) {
			if (!is_in_mms[i]){
				not_in_mms.add(i);
			}
		}
		Collections.shuffle(not_in_mms, rand);
		
		HashSet<Integer> m1 = new HashSet<Integer>();
		while (m1.size() < n1) {
			int next_candidate = not_in_mms.get(m1.size());
			if (!is_in_mms[next_candidate]) {
				m1.add(next_candidate);
				is_in_mms[next_candidate] = true;
			}

		}
		return m1;
	}

	/**
	 * Go over the graphs, flip an edge with probability corruption_p
	 */
	public void corrupt_graphs() {
		// for (int i = 0; i < 5; i++) {
		// add_random_cliques_and_bicliques();
		// }
		corrupt_graph(g1, corruption_p);
		corrupt_graph(g2, corruption_p);
	}

	public void corrupt_graph(float[][] g1, double p) {
		for (int i = 0; i < g1.length; i++) {
			for (int j = 0; j < i; j++) {
				double x = rand.nextDouble();
				if (x < p) {
					count_corrputed_edges++;

					// THIS IS WORNG!
					if (is_in_mms[i] && is_in_mms[j]) {
						count_corrputed_edges_within_mms++;
					}

					if (g1[i][j] == edge_threshold) {
						g1[i][j] = non_edge_threshold;
						g1[j][i] = non_edge_threshold;
						continue;
					}

					if (g1[i][j] == non_edge_threshold) {
						g1[i][j] = edge_threshold;
						g1[j][i] = edge_threshold;
						continue;
					}
				}
			}
		}
	}

	public void add_clique_to_graph(HashSet<Integer> m2, float[][] g1) {
		for (Iterator<Integer> iterator = m2.iterator(); iterator.hasNext();) {
			Integer in_module_1 = (Integer) iterator.next();
			for (Iterator<Integer> iterator2 = m2.iterator(); iterator2
					.hasNext();) {
				Integer in_module_2 = (Integer) iterator2.next();
				if (in_module_1!=in_module_2){
					g1[in_module_1][in_module_2] = edge_threshold;
					g1[in_module_2][in_module_1] = edge_threshold;
				}
			}
		}
	}

	public void add_biclique_to_graph(HashSet<Integer> m1, HashSet<Integer> m2,
			float[][] g2) {
		for (Iterator<Integer> iterator = m1.iterator(); iterator.hasNext();) {
			Integer in_module_1 = (Integer) iterator.next();
			for (Iterator<Integer> iterator2 = m2.iterator(); iterator2
					.hasNext();) {
				Integer in_module_2 = (Integer) iterator2.next();
				g2[in_module_1][in_module_2] = edge_threshold;
				g2[in_module_2][in_module_1] = edge_threshold;
			}
		}
	}

	public int get_random_module_size() {
		int module1_size = rand.nextInt(module_size_min);
		module1_size += (module_size_max - module_size_min);
		return module1_size;
	}

	public double getRunningMaxSimilarityMeasure(List<MetaModule> otherSol) {
		double rmax1avg = get_a_vs_b_running_max_average_similarity(otherSol,
				mms);
		double rmax2avg = get_a_vs_b_running_max_average_similarity(mms,
				otherSol);
		double total_similarity = (rmax1avg + rmax2avg) / 2.0;
		return total_similarity;
	}
	
//	public void getModules(){
//		modules = new ArrayList<HashSet<Integer>>();
//		for (Iterator<MetaModule> iterator = mms.iterator(); iterator.hasNext();) {
//			MetaModule mm = (MetaModule) iterator.next();
//			modules.add(mm.getU());
//			modules.add(mm.getV());
//		}
//	}
//	
//	public double getRunningMaxModulesSimilarityMeasure(
//			List<HashSet<Integer>> otherSol) {
//		getModules();
//		double rmax1avg = get_a_vs_b_module_running_max_average_similarity(
//				otherSol, modules);
//		double rmax2avg = get_a_vs_b_module_running_max_average_similarity(
//				modules, otherSol);
//		double total_similarity = (rmax1avg * otherSol.size() + rmax2avg
//				* modules.size())
//				/ (float) (modules.size() + otherSol.size());
//		return total_similarity;
//	}

	public double get_a_vs_b_running_max_average_similarity(
			List<MetaModule> otherSol, List<MetaModule> mms) {
		double rmax1avg = 0;
		for (Iterator<MetaModule> iterator = otherSol.iterator(); iterator
				.hasNext();) {
			MetaModule inOther = (MetaModule) iterator.next();
			double currMax = 0;
			for (Iterator<MetaModule> iterator2 = mms.iterator(); iterator2
					.hasNext();) {
				MetaModule in_mm = (MetaModule) iterator2.next();
				double currSim = getMetaModulesSimilarity(inOther, in_mm);
				if (currSim > currMax) {
					currMax = currSim;
				}
			}
			rmax1avg += currMax;
		}
		rmax1avg /= (double) otherSol.size();
		return rmax1avg;
	}

	public double get_a_vs_b_module_running_max_average_similarity(
			List<HashSet<Integer>> otherSol, List<HashSet<Integer>> mms) {
		double rmax1avg = 0;
		for (Iterator<HashSet<Integer>> iterator = otherSol.iterator(); iterator
				.hasNext();) {
			HashSet<Integer> inOther = (HashSet<Integer>) iterator.next();
			double currMax = 0;
			for (Iterator<HashSet<Integer>> iterator2 = mms.iterator(); iterator2
					.hasNext();) {
				HashSet<Integer> in_mm = (HashSet<Integer>) iterator2.next();
				double currSim = jaccardCoef(inOther, in_mm);
				if (currSim > currMax) {
					currMax = currSim;
				}
			}
			rmax1avg += currMax;
		}
		if (otherSol.size() > 0)
			rmax1avg /= (double) otherSol.size();
		return rmax1avg;
	}

	public static double getMetaModulesSimilarity(MetaModule a, MetaModule b) {
		HashSet<Integer> a_u = a.getU();
		HashSet<Integer> a_v = a.getV();

		HashSet<Integer> b_u = b.getU();
		HashSet<Integer> b_v = b.getV();

		double u_vs_u_score = 0.5 * (jaccardCoef(a_u, b_u) + jaccardCoef(a_v,
				b_v));
		double u_vs_v_score = 0.5 * (jaccardCoef(a_v, b_u) + jaccardCoef(a_u,
				b_v));

		return Math.max(u_vs_u_score, u_vs_v_score);
	}

	public static float jaccardCoef(Set<Integer> a, Set<Integer> b) {
		float intersect = 0;
		for (Iterator<Integer> iterator = b.iterator(); iterator.hasNext();) {
			Integer inB = (Integer) iterator.next();
			if (a.contains(inB)) {
				intersect++;
			}
		}

		float union = a.size() + b.size() - intersect;

		return intersect / union;
	}

	public static void main(String[] args) {

		String outPath = "Z:\\davidama\\DICER_analysis\\dicer_2013\\bimodule_discrete_n500_5MM_HGtest.txt";
		try {
			BufferedWriter writer = new BufferedWriter(new FileWriter(outPath));
			String header = "seed algo\timprove algo\tnumMMs\tnumCorruptedEdges\tp\trand index\tJaccard";

			System.out.println(header);
			writer.write(header + "\n");

			for (int numMMs = 5; numMMs <= 5; numMMs += 1) {
				for (double p = 0; p <= 0.25; p += 0.02) {

					MetaModuleDiscreteCorruptedGraph corrupted_graph = new MetaModuleDiscreteCorruptedGraph(
							500, numMMs, p);

					AlgorithmComparisonRunner comps = new AlgorithmComparisonRunner(
							corrupted_graph.g1, corrupted_graph.g2,
							corrupted_graph.node_names,0);

					comps.doComparisons(true);
					for (Iterator<String> iterator = comps
							.getAlgo2modulesAfterImprovement().keySet()
							.iterator(); iterator.hasNext();) {
						String algoName = (String) iterator.next();
						List<HashSet<Integer>> modules = comps
								.getAlgo2modulesAfterImprovement()
								.get(algoName);
						
//						double runningMaxJ = corrupted_graph
//								.getRunningMaxModulesSimilarityMeasure(modules);
//
//						String[] nameArr = algoName.split("_");
//
//						String outLine2 = nameArr[0] + "\t" + nameArr[1] + "\t"
//								+ numMMs + "\t"
//								+ corrupted_graph.count_corrputed_edges + "\t"
//								+ p + "\t" + runningMaxJ;
//
//						System.out.println(outLine2);
//						writer.write(outLine2 + "\n");
					}

					for (Iterator<String> iterator = comps.getAlgo2seed()
							.keySet().iterator(); iterator.hasNext();) {
						String algoName = (String) iterator.next();
						List<HashSet<Integer>> modules = comps.getAlgo2seed()
								.get(algoName);
						if (modules == null) {
							continue;
						}
						
//						double runningMaxJ = corrupted_graph
//								.getRunningMaxModulesSimilarityMeasure(modules);
//
//						algoName += "\tseeds";
//						String outLine2 = algoName + "\t" + numMMs + "\t"
//								+ corrupted_graph.count_corrputed_edges + "\t"
//								+ p + "\t" + runningMaxJ;
//
//						System.out.println(outLine2);
//						writer.write(outLine2 + "\n");
					}
				}
			}

			writer.close();

		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

}
