package scripts;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;

import math_utils.ExpressionMatrix;
import math_utils.staticMathHelperMethods;

/**
 * 
 * This script receives as input an expression matrix and separation
 * of the conditions to classes and writes a new matrix in which every 
 * class is standardized separately.
 * Standardization is of the genes.
 * Expression matrix format: tab delimited file.
 * Rows are genes\probes, columns are samples. First row\column is the row\column name.
 *
 */

public class ByClassesStandardizer {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		if (args.length != 2){
			System.out.println("usage:<geData><classes file>");
			System.exit(1);
		}
		
		ExpressionMatrix data = new ExpressionMatrix(args[0]);
		try {
			BufferedReader reader = new BufferedReader(new FileReader(args[1]));
			ArrayList<Integer> classes = new ArrayList<Integer>();
			String currLine = "";
			while ((currLine = reader.readLine())!=null){
				currLine.trim();
				String[] line = currLine.split("\t");
				classes.add(Integer.parseInt(line[0]));
				classes.add(Integer.parseInt(line[1]));
			}
			
			int[] borders = new int[classes.size()];
			for (int i = 0; i < borders.length; i++) {
				borders[i] = classes.get(i);
			}
			staticMathHelperMethods.standardizeByGenes(data.matrix, borders);
			data.printMatrix();
			
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

}
