package scripts;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;

import main_scripts.TscoresCalculator;
import math_utils.ExpressionMatrix;

/**
 * This script is for creating sif files for analysis in cytoscape.
 * The input is an expression matrix, a list of genes of interest, 
 * output path name, minimum correlation value, classes file, and the class of interest index.
 * 
 * The output is as follows: create a sif file. A node is a gene from the input list. An edge is 
 * a gene pair with correlation > min_value. The correlation is calculated in 
 * the window of the selected class. These windows (i.e., sample to class
 * mapping) is given in the classes file.
 *
 */

public class GenerateWeightedGraphForCytoscape {

	/**
	 * @param args
	 */
	public static void main(String[] args) {

		if (args.length != 6){
			System.out.println("usage: <GE matrix> <a list of wanted genes><output name><min value><classes file><wanted class>");
			System.exit(1);
		}
		
		ExpressionMatrix mat = new ExpressionMatrix(args[0]);
		ArrayList<String> genes = new ArrayList<String>();
		
		try {
			BufferedReader reader = new BufferedReader(new FileReader(args[1]));
			String myLine = "";
			while ((myLine = reader.readLine())!=null){
				String id = (myLine.split("\t"))[0];
				if (!genes.contains(id)){
					genes.add(id);
				}
			}
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		LinkedList<Integer> wantedIndices = new LinkedList<Integer>();
		for (Iterator<String> iterator = genes.iterator(); iterator.hasNext();) {
			String geneId = (String) iterator.next();
			wantedIndices.add(mat.getGenesID().indexOf(geneId));
			
		}
		Collections.sort(wantedIndices);
		
		ExpressionMatrix subMat = mat.getSubMatrixByRowsIndices(wantedIndices);
		String name = args[2];
		TscoresCalculator alg = new TscoresCalculator(subMat, args[4], "pearson");
		alg.claculateSimilarities();
		int wantedClass = Integer.parseInt(args[5]);
		
		double minValue = Double.parseDouble(args[3]);
		
		try {
			BufferedWriter writer = new BufferedWriter(new FileWriter(name+".sif"));
			for (int i = 0; i < genes.size(); i++) {
				int a = subMat.genesID.indexOf(genes.get(i));
				for (int j = 0; j < i; j++) {
					int b = subMat.genesID.indexOf(genes.get(j));
					float currentCorrelation = alg.class2Correlations.get(wantedClass)[a][b];
					if (currentCorrelation > minValue){
						writer.write(genes.get(i)+" Edge " + genes.get(j)+"\n");
					}
				}
			}
			writer.flush();
			writer.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
//		try {
//			BufferedWriter writer = new BufferedWriter(new FileWriter(name+".txt"));
//			writer.write("PearsonCorrelations\n");
//			for (int i = 0; i < subMat.getGenesID().size(); i++) {
//				for (int j = 0; j < i; j++) {
//					float currentCorrelation = alg.class2Correlations.get(wantedClass)[i][j];
//					if (currentCorrelation > minValue){
//						writer.write(genes.get(i)+" (Edge) " + genes.get(j)+ " = " + currentCorrelation + "\n");
//					}
//				}
//			}
//			writer.flush();
//			writer.close();
//		} catch (IOException e) {
//			// TODO Auto-generated catch block
//			e.printStackTrace();
//		}
	}

}
